/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.ant.taskdefs;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import org.jboss.profiler.ant.ExecutionInfoSummary;
import org.jboss.profiler.ant.ExecutionInfoSummaryComparator;
import org.jboss.profiler.ant.GraphGenerator;
import org.jboss.profiler.engine.StandaloneProcess;
import org.jboss.profiler.graphmodel.GraphCallings;
import org.jboss.profiler.model.JBPClass;
import org.jboss.profiler.model.JBPMemoryCount;
import org.jboss.profiler.model.JBPMethod;
import org.jboss.profiler.model.JBPMethodCount;
import org.jboss.profiler.util.XmlGenerationUtil;

/**
 * @author Clebert Suconic
 */
public class AntProfiler extends Task {

    ArrayList classesToAnalyze = new ArrayList();

    ArrayList filesets = new ArrayList();

    ArrayList classes = new ArrayList();

    HashMap methodsSummary = new HashMap();

    String outputDir;

    StandaloneProcess standalone = null;

    public String getOutputDir() {
        return outputDir;
    }

    public void setOutputDir(String outputDir) {
        this.outputDir = outputDir;
    }

    /** The places where the logs are generated */
    String logDirectory;

    public void addTestset(FileSet fileset) {
        filesets.add(fileset);
    }

    public void addClassName(Value className) {
        classes.add(className);
    }

    /**
     * @return Returns the logDirectory.
     */
    public String getLogDirectory() {
        return logDirectory;
    }

    /**
     * @param logDirectory
     *            The logDirectory to set.
     */
    public void setLogDirectory(String logDirectory) {
        this.logDirectory = logDirectory;
    }

    /**
     * @throws Exception
     */
    public AntProfiler() throws Exception {
        super();
    }

    public void execute() throws BuildException {
        if (outputDir==null) {
            throw new BuildException("outputDir is mandatory");
        }
        if (logDirectory==null) {
            throw new BuildException("logDirectory is mandatory");
        }
        String[] ids = findPIDs();

        for (int i=0;i<ids.length;i++) {
	        System.gc();
	        standalone = new StandaloneProcess();
	        try {
	            standalone.doProcess(logDirectory,ids[i],1000);
	        }
	        catch (Exception e) {
	            System.out.println("Error on PID " + ids[i]);
	            e.printStackTrace();
	            continue;
	        }
	        try {
	            String baseFileName = "/geralPID_" + ids[i]+ "_";
                String classesArray[] = getClasses();
                generateXMLs(standalone, baseFileName, classesArray);
                generateGraphs(standalone, baseFileName, classesArray,ids[i]);
            } catch (IOException e) {
                e.printStackTrace();
                throw new BuildException (e.toString());
            }
	        System.gc();
        }

        try {
            generateSummary();
            generateAppletFile();
        }
        catch (IOException e) {
            e.printStackTrace();
            throw new BuildException (e.getMessage());
        }
    }

    /**
     * @throws FileNotFoundException
     * @throws IOException
     */
    private void generateAppletFile() throws  IOException {
        InputStream recInput = this.getClass().getClassLoader().getResourceAsStream("graphApplet.jar");
        InputStream inputFile = new BufferedInputStream(recInput);
        OutputStream outJar = new FileOutputStream(outputDir + "/graphApplet.jar");
        int byteRead=0;
        while ((byteRead = inputFile.read())>=0) {
            outJar.write(byteRead);
        }
        outJar.close();
    }

    private void generateSummary() throws IOException {

        generateSummaryOrderBy("index.html", ExecutionInfoSummaryComparator.NAME);
        generateSummaryOrderBy("indexCPU.html", ExecutionInfoSummaryComparator.CPU);
        generateSummaryOrderBy("indexBytes.html", ExecutionInfoSummaryComparator.BYTES);
        generateSummaryOrderBy("indexReleasedBytes.html", ExecutionInfoSummaryComparator.BYTES);
        generateSummaryOrderBy("indexNumberOfCallings.html", ExecutionInfoSummaryComparator.NUMBER_OF_CALLINGS);
    }

    /**
     * @param fileName
     * @param order
     * @throws IOException
     */
    private void generateSummaryOrderBy(String fileName, int order) throws IOException {
        TreeSet set = new TreeSet(new ExecutionInfoSummaryComparator(order));
        set.addAll(methodsSummary.values());
        Iterator iter = set.iterator();
        generateSummaryFile(fileName, iter);
    }

    /**
     * @param out
     * @param iter
     */
    private void generateSummaryFile(String fileName, Iterator iter) throws IOException {
        PrintStream out = new PrintStream(new BufferedOutputStream(new FileOutputStream(outputDir + "/" + fileName)));
        out.println("<html><body>");

        out.println("<table>");
        out.println("<tr bgcolor=\"#003366\" bordercolor=\"#666666\" style=\"COLOR:white;\">");
        out.print("<td nowrap='nowrap'><a href=\"./index.html\">Name</a></td>");
        out.print("<td nowrap='nowrap'><a href=\"./indexNumberOfCallings.html\">Callings</a></td>");
        out.print("<td nowrap='nowrap'><a href=\"./indexCPU.html\">CPU</a></td>");
        out.print("<td nowrap='nowrap'><a href=\"./indexCPU.html\">Time</a></td>");
        out.print("<td nowrap='nowrap'><a href=\"./indexBytes.html\">Bytes</a></td>");
        out.print("<td nowrap='nowrap'><a href=\"./indexReleasedBytes.html\">Released Bytes</a></td>");
        out.print("<td nowrap='nowrap'>Graphs</td>");
        out.println("</tr>");



        int line=0;
        while (iter.hasNext()) {
            ExecutionInfoSummary summary = (ExecutionInfoSummary)iter.next();
            if ((line++)%2==0) {
                out.println("<tr bgcolor=white>");
            } else {
                out.println("<tr bgcolor=\"#cccccc\">");
            }
            out.print("<td>" + summary.getFullName() + "</td>");
            out.print("<td>" + summary.getNumberOfCallings() + "</td>");
            out.print("<td>" + summary.getCpuTime() + "</td>");
            out.print("<td>" + summary.getTime() + "</td>");
            out.print("<td>" + summary.getBytes() + "</td>");
            out.print("<td>" + summary.getReleasedBytes() + "</td>");
            Iterator iterFiles = summary.getGraphFiles().iterator();
            out.print("<td nowrap='nowrap'>");
            int i=0;
            while (iterFiles.hasNext()) {
                out.print("&nbsp;");
                out.print("<a href=\"." + iterFiles.next().toString() + "\"> Graph " + (i++) + "</a>");
            }
            out.print("</td>");

            out.println("</tr>");

        }



        out.println("</table>");

        out.println("</body></html>");
        out.close();
    }

    private ExecutionInfoSummary prepareSummary(JBPMethod jbpMethod, String pid) {
        ExecutionInfoSummary summaryMethod = (ExecutionInfoSummary) methodsSummary.get(jbpMethod.getFullName());
        if (summaryMethod==null) {
            summaryMethod = new ExecutionInfoSummary(jbpMethod);
            methodsSummary.put(jbpMethod.getFullName(),summaryMethod);
        }

        summarizeMethod(pid, summaryMethod, jbpMethod,false);

        return summaryMethod;

    }

    /**
     * @param pid
     * @param summary
     * @param jbpMethod
     */
    private void summarizeMethod(String pid, ExecutionInfoSummary summary, JBPMethod jbpMethod,boolean includeChildrenInMemoryAnalysis) {

        JBPMethodCount consolidation = standalone.getEngine().getProcess().consolidaCallings(jbpMethod);
        Iterator iter;
        if (includeChildrenInMemoryAnalysis) {
            iter = consolidation.createConsolidatedObjects().values().iterator();
        } else {
            iter = consolidation.getCountMemory().values().iterator();
        }

        long bytes=0;
        long releasedBytes=0;
        while (iter.hasNext()) {
            JBPMemoryCount memoryCount = (JBPMemoryCount)iter.next();
            bytes += memoryCount.getSize();
            releasedBytes += memoryCount.getReleasedSize();
        }

        summary.addProcess(pid);
        summary.setBytes(summary.getBytes() + bytes);
        summary.setReleasedBytes(summary.getBytes() + releasedBytes);
        summary.setCpuTime(summary.getCpuTime() + jbpMethod.getTotalClock());
        summary.setTime(summary.getTime() + jbpMethod.getTotalElapsedClock());
        summary.setNumberOfCallings(jbpMethod.getNumberOfCalls());
    }

    private void generateGraphs(StandaloneProcess standalone, String baseFileName, String[] classesArray, String pid) throws IOException {
        for (int clazzCount=0;clazzCount<classesArray.length;clazzCount++) {
            JBPClass jbpClass = null;
            try {
                jbpClass = standalone.getEngine().getProcess().findClassByName(classesArray[clazzCount]);
            }
            catch (Exception e) {
                continue;
            }
            if (jbpClass==null) {
                continue;
            }
            Iterator methods = jbpClass.getSpyMethods().values().iterator();
            int count=0;
            while (methods.hasNext()) {
                JBPMethod jbpMethod = (JBPMethod)methods.next();

                GraphCallings graph = GraphGenerator.generateCallings(standalone.getEngine().getProcess(),jbpMethod);
                if (graph.getMethods().size()==0) {
                    continue;
                }

                ExecutionInfoSummary summary = prepareSummary(jbpMethod,pid);

                String basicFileName = baseFileName + classesArray[clazzCount]+"_method_" + (count++);
                summary.addGraphFile(basicFileName + ".html");
                String fileName = outputDir + basicFileName;


                ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(fileName + ".obj")));
                out.writeObject(graph);
                out.close();

                PrintStream printOut = new PrintStream(new BufferedOutputStream(new FileOutputStream(fileName+".html")));
                printOut.println("<html>");
                printOut.println("<body>");
                printOut.println("<br>Graph of " + jbpMethod.getFullName());
                printOut.println("<APPLET CODE=\"org.jboss.profiler.graphapplet.GraphApplet\" WIDTH=800 HEIGHT=600 ");
                printOut.println("ARCHIVE=\"graphApplet.jar\">");
                printOut.println("<param name = \"url\" value=\"./" + basicFileName + ".obj\">");
			    printOut.println("</APPLET>");
		        printOut.println("</body>");
	            printOut.println("</html>");
	            printOut.close();

            }
        }
    }

    /**
     * @param standalone
     * @param baseFileName
     * @param classesArray
     * @throws FileNotFoundException
     */
    private void generateXMLs(StandaloneProcess standalone, String baseFileName, String[] classesArray) throws FileNotFoundException {
        for (int clazzCount=0;clazzCount<classesArray.length;clazzCount++) {
            try {
	            PrintStream out = new PrintStream(new BufferedOutputStream(new FileOutputStream(outputDir + baseFileName + classesArray[clazzCount] + ".xml")));
	            XmlGenerationUtil.generateXML(classesArray[clazzCount],out,standalone.getEngine().getProcess(),0,0,false,false,(NumberFormat)new java.text.DecimalFormat());
	            out.close();
            }
            catch (Exception e) {
                System.out.println("Warning:");
                e.printStackTrace();
            }
        }
    }

    /**
     * @return
     */
    private String[] findPIDs() {
        File file = new File(logDirectory);
        if (!file.exists()){
            throw new BuildException(file.toString() + " doesn't exists");
        }

        String files[] = file.list(new ProfilerFilter());

        ArrayList list = new ArrayList();
        for (int i=0;i<files.length;i++) {
            String str = files[i].substring(10);
            str=str.substring(0,str.indexOf("_main"));
            list.add(str);
        }

        String ids[] = (String[])list.toArray(new String[list.size()]);
        return ids;
    }

    private String convertClass(String className) {
        String retClass = className.replace('\\','.');
        retClass = retClass .replace('/','.');
        int pos = retClass.lastIndexOf(".class");
        if (pos>=0) {
            retClass = retClass.substring(0,pos);
        }
        return retClass;
    }

    private String[] getClasses() {
        ArrayList classesReturn = new ArrayList();
        Iterator iter = classes.iterator();
        while (iter.hasNext()) {
            classesReturn.add(convertClass(iter.next().toString()));
        }
        iter = filesets.iterator();
        while (iter.hasNext()) {
            FileSet set = (FileSet) iter.next();
            DirectoryScanner scanner = set.getDirectoryScanner(this
                    .getProject());
            String files[] = scanner.getIncludedFiles();
            for (int i = 0; i < files.length; i++) {
                classesReturn.add(convertClass(files[i]));
            }
        }

        return (String []) classesReturn.toArray(new String[classesReturn.size()]);
    }

    class ProfilerFilter implements FilenameFilter {
        public boolean accept(File file, String name){
            return name.startsWith("serverspy_") && name.indexOf("_main")>0;
        }
    }


}