/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.ant;

import java.io.Serializable;
import java.util.ArrayList;

import org.jboss.profiler.model.JBPMethod;

/**
 * Has information about classes processed during analyzis
 * 
 * @author Clebert Suconic
 */
public class ExecutionInfoSummary implements Serializable {

    public ExecutionInfoSummary() {
    }

    public ExecutionInfoSummary(JBPMethod method) {
        this.className = method.getSpyClass().getName();
        this.name = method.getName();
        this.fullName = method.getFullName();
    }

    String className;

    String name;

    String fullName;

    long bytes;

    long releasedBytes;

    long time;

    long cpuTime;

    long numberOfCallings;

    /** ArrayList containing the PIDs executed during the processing */
    ArrayList processes = new ArrayList();

    ArrayList graphFiles = new ArrayList();

    /**
     * @return Returns the numberOfCallings.
     */
    public long getNumberOfCallings() {
        return numberOfCallings;
    }

    /**
     * @param numberOfCallings
     *            The numberOfCallings to set.
     */
    public void setNumberOfCallings(long numberOfCallings) {
        this.numberOfCallings = numberOfCallings;
    }

    /**
     * @return Returns the bytes.
     */
    public long getBytes() {
        return bytes;
    }

    /**
     * @param bytes
     *            The bytes to set.
     */
    public void setBytes(long bytes) {
        this.bytes = bytes;
    }

    /**
     * @return Returns the className.
     */
    public String getClassName() {
        return className;
    }

    /**
     * @param className
     *            The className to set.
     */
    public void setClassName(String className) {
        this.className = className;
    }

    /**
     * @return Returns the cpuTime.
     */
    public long getCpuTime() {
        return cpuTime;
    }

    /**
     * @param cpuTime
     *            The cpuTime to set.
     */
    public void setCpuTime(long cpuTime) {
        this.cpuTime = cpuTime;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }

    /**
     * @param name
     *            The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return Returns the processes.
     */
    public ArrayList getProcesses() {
        return processes;
    }


    /**
     * @return Returns the releasedBytes.
     */
    public long getReleasedBytes() {
        return releasedBytes;
    }

    /**
     * @param releasedBytes
     *            The releasedBytes to set.
     */
    public void setReleasedBytes(long releasedBytes) {
        this.releasedBytes = releasedBytes;
    }

    /**
     * @return Returns the time.
     */
    public long getTime() {
        return time;
    }

    /**
     * @param time
     *            The time to set.
     */
    public void setTime(long time) {
        this.time = time;
    }

    /**
     * @return Returns the fullName.
     */
    public String getFullName() {
        return fullName;
    }

    /**
     * @param fullName
     *            The fullName to set.
     */
    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    public void addProcess(String process) {
        processes.add(process);
    }

    public void addGraphFile(String file) {
        graphFiles.add(file);
    }

    /**
     * @return Returns the graphFiles.
     */
    public ArrayList getGraphFiles() {
        return graphFiles;
    }

    
}