/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.ant;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import org.jboss.profiler.graphmodel.GraphCallings;
import org.jboss.profiler.graphmodel.GraphMethod;
import org.jboss.profiler.graphmodel.GraphReference;
import org.jboss.profiler.model.JBPMethod;
import org.jboss.profiler.model.JBPMethodCount;
import org.jboss.profiler.model.JBPProcess;

/**
 * @author Clebert Suconic
 */
public class GraphGenerator {
    
    public static String DEFAULT_URL="";

    public static GraphCallings generateCallings(JBPProcess process, JBPMethod methodParameter) {

        java.util.HashMap spyMethods = process.getSpyMethods();

        JBPMethodCount consolidacao = process.consolidaCallings(methodParameter);
        java.util.Stack stack = new java.util.Stack();

        HashMap hashMethods = new HashMap();
        HashMap hashCallings = new HashMap();
        GraphCallings graph = new GraphCallings();

        // para ajudar na referencia entre chamadas
        GraphReference referenceKey = new GraphReference();


        long oldMethod = 0;

        int level = 0;
        // root
        GraphMethod methodKey = new GraphMethod();


        JBPMethod methodConsolidado = consolidacao.getSpyMethod();
        methodConsolidado = (JBPMethod) spyMethods.get(methodConsolidado);

        generateCallingsRecursive(level, oldMethod, methodParameter, consolidacao,
                hashMethods, hashCallings, referenceKey);

        TreeSet treeSet = new TreeSet(new CompareMethods());
        treeSet.addAll(hashMethods.values());

        ArrayList listMethods = new ArrayList();
        listMethods.addAll(treeSet);
        ArrayList listRefs = new ArrayList();
        listRefs.addAll(hashCallings.values());
        graph.setMethods(listMethods);
        graph.setReferences(listRefs);

        //graph.printDebug();

        return graph;
    }

    private static void generateCallingsRecursive(int level, long methodIdCallee,
            JBPMethod rootMethod, JBPMethodCount consolidation,
            HashMap hashMethods, HashMap hashCallings,
            GraphReference referenceKey) {

        GraphMethod keyMethod = new GraphMethod();
        Iterator iterValues = consolidation.getCountCalleds().values()
                .iterator();
        while (iterValues.hasNext()) {
            JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();

            double percenptual = ((double) subConsolidacao
                    .getTotalElapsedClock() / (double) rootMethod
                    .getTotalElapsedClock())
                    * (double) 100;
            if (percenptual < 1.00)
                continue;

            keyMethod.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
            keyMethod.setLevel(level);

            if (hashMethods.get(keyMethod) == null) {
                GraphMethod newMethod = new GraphMethod(subConsolidacao
                        .getSpyMethod(), level, DEFAULT_URL);
                hashMethods.put(newMethod, newMethod);
            }
        }

        iterValues = consolidation.getCountCalleds().values().iterator();
        while (iterValues.hasNext()) {
            JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();

            keyMethod.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
            keyMethod.setLevel(level);
            GraphMethod graphMethod = (GraphMethod) hashMethods.get(keyMethod);

            double percenptual = ((double) subConsolidacao
                    .getTotalElapsedClock() / (double) rootMethod
                    .getTotalElapsedClock())
                    * (double) 100;
            if (percenptual < 1.00)
                continue;

            if (graphMethod == null) {
                throw new RuntimeException("Unexpected null method");
            }

            referenceKey.setMethodCallee(methodIdCallee);
            referenceKey.setLevelCallee(level - 1);
            referenceKey.setLevelCalled(level);
            referenceKey.setMethodCalled(subConsolidacao.getSpyMethod()
                    .getMethodID());
            if (hashCallings.get(referenceKey) == null) {
                GraphReference ref = new GraphReference(level - 1,
                        methodIdCallee, level, subConsolidacao.getSpyMethod()
                                .getMethodID(), ((double) subConsolidacao
                                .getTotalElapsedClock() / (double) rootMethod
                                .getTotalElapsedClock())
                                * (double) 100, (double) subConsolidacao
                                .getTotalClocks()
                                / (double) rootMethod.getTotalClock()
                                * (double) 100);
                graphMethod.setWeight(graphMethod.getWeight() + percenptual);
                hashCallings.put(ref, ref);
            }

            generateCallingsRecursive(level + 1, subConsolidacao.getSpyMethod()
                    .getMethodID(), rootMethod, subConsolidacao, hashMethods,
                    hashCallings, referenceKey);

        }

    }

    //Clean up resources
    public void destroy() {
    }

    /** Comparator class that will sort GraphMethod TreeSet */
    static class CompareMethods implements java.util.Comparator {
        public int compare(Object left, Object right) {
            int iret = ((GraphMethod) left).getLevel()
                    - ((GraphMethod) right).getLevel();
            if (iret != 0)
                return iret;
            double fret = ((GraphMethod) left).getWeight()
                    - ((GraphMethod) right).getWeight();

            if (fret > 0)
                return -1;
            else if (fret < 0)
                return 1;
            else
                return ((GraphMethod) left).toString().compareTo(
                        ((GraphMethod) right).toString());
        }
    }
}
