/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.consolidator;

import org.jboss.profiler.liteversion.jni.LiteVersionController;

/**
 * Class that talks to {@link org.jboss.profiler.liteversion.jni.LiteVersionController} generating messages to the Server
 * making an interface to the client
 * @author Clebert Suconic
 */
public class Consolidator {

    public Consolidator() {
        super();
    }
    
    public static void deActivate() {
       System.out.println("Deactivating profiler");
       LiteVersionController.deActivate();    
    }
    
    public static void activate() {
        System.out.println("Activating profiler");
        LiteVersionController.activate();
    }

    public static void initData() {
        LiteVersionController.runGC();
        LiteVersionController.saveCurrentInfo();
    }
    
    public static ConsolidatorData getData() {
        LiteVersionController.runGC();
        ConsolidatorData data = new ConsolidatorData();
        data.setAllocatedMemory(LiteVersionController.getBytes());
        data.setObjectsAllocated(LiteVersionController.getCreatedObjects());
        data.setObjectsReleased(LiteVersionController.getReleasedObjects());
        data.setUsedCPU(LiteVersionController.getCPU());
        
        data.setTotalAllocatedMemory(LiteVersionController.getTotalBytes());
        data.setTotalObjectsAllocated(LiteVersionController.getTotalCreatedObjects());
        data.setTotalObjectsReleased(LiteVersionController.getTotalReleasedObjects());
        data.setTotalUsedCPU(LiteVersionController.getTotalCPU());
        
        
        String name =LiteVersionController.getName();
        if (name==null) {
            data.setName("null");
        } else {
            data.setName(name);
        }
        return data;
    }

}
