/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.consolidator;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * Represents the current data contained inside the JVM.
 * @author Clebert Suconic
 */
public class ConsolidatorData implements Externalizable {

    public ConsolidatorData() {
        super();
    }


    /**
     * @return Returns the allocatedMemory.
     */
    public long getAllocatedMemory() {
        return allocatedMemory;
    }
    /**
     * @param allocatedMemory The allocatedMemory to set.
     */
    public void setAllocatedMemory(long allocatedMemory) {
        this.allocatedMemory = allocatedMemory;
    }
    /**
     * @return Returns the objectsAllocated.
     */
    public int getObjectsAllocated() {
        return objectsAllocated;
    }

    /**
     * @param objectsAllocated
     *            The objectsAllocated to set.
     */
    public void setObjectsAllocated(int objectsAllocated) {
        this.objectsAllocated = objectsAllocated;
    }

    /**
     * @return Returns the objectsReleased.
     */
    public int getObjectsReleased() {
        return objectsReleased;
    }

    /**
     * @param objectsReleased
     *            The objectsReleased to set.
     */
    public void setObjectsReleased(int objectsReleased) {
        this.objectsReleased = objectsReleased;
    }

    /**
     * @return Returns the usedCPU.
     */
    public long getUsedCPU() {
        return usedCPU;
    }

    /**
     * @param usedCPU
     *            The usedCPU to set.
     */
    public void setUsedCPU(long usedCPU) {
        this.usedCPU = usedCPU;
    }

    long allocatedMemory;

    int objectsAllocated;

    int objectsReleased;

    long usedCPU;
    
    long totalAllocatedMemory;

    int totalObjectsAllocated;

    int totalObjectsReleased;

    long totalUsedCPU;
    
    String name = "default name";
    
    /** For use on reports. (Not filled by the profiler itself. Filled by the Report generator) 
     * @see org.jboss.ant.taskdefs.JUnitLiteProfilerListener */
    String testName;
    
    
    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }
    /**
     * @param name The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }
    /**
     * @return Returns the totalAllocatedMemory.
     */
    public long getTotalAllocatedMemory() {
        return totalAllocatedMemory;
    }
    /**
     * @param totalAllocatedMemory The totalAllocatedMemory to set.
     */
    public void setTotalAllocatedMemory(long totalAllocatedMemory) {
        this.totalAllocatedMemory = totalAllocatedMemory;
    }
    /**
     * @return Returns the totalObjectsAllocated.
     */
    public int getTotalObjectsAllocated() {
        return totalObjectsAllocated;
    }
    /**
     * @param totalObjectsAllocated The totalObjectsAllocated to set.
     */
    public void setTotalObjectsAllocated(int totalObjectsAllocated) {
        this.totalObjectsAllocated = totalObjectsAllocated;
    }
    /**
     * @return Returns the totalObjectsReleased.
     */
    public int getTotalObjectsReleased() {
        return totalObjectsReleased;
    }
    /**
     * @param totalObjectsReleased The totalObjectsReleased to set.
     */
    public void setTotalObjectsReleased(int totalObjectsReleased) {
        this.totalObjectsReleased = totalObjectsReleased;
    }
    /**
     * @return Returns the totalUsedCPU.
     */
    public long getTotalUsedCPU() {
        return totalUsedCPU;
    }
    /**
     * @param totalUsedCPU The totalUsedCPU to set.
     */
    public void setTotalUsedCPU(long totalUsedCPU) {
        this.totalUsedCPU = totalUsedCPU;
    }

    public String toString() {
        return name + " bytes = " + this.getAllocatedMemory();
    }
    
    public void add(ConsolidatorData sum) {
        this.allocatedMemory += sum.allocatedMemory;

        this.objectsAllocated += sum.objectsAllocated;

        this.objectsReleased += sum.objectsReleased;

        this.usedCPU += sum.usedCPU;
        
        this.totalAllocatedMemory += sum.totalAllocatedMemory;

        this.totalObjectsAllocated += sum.totalObjectsAllocated;

        this.totalObjectsReleased += sum.totalObjectsReleased;
        
    }


    /* (non-Javadoc)
     * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        allocatedMemory=in.readLong();
        objectsAllocated=in.readInt();
        objectsReleased=in.readInt();
        usedCPU=in.readLong();
        totalAllocatedMemory = in.readLong();
        totalObjectsAllocated = in.readInt();
        totalObjectsReleased = in.readInt();
        name = in.readUTF();
        
    }


    /* (non-Javadoc)
     * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
     */
    public void writeExternal(ObjectOutput out) throws IOException {
        if (name==null) {
            name="null";
        }
        
        out.writeLong(allocatedMemory);
        out.writeInt(objectsAllocated);
        out.writeInt(objectsReleased);
        out.writeLong(usedCPU);
        out.writeLong(totalAllocatedMemory);
        out.writeInt(totalObjectsAllocated);
        out.writeInt(totalObjectsReleased);
        out.writeUTF(name);
        
    }

    /**
     * @return Returns the testName.
     */
    public String getTestName() {
        return testName;
    }
    /**
     * @param testName The testName to set.
     */
    public void setTestName(String testName) {
        this.testName = testName;
    }
}