/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.consolidator;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 * @author Clebert Suconic
 */
public class DataProducerProxy implements Runnable {

    private static void debug(String message) {
        System.out.println("DataProducerProxy:" + message);
    }

    Socket socket=null;
    ObjectInputStream inp = null;
    ObjectOutputStream objectOutput = null;

    public DataProducerProxy(Socket socket) throws IOException {
        inp = new ObjectInputStream(socket.getInputStream());
        objectOutput = new ObjectOutputStream(socket.getOutputStream());
    }

    /*
     * @see java.lang.Runnable#run()
     */
    public void run() {
        try {
            debug("Sending DATAPRODUCER signature");
            objectOutput.writeObject("DATAPRODUCER");
            objectOutput.flush();
            objectOutput.reset();
            //objectOutput.flush();
        } catch (IOException e2) {
            e2.printStackTrace();
        }
        for (;;) {
            try {
                debug("read object (hanging)");
                Object obj = inp.readObject();
                debug("received command " + obj);
                String command = (String)obj;
                if (command.equals("GETDATA")) {
                    debug("Sending Consolidator Data");
                    // sends the current profiler data
                    objectOutput.writeObject(Consolidator.getData());
                    objectOutput.flush();
                } else if (command.equals("SAVEDATA")){
                    Consolidator.initData();
                } else if (command.equals("DEACTIVATE")){
                    Consolidator.deActivate();
                } else if (command.equals("ACTIVATE")){
                    Consolidator.activate();
                } else {
                    System.out.println("Communication error " + command);
                }
            } catch (Exception e) {
                e.printStackTrace();
                break;
            }
        }
    }
    
    public static void startProducer(String address)  {
        int local = address.indexOf(":");
        String adr1 = address.substring(0,local);
        int port = Integer.parseInt(address.substring(local+1));
        try {
            Socket socket = new Socket(adr1,port);
            DataProducerProxy producer = new DataProducerProxy(socket);
            new Thread(producer,"Profiler Producer").start();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public static void main(String arg[]) {
       if (arg.length<=2) {
            System.out.println("java org.jboss.profiler.liteversion.consolidator.DataProducerProxy <Host> <classname> parameters");
            System.out.println("Host=the address of the consolidator server");
            System.out.println("ClassName=The classname to be delegated");
            System.out.println("...=All the parameters passed here will be send to that classname");
            System.exit(0);
        }
        
        String argnew[] = new String[arg.length-2];
        
        for (int i=0;i<argnew.length;i++) {
            argnew[i] = arg[i+2];
        }
        
        startProducer(arg[0]);
        
        try {
            Class clazz = Class.forName(arg[1]);
            Method method = clazz.getMethod("main",new Class[] {arg.getClass()});
            method.invoke(null,argnew);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        
    }

}
