/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.consolidator;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * A lightweight protocol for sending commands and receiving objects.
 * We are not using rmi to avoid technical problems like messing up with server's RMI.
 * @author Clebert Suconic
 */
public class DataSynchronizer {
    
    
    public static void debug(String message) {
        System.out.println("DataSynchronizer:" + message);
    }
    
    static ArrayList outputStreams = new ArrayList();
    static ArrayList inputStreams = new ArrayList();
    
    static class SocketListener implements Runnable {
       
        public SocketListener(ServerSocket server) {
            this.serverSocket = server;
        }

        ServerSocket serverSocket;
        
        /* 
         * @see java.lang.Runnable#run()
         */
        public void run() {
            try {
	            for (;;) {
	                ObjectOutputStream out = null;
	                ObjectInputStream inp = null;
	                Socket accept = serverSocket.accept();
	                out = new ObjectOutputStream(accept.getOutputStream());
	                inp = new ObjectInputStream(accept.getInputStream());
	                String signature = (String)inp.readObject();
	                if (signature.equals("DATAPRODUCER")) {
	                    debug("Adding a dataProducer");
		                outputStreams.add(out);
		                inputStreams.add(inp);
	                } else 
	                if (signature.equals("DATACONSUMER")){
	                    Thread.sleep(100);
	                    Object messages[] = readMessages();
	                    out.writeObject(messages);
	                    out.flush();
	                    inp.close();
	                    out.close();
	                } else {
	                    sendCommand(signature);
	                }
	                
	                
	                
	            }
            } catch (IOException e) {
                System.out.println("Consolidator thread finished by Exception");
                e.printStackTrace();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
        
    }
    
    /** Sends a String command to all sockets */
    public static void sendCommand(String command) {
        Iterator iter = outputStreams.iterator();
        
        ObjectOutputStream outArray[] = (ObjectOutputStream[])outputStreams.toArray(new ObjectOutputStream[outputStreams.size()]);
        for (int position=0;position<outArray.length;position++) {
            debug("Sending command " + command + " on position " + position);
            ObjectOutputStream out =outArray[position];
            try {
	            out.writeObject(command);
	            out.flush();
	            out.reset();
            } catch (IOException e) {
                // The outputStream must be down
                debug("Position " + position + " removed");
                outputStreams.remove(position);
                inputStreams.remove(position);
            }
            
        }
    }

    /** Return object messages */
    public static Object[] readMessages() {
        sendCommand("GETDATA");
        
        ArrayList returnObjects = new ArrayList();
        
        int position=0;
        Iterator iter = inputStreams.iterator();
        while (iter.hasNext()) {
            debug("Reading object on position " + position);
            try {
                ObjectInputStream inp = (ObjectInputStream)iter.next();
                returnObjects.add(inp.readObject());
            }
            catch (IOException e) {
                e.printStackTrace();
            }
            catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }        
        return returnObjects.toArray();
    }

    public DataSynchronizer() {
        super();
    }
    
    public static void startSynchronizer(int port) throws IOException {
        debug("running synchronizer on port " + port);
        ServerSocket server = new ServerSocket(port);
        SocketListener listener = new SocketListener(server);
        Thread thread = new Thread(listener,"Server Listener");
        thread.start();
    }
    
    public static void main(String arg[]) {
        if (arg.length!=1) {
            System.err.println("USE: java -jar profilerSynchronizer.jar port");
            return;
        }
        
        try {
            startSynchronizer(Integer.parseInt(arg[0]));
        } catch (NumberFormatException e) {
             System.err.println("Invalid port number");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
