/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.output;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.BeanUtils;
import org.jboss.profiler.liteversion.consolidator.ConsolidatorData;

/**
 * Reads and generates {@link org.jboss.profiler.liteversion.consolidator.ConsolidatorData}
 * @author Clebert Suconic
 */
public class ConsolidatorDataFactory {

    private static Object[] nullObject = new Object[]{};
    
    /**
     * 
     */
    public ConsolidatorDataFactory() {
        super();
    }

    /**
     * (I know that {@link org.apache.commons.beanutils.BeanUtils#describe(java.lang.Object)} does this function but I didn't want to tight this tiny piece of code on jakarta's Beanutil due to dependencies at profiled application's runtime.)
     * @param descriptors
     * @param nullObject
     * @param obj
     * @return
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * @throws IllegalAccessException
     */
    public static Map reflectProperties(ConsolidatorData obj) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        return BeanUtils.describe(obj);
    }

    /**
     * Exports an object into properties
     * @param descriptors
     * @param nullObject
     * @param obj
     * @return
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     * @throws NoSuchMethodException
     */
    public static String exportProperties(ConsolidatorData obj) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        Map properties = reflectProperties(obj);
        StringBuffer buffer = new StringBuffer();
        Iterator iterProperties = properties.keySet().iterator();
        while (iterProperties.hasNext()) {
            String key = (String)iterProperties.next();
            buffer.append(key + "=" + properties.get(key));
            if (iterProperties.hasNext()) {
                buffer.append(";");
            }
        }
        String result = buffer.toString();
        return result;
    }
    
    
    /** 
     * Create a object from properties.
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     * */
    public static ConsolidatorData importProperties(String line) throws IllegalAccessException, InvocationTargetException {
        ConsolidatorData data = new ConsolidatorData();
        
        StringTokenizer tokenString = new StringTokenizer(line,";");

        Map map = new HashMap();
        while (tokenString.hasMoreTokens()) {
            String nextToken = tokenString.nextToken();
            String name = nextToken.substring(0,nextToken.lastIndexOf('='));
            String value = nextToken.substring(nextToken.lastIndexOf('=')+1);
            map.put(name,value);
        }
        
        BeanUtils.populate(data,map);
        
        return data;
    }
    

}
