/*
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.profiler.liteversion.output;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.jboss.profiler.liteversion.consolidator.ConsolidatorData;

/**
 * Generates the output of JUnitProfiler based on 
 * @author Clebert Suconic
 */
public class JUnitProfilerOutputFormater {
    
    /** Each element is a hashtable containing the names for that level */
    ArrayList items = new ArrayList();
    
    private HashMap findLevel(int level) {
        while (items.size()<=level) {
            items.add(new HashMap());
        }
        
        return (HashMap)items.get(level);        
    }
    
    private ConsolidatorData findData(int level, String name) {
        HashMap map = findLevel(level);
        ConsolidatorData data = (ConsolidatorData) map.get(name);
        if (data==null) {
            data = new ConsolidatorData();
            data.setName(name);
            map.put(name,data);
        }        
        return data;
    }
    
    InputStream input;

    /**
     * 
     */
    public JUnitProfilerOutputFormater(InputStream input) {
        super();
        this.input=input;
    }
    
    public void doProcess(File outputDir) throws IOException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        doAgregation();
        doOutput(outputDir);
    }

    /**
     * @throws IOException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    void doAgregation() throws IOException, IllegalAccessException, InvocationTargetException {
        BufferedInputStream buff = null;
        if (input instanceof BufferedInputStream) {
            buff = (BufferedInputStream)buff;
        } else {
            buff = new BufferedInputStream(input);
        }
        BufferedReader reader = new BufferedReader(new InputStreamReader(input));
        String line=null;
        while ((line = reader.readLine())!=null) {
            ConsolidatorData data = ConsolidatorDataFactory.importProperties(line);
            StringTokenizer token = new StringTokenizer(data.getTestName(),".");
            String currentName = "";
            int level=0;
            while (token.hasMoreTokens()) {
                String strToken = token.nextToken();
                if (strToken.indexOf(':')>=0) {
                    token = new StringTokenizer(strToken,":");
                    continue;
                }
                currentName=currentName+strToken;
                ConsolidatorData dataAgregator = findData(level++,currentName);
                dataAgregator.add(data);
                currentName+=".";
            }
        }
    }
    
    void doOutput(File outputdir) throws IOException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
        Iterator iter = items.iterator();
        int level=0;
        while (iter.hasNext()) {
            HashMap map = (HashMap)iter.next();
            PrintStream out = new PrintStream(new FileOutputStream(new File(outputdir,"test" + level + ".html")));
            out.println("<html><body>");
            out.println("<br>Level="+(level++));
            out.println("<table border=1>");
            
            out.println("<tr>");
            out.println("<td>Name</td>");
            out.println("<td>#Objects</td>");
            out.println("<td>#Bytes</td>");
            out.println("<td nowrap=nowrap>#Released Objects</td>");
            out.println("<td nowrap=nowrap>CPU MilliSeconds</td>");
            out.println("</tr>");
            Iterator iterItems = map.values().iterator();
            while (iterItems.hasNext()) {
                out.println("<tr>");
                
                ConsolidatorData data = (ConsolidatorData)iterItems.next();
                out.println("<!--<br>"+ConsolidatorDataFactory.exportProperties(data) + "-->");
                out.println("<td>" + data.getName() + "</td>");
                out.println("<td align=right>" + data.getObjectsAllocated() + "</td>");
                out.println("<td align=right>" + data.getAllocatedMemory() + "</td>");
                out.println("<td align=right>" + data.getObjectsReleased() + "</td>");
                out.println("<td align=right>" + data.getUsedCPU()/1000 + "</td>");
                
                out.println("</tr>");
            }
            out.println("</table>");
            out.println("</body></html>");
            out.close();
        }
    }

    
}
