/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.logger;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.jboss.profiler.fileProcessor.InputUtil;
import org.jboss.profiler.fileProcessor.SpyRegister;

/**
 * @author Clebert Suconic
 */
public class FileProfileLogger extends ProfileLogger {

    DataOutputStream output;

    public  FileProfileLogger (OutputStream output) {
        if (output instanceof DataOutputStream) {
            this.output=(DataOutputStream) output;
        } else {
            this.output = new DataOutputStream(output);
        }
    }

    /** Write what is common to every register.
     * format:
     * <table>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>registerType</td><td>1</td><td>byte</td></tr>
     * <tr><td>time (CPU)</td><td>8</td><td>long</td></tr>
     * <tr><td>time (elapsed)</td><td>8</td><td>long</td></tr>
     * </table>
     * */
    protected void writePrefix(short type, long time) throws IOException{
        output.writeByte(type);
        output.writeLong(time);
        output.writeLong(time);
    }

    /*
     * format:
     * <table>
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>threadId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>classId</td><td>4</td><td>int</td></tr>
     * <tr><td>methodId</td><td>4</td><td>int</td></tr>
     * <tr><td>objectId (no use on aop... experimental use on JVMPI (almost no use expected))</td><td>4</td><td>int</td></tr>
     * </table>
     * @see org.jboss.proflier.aop.logger.ProfileLogger#enterMethod(long, int, int)
     */
    public void enterMethod(long time, int threadId, int classId, int methodId) throws IOException {
        writePrefix(SpyRegister.CONST_ENTERMETHOD,time);
        output.writeInt(threadId);// we did it two times because in JVMPI i was using EnvID and ThreadID (Probably I'll get over it when I have JVMTI)
        output.writeInt(threadId);
        output.writeInt(classId);
        output.writeInt(methodId);
        output.writeInt(methodId);
    }


    /*
     * format:
     * <table>
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>threadId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>methodId</td><td>4</td><td>int</td></tr>
     * </table>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#exitMethod(long, int, int)
     */
    public void exitMethod(long time, int threadId, int methodId)
            throws IOException {
        writePrefix(SpyRegister.CONST_EXITMETHOD,time);
        output.writeInt(threadId);
        output.writeInt(threadId);
        output.writeInt(methodId);
    }

    /*
     * <table>
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>threadId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>len(name)</td><td>4</td><td>int</td></tr>
     * <tr><td>name</td><td>byte*size(name)</td><td>int</td></tr>
     * </table>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#startThread(long, int)
     */
    public void startThread(long time, int threadId, String threadName) throws IOException {
        writePrefix(SpyRegister.CONST_THREADSTART,time);
        output.writeInt(threadId);
        output.writeInt(threadId);
        InputUtil.writeString(threadName,output);
    }

    /*
     * <table>
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>threadId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>len(name)</td><td>4</td><td>int</td></tr>
     * <tr><td>name</td><td>byte*size(name)</td><td>int</td></tr>
     * </table>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#finishThread(long, int)
     */
    public void finishThread(long time, int threadId) throws IOException {
        writePrefix(SpyRegister.CONST_THREADEND,time);
        output.writeInt(threadId);
        output.writeInt(threadId);
    }

    /*
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>classId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>len(className)</td><td>4</td><td>int</td></tr>
     * <tr><td>className</td><td>len(classname)</td><td>char * len(className)</td></tr>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#loadClass(long, int, java.lang.String)
     */
    public void loadClass(long time, int classId, String name, int numberOfMethods)
            throws IOException {
        writePrefix(SpyRegister.CONST_LOADCLASS,time);
        output.writeInt(classId);
        InputUtil.writeString(name,output);
        output.writeInt(numberOfMethods);
    }

    /*
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>classId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>methodId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>len(methodName)</td><td>4</td><td>int</td></tr>
     * <tr><td>methodName</td><td>len(methodName)</td><td>char * len(methodName)</td></tr>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#loadMethod(long, int, int, java.lang.String)
     */
    public void loadMethod(long time, int classId, int methodId, String method, String signature)
            throws IOException {
        writePrefix(SpyRegister.CONST_LOADMETHOD,time);
        output.writeInt(classId);
        output.writeInt(methodId);
        InputUtil.writeString(method,output);
        InputUtil.writeString(signature,output);

    }

    /*
     * <tr><td colspan=3>{@link writePrefix(short,long)}</td></tr>
     * <tr><td><b>column</b></td><td><b>size</b></td><td><b>data type</b></td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>threadId</td><td>4</td><td>int</td></tr>
     * <tr><td>classId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>objectId(envId)</td><td>4</td><td>int</td></tr>
     * <tr><td>size(envId)</td><td>4</td><td>int</td></tr>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#newObject(long, int, int, int)
     */
    public void newObject(long time, int threadId, int classId, int objectId, int size)
            throws IOException {
        writePrefix(SpyRegister.CONST_LOADOBJECT,time);
        output.writeInt(threadId);
        output.writeInt(threadId);
        output.writeInt(classId);
        output.writeInt(objectId);
        output.writeInt(size);
    }

    /* (non-Javadoc)
     * @see org.jboss.profiler.aop.logger.ProfileLogger#releaseObject(long, int)
     */
    public void releaseObject(long time, int objectId) throws IOException {
        writePrefix(SpyRegister.CONST_UNLOADOBJECT,time);
        output.writeInt(objectId);

    }

    /*
     * </table>
     * @see org.jboss.profiler.aop.logger.ProfileLogger#headerSignature(long)
     */
    public void headerSignature(long time) throws IOException {
        byte[] signature = new byte [] {'D','P','I','N','S','P'};
        output.write(signature);
        output.writeInt(0xff);
        output.writeLong(time);
    }


}
