/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.logger;

import java.io.IOException;

/**
 * Methods definitions for profiling logger.
 * This class supports those events defined here.
 * @author Clebert Suconic
 */
public abstract class ProfileLogger {
   /** Stabilishes the signature for the Output. (If applicable in the format) */
   public abstract void headerSignature(long time) throws IOException;
   public abstract void startThread(long time, int threadId, String threadName) throws IOException;
   public abstract void finishThread(long time, int threadId) throws IOException;
   public abstract void enterMethod(long time,int threadId, int classId, int methodId) throws IOException;
   public abstract void exitMethod(long time, int threadId, int methodId) throws IOException;
   public abstract void loadClass(long time, int classId, String name, int numberOfMethods) throws IOException;
   public abstract void loadMethod(long time, int classId, int methodId, String method, String signature) throws IOException;
   public abstract void newObject(long time,int threadId, int classId,int objectId,int size) throws IOException;
   public abstract void releaseObject(long time, int objectId) throws IOException;
}
