/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.monitoring;

import java.lang.reflect.Method;
import java.util.HashMap;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.profiler.aop.monitoring.control.MonitoringController;
import org.jboss.profiler.aop.util.ProfilerUtil;

/**
 * @author Clebert Suconic
 */
public class MonitoringInterceptor implements Interceptor {
    public String getName() {
        return "org.jboss.profiler.aop.monitoring.MonitoringInterceptor";
    }

    static HashMap mapMethods = new HashMap();

    protected static String getMethodName(Method method) {
        String methodName = (String)mapMethods.get(method);
        if (methodName==null) {
            synchronized (mapMethods) {
                methodName = (String)mapMethods.get(method);
                if (methodName==null) {
                    methodName = method.getDeclaringClass().getName() + ":" + method.getName() + ProfilerUtil.getMethodSignature(method);
                    mapMethods.put(method,methodName);
                }
            }
        }

        return methodName;
    }


    public Object invoke(Invocation invocation) throws Throwable {
        String methodName = null;
        long timeStart=0, timeEnd=0;
        String className = null;
        if (invocation instanceof MethodInvocation) {
            methodName = getMethodName(((MethodInvocation)invocation).getMethod());
            className = ((MethodInvocation)invocation).getMethod().getDeclaringClass().getName();
            timeStart = System.currentTimeMillis();
            MonitoringController.openStack(className,methodName,timeStart);
        }

        Object object = invocation.invokeNext();

        if (invocation instanceof MethodInvocation) {
            methodName = getMethodName(((MethodInvocation)invocation).getMethod());
            timeEnd = System.currentTimeMillis();
            long timeDifference = timeEnd-timeStart;
            MonitoringController.closeStack(className,methodName,timeStart,timeDifference,timeDifference);
        }

        return object;
    }

}
