/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.monitoring.model;

import java.util.Collection;
import java.util.HashMap;

/**
 * An aggregating entity for MonitoringMeasures.
 * Usually this class contains classes, Servlets or other aggregating entity for measures.
 * For example: A path for a given Servlet (it's good for struts)
 * @author Clebert Suconic
 */
public class MeasureAggregation {

    String name;


    HashMap measures = new HashMap();

    public void setMeasure(String name, MonitoringMeasure measure) {
        if (measures.get(name)==null) {
            measures.put(name,measure);
        }
    }

    public MonitoringMeasure getMeasure(String name) {
        return (MonitoringMeasure) measures.get(name);
    }

    public Collection getMeasureSet() {
        return measures.values();
    }


    public MeasureAggregation() {
    }

    public MeasureAggregation(String name) {
        this.name=name;
    }




    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        return name.equals(((MeasureAggregation)obj).name);
    }
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return name.hashCode();
    }
    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }
    /**
     * @param name The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }
}
