/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.monitoring.model;

import java.io.Serializable;
import java.util.HashMap;


/**
 * @author Clebert Suconic
 */
public class MonitoringMeasure implements Serializable{

    /**
     * @return Returns the values.
     */
    public HashMap getValues() {
        return values;
    }

    private static ThreadLocal cacheKey = new ThreadLocal();

    /** We are caching TimeValues for each Thread, for finding the right key the the HashMap */
    private static TimeValue getCacheKey() {
        TimeValue value = null;
        if ((value=(TimeValue)cacheKey.get())==null) {
            value = new TimeValue(0);
            cacheKey.set(value);
        }

        return value;
    }

    /** contains TimeValues */
    HashMap values = new HashMap();
    TimeValue rootValue = new TimeValue(0);

    String name;


    public MonitoringMeasure(String name) {
        this.name=name;
    }


    public int hashCode() {
        return name.hashCode();
    }

    /** Finds the TimeValue for the timeKey passed as paramter */
    public TimeValue findValue(long timeKey,int grade) {
        TimeValue key = getCacheKey();
        key.setTimeKey(timeKey);

        TimeValue value = (TimeValue)values.get(key);

        if (value==null) {
            synchronized (values) {
                value = new TimeValue(timeKey);
                value.setTimeStart(timeKey*grade);
                values.put(value,value);
            }
        }
        return value;
    }


    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        return name.equals(((MonitoringMeasure)obj).name);
    }

    public String toString() {
        return name;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }
    /**
     * @return Returns the rootValue.
     */
    public TimeValue getRootValue() {
        return rootValue;
    }
}
