/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.profiler.aop.profiler;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.profiler.aop.profiler.control.ProfilerController;

/**
 * Intercepts methods callings, instantiations and finalizes on methods...
 *
 * @author Clebert Suconic
 */
public class ProfilerAgentInterceptor implements Interceptor {

    public String getName() {
        return "org.jboss.profiler.aop.ProfilerAgentInterceptor";
    }

    public Object invoke(Invocation invocation) throws Throwable {
        boolean started= ProfilerController.getInstance().isStarted();

        if (started) {
	        if (invocation instanceof MethodInvocation) {
	            Method method = ((MethodInvocation)invocation).getMethod();
	            ProfilerController.getInstance().enterMethod(method);
	        } else if (invocation instanceof ConstructorInvocation) {
	            ConstructorInvocation constr = (ConstructorInvocation) invocation;
	            Constructor constructor = constr.getConstructor();
	        }
        }


        Object object = invocation.invokeNext();

        if (started) {
	        if (invocation instanceof MethodInvocation) {
	            Method method = ((MethodInvocation)invocation).getMethod();
	            ProfilerController.getInstance().exitMethod(method);
	        }
        }

        return object;
    }

}