/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.aop.profiler.control;

/**
 * Hashkey for loadedClasses, loadThreads and loadedMethods.
 * Using instances of Integer as for keys on loaded classes, or loaded Threads or whatever, would be expensive due to GC operations.
 * And as we don't want to consume any resource from the real application,
 * this class was created to save the need of new objects.
 * Warning: This class of course is not thread-safe. So, always use in the context of thread.
 * Actually, re-use instances of this class only inside ThreadContext, which is used inside a ThreadLocal.
 * A second warning on it's reuse also is, hashCodes should be immutable, but this is a hacking we are doing
 * to improve performance.
 * @author Clebert Suconic
 */
public class HashKey {

    int hash;

    public HashKey(int hash){
        this.hash=hash;
    }

    public int hashCode() {
        return hash;
    }

    /**
     * @return Returns the hash.
     */
    public int getHash() {
        return hash;
    }
    /**
     * @param hash The hash to set.
     */
    public void setHash(int hash) {
        this.hash = hash;
    }

    public boolean equals(Object obj) {
        return hash == ((HashKey)obj).hash;
    }
}
