/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.graphapplet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JApplet;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.DefaultPort;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.ParentMap;

import org.jboss.profiler.graphmodel.GraphCallings;
import org.jboss.profiler.graphmodel.GraphMethod;
import org.jboss.profiler.graphmodel.GraphReference;


/**
 @author Clebert Suconic
*/
public class GraphMemoryApplet extends JApplet {
	boolean isStandalone = false;
	DefaultGraphModel graphModel = new DefaultGraphModel();
	JGraph jgraph = new JGraph(graphModel);
	//JGraph jgraph = new JGraph();
	JScrollPane jScrollPane1 = new JScrollPane(jgraph);

	java.text.NumberFormat numberFormat = java.text.NumberFormat.getInstance();

	private void openModel() {
		try {
			// Intersections model
			ConnectionSet intersectionSet = new ConnectionSet();

			// Attributes
			java.util.HashMap attributes = new java.util.HashMap();

			GraphCallings callings = readObject();
			callings.printDebug();

			DefaultGraphCell elements[] =
				new DefaultGraphCell[callings.getMethods().size()
					+ callings.getReferences().size()];

			int currentLevel = -1;
			int currentCell = -1;
			int currentElement = 0;

			Iterator iter = callings.getMethods().iterator();

			HashMap hashMethods = new HashMap();

			while (iter.hasNext()) {
				GraphMethod graphmethod = (GraphMethod) iter.next();

				if (graphmethod.getLevel() != currentLevel) {
					currentLevel = graphmethod.getLevel();
					currentCell = 0;
				} else {
					currentCell++;
				}

				DefaultGraphCell graphcell = new DefaultGraphCell(graphmethod);
				graphcell.add(new DefaultPort("port/Center"));
				graphcell.add(new DefaultPort("port/Left"));
				Color color = null;
				switch (currentCell) {
					case 0 :
						color = Color.red;
						break;
					case 1 :
						color = Color.yellow;
						break;
					default :
						color = Color.blue;
						break;
				}
				attributes.put(
					graphcell,
					createBounds(currentLevel, currentCell, color));
				elements[currentElement++] = graphcell;
				hashMethods.put(graphmethod, graphcell);
			}

			Map simpleArrow = GraphConstants.createMap();
			GraphConstants.setLineEnd(
				simpleArrow,
				GraphConstants.ARROW_CLASSIC);
			GraphConstants.setBeginSize(simpleArrow, 10);
			GraphConstants.setDashPattern(simpleArrow, new float[] { 3, 3 });
			GraphConstants.setFont(
				simpleArrow,
				GraphConstants.defaultFont.deriveFont(10));

			GraphMethod key = new GraphMethod();

			iter = callings.getReferences().iterator();
			while (iter.hasNext()) {
				GraphReference referenceCalling = (GraphReference) iter.next();

				key.setMethodId(referenceCalling.getMethodCallee());
				key.setLevel(referenceCalling.getLevelCallee());
				DefaultGraphCell cellcallee =
					(DefaultGraphCell) hashMethods.get(key);

				key.setMethodId(referenceCalling.getMethodCalled());
				key.setLevel(referenceCalling.getLevelCalled());
				DefaultGraphCell cellcalled =
					(DefaultGraphCell) hashMethods.get(key);

				DefaultEdge relationship =
					new DefaultEdge(
						numberFormat.format(
							referenceCalling.getInstancesCreated())
							+ " instances");
				elements[currentElement++] = relationship;

                if (cellcallee!=null && cellcalled!=null) {
					intersectionSet.connect(
						relationship,
						cellcallee.getChildAt(0),
						cellcalled.getChildAt(1));

					attributes.put(relationship, simpleArrow);
                } else {
					if (cellcallee==null) System.out.println("Callee id=" + referenceCalling.getMethodCallee());
					if (cellcalled==null) System.out.println("Called id=" + referenceCalling.getMethodCalled());
                }

			}

			graphModel.insert(
				elements,
				attributes,
				intersectionSet,
				new ParentMap(),
				new javax.swing.undo.UndoableEdit[0]);

		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, e.getMessage());
		}

	}

	private GraphCallings readObject() throws MalformedURLException, IOException, ClassNotFoundException {
		java.net.URL url =
			new java.net.URL(
				this.getCodeBase().toString() + getParameter("url"));
		java.io.InputStream inp = url.openStream();
		java.io.ObjectInputStream obj = new java.io.ObjectInputStream(inp);
		GraphCallings callings = (GraphCallings) obj.readObject();

		return callings;
	}

	/**
	 * Returns an attributeMap for the specified position (line,column) and color.
	 */
	public static Map createBounds(int line, int column, Color c) {
		Map map = GraphConstants.createMap();
		GraphConstants.setBounds(
			map,
			new Rectangle(column * 130 + 10, line * 60 + 10, 90, 30));
		GraphConstants.setBorder(map, BorderFactory.createRaisedBevelBorder());
		GraphConstants.setBackground(map, c.darker());
		GraphConstants.setForeground(map, Color.white);
		GraphConstants.setFont(
			map,
			GraphConstants.defaultFont.deriveFont(Font.BOLD, 12));
		GraphConstants.setOpaque(map, true);
		return map;
	}

	//Get a parameter value
	public String getParameter(String key, String def) {
		return isStandalone
			? System.getProperty(key, def)
			: (getParameter(key) != null ? getParameter(key) : def);
	}

	//Construct the applet
	public GraphMemoryApplet() {
	}

	//Initialize the applet
	public void init() {
		try {
			openModel();
			jbInit();
			jgraph.setEditable(false);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	//Component initialization
	private void jbInit() throws Exception {
		this.setSize(new Dimension(400, 300));
		jgraph.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				jgraph_mouseClicked(e);
			}
		});
		this.getContentPane().add(jScrollPane1, BorderLayout.CENTER);
	}
	//Start the applet
	public void start() {
	}
	//Stop the applet
	public void stop() {
	}
	//Destroy the applet
	public void destroy() {
	}
	//Get Applet information
	public String getAppletInfo() {
		return "Applet Information";
	}
	//Get parameter info
	public String[][] getParameterInfo() {
		return null;
	}

	void jgraph_mouseClicked(MouseEvent ev) {
		try {
			if (ev.getClickCount() == 2) {
				Object obj =
					jgraph.getFirstCellForLocation(ev.getX(), ev.getY());
				if (obj != null && obj instanceof DefaultGraphCell) {
					DefaultGraphCell graphCell = (DefaultGraphCell) obj;
					GraphMethod graphMethod =
						(GraphMethod) graphCell.getUserObject();
					java.net.URL urlNavigate =
						new java.net.URL(
							this.getCodeBase().toString()
								+ "instancesStackGraph.jsp"
								+ graphMethod.getUrl());
					this.getAppletContext().showDocument(urlNavigate);
				}
			} else {
				Object obj =
					jgraph.getFirstCellForLocation(ev.getX(), ev.getY());
				if (obj != null && obj instanceof DefaultGraphCell) {
					DefaultGraphCell graphCell = (DefaultGraphCell) obj;
					GraphMethod graphMethod =
						(GraphMethod) graphCell.getUserObject();
				}
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, ex.toString());
		}
	}
}
