/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.console;

import java.io.*;
import java.util.*;

import org.jboss.profiler.threadcheck.StartupProfilerCheck;

/**
 * Classe que recebe ordens do usuario, ativando/desativando o request.
 * A intencao de escrever esta classe como console, foi nao depender
 * de constraints de firewal/disponibilidade de terminais graficos.
 @author Clebert Suconic
*/

public class ProfilerConsole {


	private static final String STRING_PAUSE = "pause";
	private static final String STRING_START = "start";
	private static final String STRING_QUIT = "quit";
	private static final String STRING_STOP = "stop";
	public static final int QUIT_COMMAND=4;

  ProfilerControl control;
  int port;

  public ProfilerConsole(int port) {
     this.port=port;
  }


  public void printMenu() {
     synchronized (System.out) {
        System.out.println();
        Iterator iter = control.getHash().values().iterator();

        System.out.println("PIDs");
        while (iter.hasNext()) {
          ProfilerPID iterPid = (ProfilerPID) iter.next();
          System.out.println("PID =>" + iterPid.getPid() + " (" + iterPid.getActiveCommand() + ")");
        }

        System.out.println("Usage:");
        System.out.println("quit - finish console");
		System.out.println("start <pid> - starts the profiler");
		System.out.println("stop <pid>  - stops the profiler");
		System.out.println("pause <pid> - pauses the profiler (it can be restared with start)");
        System.out.println();
        System.out.print("?");
     }
  }


  /** tokenize string by using StringTokenizer. */
  private String [] token(String string) {
  	StringTokenizer tokenizer = new StringTokenizer(string," ");
  	String []tokens = new String[2];
  	int i=0;

  	while (tokenizer.hasMoreTokens()) {
  		tokens[i++] = tokenizer.nextToken().trim();
  	}

	boolean isQuit=i==1 && tokens[0].trim().equals(STRING_QUIT);

	if (i!=2 && !(isQuit)) {
		return null;
	}


  	return tokens;
  }

  /** Get the command based on a string */
  private int getCommand(String command) {
	if (command.equals(STRING_QUIT)) return QUIT_COMMAND;
  	else if (command.equals(STRING_START)) return StartupProfilerCheck.COMMAND_START;
  	else if (command.equals(STRING_STOP)) return StartupProfilerCheck.COMMAND_STOP;
  	else if (command.equals(STRING_PAUSE)) return StartupProfilerCheck.COMMAND_PAUSE;
  	else return -1;
  }

  public void doConsole() throws IOException {
     control = new ProfilerControl(this,port);
     Thread threadControl = new Thread(control);
     threadControl.start();

     BufferedReader input = new BufferedReader(new InputStreamReader(System.in));

     String strLine=null;
     while (true) {
         printMenu();
         strLine=input.readLine();
         String []tokens = token(strLine);
         if (tokens==null) {
         	System.out.println("Invalid parameters!");
         	continue;
         }

         int command=getCommand(tokens[0]);
         if (command<0) {
			System.out.println("Invalid command");
         	continue;
         }

         if (command==QUIT_COMMAND) {
            System.out.println("Datacapturing quit");
            System.exit(0);
         }

		 Integer pid = new Integer(tokens[1]);
		 ProfilerPID profilerPid = (ProfilerPID)control.getHash().get(pid);
		 profilerPid.executeCommand(tokens[0],command);
		 System.out.println("changed to " + command + " - " + profilerPid);

     }
  }

  public static void main(String arg[]) {
    try {
       if (arg.length==0) {
          System.out.println("Usage: java -jar profilerConsole.jar port");
          System.exit(-1);
       }
       int port = Integer.parseInt(arg[0]);
       ProfilerConsole console = new ProfilerConsole(port);
       console.doConsole();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

}