/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.console;

import java.io.*;
import java.net.*;

/**
 * Represents a Process ID to start/stop/pause processes.
 @author Clebert Suconic
*/
public class ProfilerPID  {
   ProfilerControl control;
   Socket socket;
   long pid;
   String activeCommand="stop";

   InputStream inp = null;
   OutputStream out = null;


   public ProfilerPID(Socket socket, ProfilerControl control) throws java.io.IOException {
      this.socket=socket;
      this.control=control;
      inp = socket.getInputStream();
      out = socket.getOutputStream();
      BufferedReader reader = new BufferedReader(new InputStreamReader(inp));

      // logo apos estabelecido o socket, eh enviado o PID (+ \n)
      String pidSTR = reader.readLine();
      try {
        pid = Long.parseLong(pidSTR);
      }
      catch (Exception e) {
        e.printStackTrace();
        System.exit(-1);
      }

   }


  /** executes the socket-send of the command and changes the state of this object by chaging the value
   * of activeCommand's value */
  public void executeCommand(String commandStr, int commandNumber) {
  	String oldCommandStr=this.activeCommand;
  	try {
		out.write(commandNumber);
		this.activeCommand=commandStr;
  	}
  	catch (IOException e) {
  		e.printStackTrace();
  		this.activeCommand = oldCommandStr;
  	}
  }

  public long getPid() {
    return pid;
  }

  public String getActiveCommand() {
     return activeCommand;
  }

}
