/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.threadcheck;

import java.io.*;
import java.net.*;

/** Start controller point for using inside MBeans, Consoles or to be used as an API to start/pause/stop the profiler.
@author Clebert Suconic
 */
public final class StartupProfilerCheck implements java.lang.Runnable {

   public static final int COMMAND_START=1;
   public static final int COMMAND_PAUSE=2;
   public static final int COMMAND_STOP=3;

   static {
      System.loadLibrary("jbossInspector");
   }

   private String endereco;
   int porta;
   long pid;

   private StartupProfilerCheck(String endereco, int porta, long pid) {
      this.endereco = endereco;
      this.porta = porta;
      this.pid=pid;
   }

   public void run() {
           try {
              Socket socket = new Socket(endereco,porta);
              OutputStream out = socket.getOutputStream();
              PrintStream printStream = new PrintStream(out);
              printStream.println(pid);
              out.flush();
              InputStream inp = socket.getInputStream();
              while(true) {
                int command=inp.read();
				try {
					if (command==COMMAND_START) {
						startProfiler();
					} else if (command==COMMAND_STOP) {
						stopProfiler();
						break;
					} else if (command==COMMAND_PAUSE) {
						pauseProfiler();
					}
				}
				catch (Exception e) {
					System.out.println(e.toString());
				}
              }
           }
           catch (Exception e) {
             synchronized (System.out) {
              System.out.println("Error in Profiller Startup Checker");
              e.printStackTrace();
             }
           }
   }

   /** Initialize thread for check console startups
    *  @param paramEndereco IP:PORT
    *  @param pid the PID of the process */
   public static void initCheck(String paramEndereco,long pid) {

      int doisPontos=paramEndereco.indexOf(':');
      if (doisPontos<0) {
         System.err.println("Error: Invalid Addres " + paramEndereco);
         System.err.println("Valid format is <IP>:<PORT>");
         return;
      }

      String endereco=paramEndereco.substring(0,doisPontos);
      String porta = paramEndereco.substring(doisPontos+1);

      System.out.println("endereco = " + endereco);
      System.out.println("porta = " + porta);

      Thread thread = new Thread(new StartupProfilerCheck(endereco,Integer.parseInt(porta),pid));
      thread.start();
   }

   private static final int STOPPED=0;
   private static final int RUNNING=1;
   private static final int PAUSED=2;
   private static final int DETTACHED=3;
   static int profilerState=STOPPED;

   public static void startProfiler() {
	   if (profilerState==DETTACHED) {
		   throw new RuntimeException ("Can't start profiler - dettached state");
	   }

	   if (profilerState!=RUNNING) {
		   startProfilerInternal();
		   profilerState=RUNNING;
	   }
   }

   public static void stopProfiler() {
	   if (profilerState==DETTACHED) {
		   throw new RuntimeException ("Can't start profiler - dettached state");
	   }
	   if (profilerState!=STOPPED) {
		   stopProfilerInternal();
		   profilerState=STOPPED;
	   }
   }

   public static void pauseProfiler() {
	   if (profilerState!=DETTACHED) {
		   pauseProfilerInternal();
		   profilerState=PAUSED;
	   }
   }

   private native static void startProfilerInternal();
   private native static void pauseProfilerInternal();
   private native static void stopProfilerInternal();

}
