/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.engine;

import org.jboss.profiler.fileProcessor.*;
import org.jboss.profiler.model.*;
import org.jboss.profiler.util.SPYConsts;
import java.io.*;
import java.util.*;
import java.util.zip.*;

/**
 * This is the heart of the Disconnected Profiler. (Well, at least on the
 * Java's side as there is the native stuff) Everything is processed here. The
 * dataModel is created using this class or any of their inhiterances.
 *
  @author Clebert Suconic
 */
public class LogFileEngine implements SPYConsts {

	/** Ao extrair o log do profiller, se esta opcao estiver ligada, um
	 * arquivo por cada iteracao metodo selecionado na interface (ou seja,
	 * quando levelAuthShowRecords==1) , sera aberto um novo arquivo */
	private boolean useIteractionsOnOutput = false;

	private static int iteractionDetail = 0;

	private RunnerEvent event = new RunnerEvent(this);

	private JBPProcess process = null;
	private int counterLine = 0;
	private Vector runnerEventListeners = new Vector();

	private boolean showRecords;
	private java.io.File outputDirectory;
	private boolean processStackOnObjects = false;

	/** Para facilitar a busca em HashMaps economizando instancias */
	private JBPThread keyThread = new JBPThread();
	/** Para facilitar a busca em HashMaps economizando instancias */
	private JBPMethod keyMethod = new JBPMethod();
	/** Para facilitar a busca em HashMaps economizando instancias */
	private JBPClass keyClass = new JBPClass();
	/** Para facilitar a busca em HashMaps economizando instancias */
	private JBPObject keyObject = new JBPObject();

	private boolean processReleaseOnObjects = false;
	private boolean processStackOnMethods;
	private java.util.HashMap detailMethods;
	private java.util.HashMap resolvedDetailMethods;

	protected SpyReader reader;

	public LogFileEngine(SpyReader reader) {
		this.reader = reader;
	}

	public void doProcess(boolean pshowRecords, String pid)
		throws java.io.IOException {
		doProcess(pshowRecords, pid, 10000);
	}

	public void doProcess(boolean pshowRecords, String pid, int tickInterval)
		throws java.io.IOException {
		this.setShowRecords(pshowRecords);

		counterLine = 0;
		SpyRegister register = null;
		SpyRegister lastRegister = null;
		process = new JBPProcess();
		process.setDate(reader.getStartDate());
		process.setPid(pid);
		//process.setClocksPerSecond(reader.getClocksPerSecond());

		while ((register = reader.getBestRegister()) != null) {
			counterLine++;

			if (tickInterval > 0 && counterLine % tickInterval == 0) {
				event.setCurrentRegister(counterLine);
				fireProcessTick(event);
			}

			if (register instanceof SpyRegisterBeginThread) {
				process((SpyRegisterBeginThread) register);
			} else if (register instanceof SpyRegisterEndThread) {
				process((SpyRegisterEndThread) register);
			} else if (register instanceof SpyRegisterLoadClass) {
				process((SpyRegisterLoadClass) register);
			} else if (register instanceof SpyRegisterLoadMethod) {
				process((SpyRegisterLoadMethod) register);
			} else if (register instanceof SpyRegisterGCStart) {
				process((SpyRegisterGCStart) register);
			} else if (register instanceof SpyRegisterGCEnd) {
				process((SpyRegisterGCEnd) register);
			} else if (register instanceof SpyRegisterFreeObject) {
				process((SpyRegisterFreeObject) register);
			} else if (register instanceof SpyRegisterLoadObject) {
				process((SpyRegisterLoadObject) register);
			} else if (register instanceof SpyRegisterEnterMethod) {
				process((SpyRegisterEnterMethod) register);
			} else if (register instanceof SpyRegisterExitMethod) {
				process((SpyRegisterExitMethod) register);
			} else if (register instanceof SpyRegisterMonitorIn) {
				process((SpyRegisterMonitorIn) register);
			} else if (register instanceof SpyRegisterMonitorOut) {
				process((SpyRegisterMonitorIn) register);
			} else if (register instanceof SpyRegisterContended) {
				process((SpyRegisterContended) register);
			} else if (register instanceof SpyRegisterRawContended) {
				process((SpyRegisterRawContended) register);
			}

			lastRegister = register;

			reader.readNextRegister();
		}

		closeThreads();

		if (lastRegister != null) {
			process.setFinishDate(lastRegister.getTimestamp());
			sumTotalClocks(process);
		}

		this.processGCS();

	}

	private void sumTotalClocks(JBPProcess process) {
		Iterator iter = process.getSpyThreads().values().iterator();

		long totalClocks = 0;
		long totalElapsedClocks = 0;
		while (iter.hasNext()) {
			JBPThread loopThread = (JBPThread) iter.next();
			/*Iterator iterRoot = loopThread.getSpyMethodCountRoots().values().iterator();
			while (iterRoot.hasNext()) {
			   JBPMethodCount count = (JBPMethodCount)iterRoot.next();
			   totalClocks+=count.getTotalClocks();
			   totalElapsedClocks+=count.getTotalElapsedClock();
			}*/
			if (loopThread.getFinishClock() >= 0
				&& loopThread.getStartClock() >= 0) {
				long tmpClock =
					(loopThread.getFinishClock() - loopThread.getStartClock());
				if (tmpClock < 0) {
					tmpClock = tmpClock * -1;
				}
				totalClocks += tmpClock;
			}
			long tmpElapsed =
				(loopThread.getElapsedClockFinish()
					- loopThread.getElapsedClockStart());
			totalElapsedClocks += tmpElapsed;

		}

		process.setTotalClocks(totalClocks);
		process.setTotalElapsedClocks(totalElapsedClocks);
	}

	/** Get all methods and put then into global information at Process */
	private void closeThreads() {
		Iterator threadsIterator = process.getSpyThreads().values().iterator();
		while (threadsIterator.hasNext()) {
			JBPThread thread = (JBPThread) threadsIterator.next();;
			threadFakeCloseThread(thread);
			Iterator enumMethods = thread.getSpyMethods().values().iterator();
			while (enumMethods.hasNext()) {
				JBPMethod spyMethod = (JBPMethod) enumMethods.next();
				JBPMethod foundMethod =
					(JBPMethod) process.getSpyMethods().get(spyMethod);
				if (foundMethod == null) {
					System.err.println(
						"Error on closeThreads: MethodID "
							+ spyMethod.getMethodID()
							+ " not found");
					System.out.println(
						"Error on closeThreads: MethodID "
							+ spyMethod.getMethodID()
							+ " not found");
					continue;
				}
				foundMethod.sum(spyMethod);
				foundMethod.addThread(thread, spyMethod);
			}
			if (thread.getDetailedOutput() != null) {
				thread.getDetailedOutput().close();
			}
		}
	}

	protected JBPThread lookupThreadByThreadId(
		int pthreadId,
		HashMap spyThreads,
		SpyRegister register) {
		keyThread.setThreadId(pthreadId);
		JBPThread spyThread = (JBPThread) spyThreads.get(keyThread);
		if (spyThread == null) {
			JBPThread newThread = new JBPThread();
			newThread.setThreadId(pthreadId);
			newThread.setName("Undefined");
			newThread.setClosed(false);
			//newThread.setStartTimestamp(register.getTimestamp());
			//newThread.setStartClock(register.getClock());
			spyThreads.put(newThread, newThread);

			return newThread;
		}

		//if (spyThread.getStartClock()<0) spyThread.setStartClock(register.getClock());

		return spyThread;
	}

	protected void process(SpyRegisterMonitorIn register) throws IOException {
        waitStart(register,register.getThreadId(),register.getObjectId());
    }

    private void waitStart(SpyRegister register, int threadId, int objectId) throws IOException {
        HashMap spyThreads = process.getSpyThreads();

        JBPThread spyThread =
            lookupThreadByThreadId(
                threadId,
                spyThreads,
                register);

        if (spyThread.getSpyCalls().isEmpty())
        {
        	return;
        }
        JBPCall spyCall = (JBPCall) spyThread.getSpyCalls().peek();
        if (spyCall == null) {
            System.out.println(
                "Warning (MonitoringIn) - spyCall is null at Thread "
                    + spyThread);
            return;
        }
        spyCall.getSpyMethod().setLastClockMonitoring(register.getElapsedClock());

        if (isShowRecords(spyThread) && spyThread != null) {
            doCommonOutput(spyThread, register);
            if (spyThread.getDetailedOutput() != null)
                spyThread.getDetailedOutput().println(
                    "MonitorIn at \""
                        + spyThread.getName()
                        + " on objectId = "
                        + objectId);
        }
    }

    protected void process(SpyRegisterMonitorOut register) throws IOException {
        waitStop(register,register.getThreadId(),register.getObjectId());
    }

    private void waitStop(SpyRegister register,int threadId, long objectId) throws IOException {
        HashMap spyThreads = process.getSpyThreads();

        JBPThread spyThread =
            lookupThreadByThreadId(
                threadId,
                spyThreads,
                register);

        if (spyThread.getSpyCalls().isEmpty())
        {
            return;
        }

        JBPCall spyCall = (JBPCall) spyThread.getSpyCalls().peek();
        if (spyCall == null) {
            System.out.println(
                "Warning - spyCall is null at Thread " + spyThread);
            return;
        }
        if (spyCall.getSpyMethod().getLastClockMonitoring() == 0) {
            System.out.println(
                "Warning (MonitoringOut) - lastClockMonitoring is empty  at Thread "
                    + spyThread);
            return;
        }
        long clockMonitoring =
            register.getElapsedClock()
                - spyCall.getSpyMethod().getLastClockMonitoring();

        spyThread.findMethodPath();



        spyCall.getSpyMethod().setLastClockMonitoring(0);
        Iterator iter = spyThread.getSpyCalls().iterator();
        while (iter.hasNext()) {
            JBPCall call = (JBPCall) iter.next();
            call.getSpyMethod().addClockInWait(clockMonitoring);
            JBPMethod methodOnThread = (JBPMethod)spyThread.getSpyMethods().get(call.getSpyMethod());
            JBPMethod methodOnProcess = (JBPMethod) process.getSpyMethods().get(keyMethod);
            //methodOnThread.addClockInWait(clockMonitoring);
            if (call.getSpyMethodCount()!=null)
            {
                call.getSpyMethodCount().addWaitTime(clockMonitoring,1);
            }
        }

        if (isShowRecords(spyThread) && spyThread != null) {
            doCommonOutput(spyThread, register);
            if (spyThread.getDetailedOutput() != null)
                spyThread.getDetailedOutput().println(
                    "MonitorOut at \""
                        + spyThread.getName()
                        + " on objectId = "
                        + objectId);
        }


    }

    static int locks=0;

    protected void process(SpyRegisterContended register) throws IOException {

        if (register.getContendedType()==SpyRegisterContended.CONTENDED_ENTER)
        {
            waitStart(register,register.getThreadId(),register.getObjectId());
        } else
        if (register.getContendedType()==SpyRegisterContended.CONTENDED_ENTERED)
        {
            waitStop(register,register.getThreadId(),register.getObjectId());
        }

    }

	protected void process(SpyRegisterRawContended register)
		throws IOException {
		if (isShowRecords()) {
			HashMap spyThreads = process.getSpyThreads();

			JBPThread spyThread =
				lookupThreadByThreadId(
					register.getThreadId(),
					spyThreads,
					register);
			if (spyThread == null)
				return;

			doCommonOutput(spyThread, register);
			String strtype = null;
			if (register.getContendedType()
				== SpyRegisterContended.CONTENDED_ENTER) {
				strtype = "Enter";
				spyThread.getDetailedOutput().print("<font color=\"yellow\">");
			} else if (
				register.getContendedType()
					== SpyRegisterContended.CONTENDED_ENTERED) {
				strtype = "Entered";
				spyThread.getDetailedOutput().print("<font color=\"red\">");
			} else if (
				register.getContendedType()
					== SpyRegisterContended.CONTENDED_ENTERED) {
				strtype = "Exit";
				spyThread.getDetailedOutput().print("<font color=\"green\">");
			}
			spyThread.getDetailedOutput().println(
				"ContendedRawMonitor "
					+ strtype
					+ " at \""
					+ spyThread.getName()
					+ " monitorName = "
					+ register.getMonitorName()
					+ "rawId = "
					+ register.getRawId()
					+ "</font>");
		}
	}

	void setDataThreadStart(
		JBPThread thread,
		Date timestamp,
		long clock,
		long elapsedTime) {
		thread.setStartClock(clock);
		thread.setElapsedClockStart(elapsedTime);
		thread.setStartTimestamp(timestamp);
	}

    /** What was the latest clocks*/
	void setDataThreadEnd(
		JBPThread thread,
		long time_t,
		Date timestamp,
		long clock,
		long elapsedTime) {

		thread.setLastTime_T(time_t);
		thread.setFinishTimestamp(timestamp);
		thread.setFinishClock(clock);
		thread.setElapsedClockFinish(elapsedTime);
	}

	protected void process(SpyRegisterEnterMethod register)
		throws IOException {
		HashMap spyClasses = process.getSpyClasses();
		HashMap spyMethods = process.getSpyMethods();
		HashMap spyThreads = process.getSpyThreads();
		keyMethod.setMethodID(register.getMethodId());
		keyClass.setClassID(register.getClassId());

		JBPThread spyThread =
			lookupThreadByThreadId(
				register.getThreadId(),
				spyThreads,
				register);
		if (spyThread == null) {
			System.err.println(
				"Thread " + register.getThreadId() + " not found");
			return;
		}

		setDataThreadEnd(
			spyThread,
			register.getTime_t(),
			register.getTimestamp(),
			register.getClock(),
			register.getElapsedClock());


		if (spyThread.getStartClock() < 0) {
			setDataThreadStart(
				spyThread,
				register.getTimestamp(),
				register.getClock(),
				register.getElapsedClock());
		}

		// spyMethod utilizado no processo inteiro
		JBPMethod spyMethodProcess = (JBPMethod) spyMethods.get(keyMethod);

		if (this.showRecords) {
			if (spyMethodProcess != null
				&& this.getResolvedDetailMethods().get(spyMethodProcess) != null) {
				// aumenta nivel de metodo, para fazer saida individual
				spyThread.addLevelAuthShow();

				if (this.isUseIteractionsOnOutput()) {
					if (spyThread.getDetailedOutput() != null) {
						spyThread.getDetailedOutput().close();
						spyThread.setDetailedOutput(null);
						spyThread.setDetailedOutputFile(null);
					}
				}

			}
		}

		if (spyMethodProcess == null) {
			JBPClass spyClass = (JBPClass) spyClasses.get(keyClass);
			if (spyClass == null) {
				spyClass = new JBPClass();
				spyClass.setClassID(register.getClassId());
				spyClass.setName("Unknown");
				spyClasses.put(spyClass, spyClass);
			}

			spyMethodProcess = new JBPMethod();
			spyMethodProcess.setMethodID(register.getMethodId());
			spyMethodProcess.setName("Unknown");
			spyMethodProcess.setSignature("Unknown");
			spyMethodProcess.setSpyClass(spyClass);

			spyClass.getSpyMethods().put(spyMethodProcess, spyMethodProcess);
			spyClasses.put(spyClass, spyClass);
			spyMethods.put(spyMethodProcess, spyMethodProcess);

			//System.out.println("MethodId " + register.getMethodId() + " not found");

			//fireProcessWarning("WARNING: MethodId = " + register.getMethodId() + " not found");
			//return;
		}

		/*if (isProcessStackOnObjects()) {
		  java.util.Stack spyCalls = spyThread.getSpyCalls();
		  if (!spyCalls.empty()) {
		    JBPMethod callingMethod = ((JBPCall)spyCalls.peek()).getSpyMethod();
		    callingMethod = (JBPMethod)spyMethods.get(callingMethod);
		    if (callingMethod!=null) {
		        JBPGenericCount methodCount = (JBPGenericCount) callingMethod.getSpyMethodsCalling().get(spyMethod);
		        if (methodCount==null) {
		            methodCount = new JBPGenericCount(spyMethod);
		            callingMethod.getSpyMethodsCalling().put(spyMethod,methodCount);
		        }
		        methodCount.add();
		    }
		  }
		} */

		JBPMethod spyMethodThread =
			(JBPMethod) spyThread.getSpyMethods().get(keyMethod);

		if (spyMethodThread == null) {
			spyMethodThread = (JBPMethod) spyMethodProcess.clone();
			spyThread.getSpyMethods().put(spyMethodThread, spyMethodThread);
		}

		threadopenCall(
			spyThread,
			spyMethodThread,
			spyMethodProcess,
			register.getTime_t(),
			register.getClock(),
			register.getElapsedClock());

		if (isShowRecords(spyThread) && spyThread != null) {
			doCommonOutput(spyThread, register);
			String strMethod = null;
			if (spyMethodProcess != null) {
				strMethod =
					((spyMethodProcess.getSpyClass() != null)
						? (spyMethodProcess.getSpyClass().getName())
						: "<Unknow>")
						+ "::"
						+ spyMethodProcess.getName();
			} else {
				strMethod = "Undefined -> methodId = " + register.getMethodId();
			}
			if (spyThread.getDetailedOutput() != null) {
				spyThread.getDetailedOutput().println(
					"Method \"" + strMethod + " called");
			}
		}

	}

	protected void process(SpyRegisterExitMethod register) throws IOException {
		HashMap spyClasses = process.getSpyClasses();
		HashMap spyMethods = process.getSpyMethods();
		HashMap spyThreads = process.getSpyThreads();

		JBPThread spyThread =
			lookupThreadByThreadId(
				register.getThreadId(),
				spyThreads,
				register);
		if (spyThread == null) {
			System.err.println(
				"Thread " + register.getThreadId() + " not found");
			return;
		}

		setDataThreadEnd(
			spyThread,
			register.getTime_t(),
			register.getTimestamp(),
			register.getClock(),
			register.getElapsedClock());

		keyMethod.setMethodID(register.getMethodId());
		JBPMethod spyMethodProcess = (JBPMethod) spyMethods.get(keyMethod);

		if (this.showRecords) {
			if (spyMethodProcess != null
				&& this.getResolvedDetailMethods().get(spyMethodProcess) != null) {
				spyThread.setLevelAuthShowRecords(-1);
			}
		}

		if (isShowRecords(spyThread) && spyThread != null) {
			doCommonOutput(spyThread, register);
			String strMethod = null;
			if (spyMethodProcess != null) {
				strMethod =
					((spyMethodProcess.getSpyClass() != null)
						? (spyMethodProcess.getSpyClass().getName())
						: "<Unknow>")
						+ "::"
						+ spyMethodProcess.getName();
			} else {
				strMethod = "Undefined -> methodId = " + register.getMethodId();
			}
			if (spyThread.getDetailedOutput() != null) {
				spyThread.getDetailedOutput().println(
					"Method \"" + strMethod + " exit");
			}
		}

		if (spyMethodProcess == null) {
			spyMethodProcess = new JBPMethod();
			spyMethodProcess.setMethodID(register.getMethodId());
			spyMethodProcess.setName("Unknow");
			spyMethodProcess.setSignature("Unknow");
			spyMethods.put(spyMethodProcess, spyMethodProcess);
		}

		JBPMethod spyMethodThread =
			(JBPMethod) spyThread.getSpyMethods().get(keyMethod);

		if (spyMethodThread == null) {
			spyMethodThread = (JBPMethod) spyMethodProcess.clone();
			spyThread.getSpyMethods().put(spyMethodThread, spyMethodThread);
		}

		threadcloseCall(
			spyThread,
			spyMethodThread,
			spyMethodProcess,
			register.getTime_t(),
			register.getClock(),
			register.getElapsedClock());
		spyThread.setFinishClock(register.getClock());
		spyThread.setElapsedClockFinish(register.getElapsedClock());
		//spyThread.setFinishTimestamp(register.getTimestamp());

	}

	protected void process(SpyRegisterLoadObject register) throws IOException {
		if (register.getClassId() == 0)
			return;

		HashMap processClasses = process.getSpyClasses();
		JBPClass spyClass = null;
		keyClass.setClassID(register.getClassId());
		spyClass = (JBPClass) processClasses.get(keyClass);
		if (spyClass == null) {
			//System.err.println("Class " + register.getClassId() + " not found em loadObject");
			spyClass = new JBPClass();
			spyClass.setName("Undefined");
			spyClass.setClassID(register.getClassId());
			processClasses.put(spyClass, spyClass);
			//return;
		}

		JBPThread spyThread =
			lookupThreadByThreadId(
				register.getThreadId(),
				process.getSpyThreads(),
				register);
		if (spyThread == null) {
			System.err.println(
				"Thread " + register.getThreadId() + " not found");
			return;
		}

		/*if (spyThread.getStartClock()<0) {
		   setDataThreadStart(spyThread,register.getClock(),register.getElapsedClock());
		} */

		JBPMemoryState currentState = process.getCurrentMemoryState();

		JBPMemoryClassCount memoryCount =
			(JBPMemoryClassCount) currentState.getInUse().get(spyClass);
		if (memoryCount == null) {
			memoryCount = new JBPMemoryClassCount(spyClass);
			currentState.getInUse().put(spyClass, memoryCount);
		}

		memoryCount.addSize(register.getSize());

		HashMap threadCountHash = memoryCount.getThreadCount();
		JBPMemoryCount spyThreadCount =
			(JBPMemoryCount) threadCountHash.get(spyThread);
		if (spyThreadCount == null) {
			spyThreadCount = new JBPMemoryCount(spyThread);
			threadCountHash.put(spyThread, spyThreadCount);
		}
		spyThreadCount.addSize(register.getSize());

		if (processReleaseOnObjects) {
			JBPObject spyObject =
				new JBPObject(
					spyClass.getClassID(),
					register.getObjectId(),
					register.getSize());

			int[] methodPath = createPathForObjectCreation(spyThread);

            spyObject.setGraphPathCreation(methodPath);
			spyObject.setThreadIdCreated(spyThread.getThreadId());
			process.getSpyObjects().add(spyObject);

			process.summarizeObject(spyObject,keyMethod,keyThread,keyClass,false);
		}

		// Objetos criados em stack trace
		if (processStackOnMethods) {
			JBPMethodCount metCount = spyThread.findMethodPath();
			if (metCount != null) {
				JBPMemoryCount memCount = metCount.getCountForClass(spyClass);
				memCount.addSize(register.getSize());
			}


		}

		// Objetos criados, visao geral
		/*
		 * -- Feature desativada, em favor da arvore de chamadas mais complexa
		 if (processStackOnObjects) {
			java.util.Stack stackCalls = spyThread.getSpyCalls();
			for (int count = 1, i = stackCalls.size() - 1;
				i >= 0;
				i--, count++) {
				JBPCall spyCall = (JBPCall) stackCalls.get(i);
				ArrayList arrayCount = memoryCount.getMethodArrayCountLevel();
				for (int currSize = arrayCount.size();
					currSize < count;
					currSize++) {
					arrayCount.add(new HashMap());
				}

				processMethodStackTrace(
					spyCall.getSpyMethod(),
					(HashMap) arrayCount.get(count - 1),
					register.getSize());
			}
		}*/

		//java.util.Enumeration enumCalls = spyThread.getSpyCalls().elements();

		if (isShowRecords(spyThread) && spyThread != null) {
			doCommonOutput(spyThread, register);
			if (spyThread.getDetailedOutput() != null) {
				spyThread.getDetailedOutput().println(
					"Instance of \"" + spyClass.getName() + " created");
			}
		}

	}

	/** Extract graph-path on object creation.
	 * Method used when loading objects */
	private int[] createPathForObjectCreation(JBPThread spyThread) {
		int graphSize = spyThread.getSpyCalls().size();

		int methodPath[] = new int[graphSize];
		for (int i=spyThread.getSpyCalls().size() - graphSize,j=0;
		     i<spyThread.getSpyCalls().size();
		     i++,j++) {
		     	JBPCall call = (JBPCall)spyThread.getSpyCalls().get(i);
		     	methodPath[j]=call.getSpyMethod().getMethodID();
		     }
		return methodPath;
	}

	/* Processa a contagem de stack em Metodo
	 * Feature desativada em favor da nova forma de visualizacao dos objetos criados
	private void processMethodStackTrace(
		JBPMethod spyMethod,
		HashMap methodCountHash,
		int size) {
		JBPMemoryCount methodCount =
			(JBPMemoryCount) methodCountHash.get(spyMethod);
		if (methodCount == null) {
			methodCount = new JBPMemoryCount(spyMethod);
			methodCountHash.put(spyMethod, methodCount);
		}
		methodCount.addSize(size);
	} */

	protected void process(SpyRegisterFreeObject register) {
		/* if (processReleaseOnObjects) {
			keyObject.setObjectId(register.getObjectId());

			SPYObject spyObject =
				(SPYObject) process.getSpyObjects().get(keyObject);
			if (spyObject != null) {
				SPYClass spyClass = process.findClass(spyObject.getClassId(),keyClass);
				JBPMemoryClassCount classCount =
					(JBPMemoryClassCount) process
						.getCurrentMemoryState()
						.getInUse()
						.get(spyClass);
				classCount.addRelease(spyObject.getBytesAllocated());
				process.getSpyObjects().remove(spyObject);

				process.summarizeObject(spyObject,keyMethod,keyThread,keyClass , true);
			}


			//SPYObject spyObject = new SPYObject(spyClass,register.getObjectId());
			//process.getSpyObjects().put(spyObject,spyObject)
		} */
	    if (processReleaseOnObjects) {
			JBPObjectInstance gcInstance = new JBPObjectInstance(process.getNumberOfGarbageCollects(),register.getObjectId());
			process.getSpyObjectsGC().add(gcInstance);
	    }
	}

	protected void processGCS() {
		try {
			System.out.print("sort - ");
			Object [] objs = process.getSpyObjectsGC().toArray();
			Arrays.sort(objs);
			for (int i=0;i<objs.length;i++) {
				((List)process.getSpyObjectsGC()).set(i,objs[i]);
			}
			System.out.println("ok");

			JBPObjectInstance gcInstanceKey = new JBPObjectInstance();
			Iterator iterObjs = process.getSpyObjects().iterator();
			while (iterObjs.hasNext()) {
				JBPObject spyObject = (JBPObject)iterObjs.next();
				gcInstanceKey.setObjectId(spyObject.getObjectId());
				int local = Arrays.binarySearch(objs,gcInstanceKey);
				if (local>=0 && spyObject!=null) {
					processGC((JBPObjectInstance)objs[local], spyObject);

				}
			}
		}
		catch (Exception e) {
		  e.printStackTrace();
		}
	}

	private void processGC(JBPObjectInstance gcInstance, JBPObject spyObject) {
		JBPClass spyClass = process.findClass(spyObject.getClassId(),keyClass);
		JBPMemoryClassCount classCount =
			(JBPMemoryClassCount) process
				.getMemoryState(gcInstance.getGcNumber())
				.getInUse()
				.get(spyClass);
		classCount.addRelease(spyObject.getBytesAllocated());
		//process.getSpyObjects().remove(spyObject);

		process.summarizeObject(spyObject,keyMethod,keyThread,keyClass , true);
	}

	protected void process(SpyRegisterGCStart register) {
		JBPMemoryState currentState = process.getCurrentMemoryState();
		currentState.setCpuStartGarbage(register.getClock());
		currentState.setStartGarbage(register.getTimestamp());
	}

	protected void process(SpyRegisterGCEnd register) {
		JBPMemoryState currentState = process.getCurrentMemoryState();
		currentState.setCpuEndGarbage(register.getClock());
		currentState.setEndGarbage(register.getTimestamp());
		JBPMemoryState newState = (JBPMemoryState) currentState.clone();
		newState.setCpuEndGarbage(0);
		newState.setCpuStartGarbage(0);
		newState.setStartGarbage(null);
		newState.setEndGarbage(null);
		process.getMemoryStates().add(newState);
	}

	private void printDebugMemoryState(ArrayList memoryStates) {
		Iterator iter = memoryStates.iterator();
		System.out.println("------------------------------------");
		System.out.println("Memory State Debug Detail");
		int i = 0;
		while (iter.hasNext()) {
			System.out.println("****** State " + (i++) + " *****");
			JBPMemoryState state = (JBPMemoryState) iter.next();
			Iterator iterClass = state.getInUse().values().iterator();
			while (iterClass.hasNext()) {
				JBPMemoryCount count = (JBPMemoryCount) iterClass.next();
				if (count.getCountRelease() > 0)
					System.out.println(
						"class = "
							+ count.getReferencedObject()
							+ " count = "
							+ count.getCount()
							+ " release = "
							+ count.getCountRelease());
			}
		}
	}

	protected void process(SpyRegisterLoadClass register) {
		boolean bfound = true;
		HashMap processClasses = process.getSpyClasses();
		JBPClass spyClass = null;
		keyClass.setClassID(register.getClassId());
		spyClass = (JBPClass) processClasses.get(keyClass);
		if (spyClass == null) {
			spyClass = new JBPClass();
			bfound = false;
		}
		spyClass.setData(register);
		if (!bfound)
			processClasses.put(spyClass, spyClass);
	}

	protected void process(SpyRegisterLoadMethod register) {
		HashMap processMethods = process.getSpyMethods();
		HashMap processClasses = process.getSpyClasses();
		keyClass.setClassID(register.getClassId());
		JBPClass spyClass = (JBPClass) processClasses.get(keyClass);
		if (spyClass == null) {
			fireProcessWarning(
				"WARNING: ClassID " + register.getClassId() + " not found");
		}
		keyMethod.setMethodID(register.getMethodId());
		JBPMethod spyMethod = (JBPMethod) processMethods.get(keyMethod);
		if (spyMethod == null) {
			spyMethod = new JBPMethod();
			processMethods.put(spyMethod, spyMethod);
		}
		spyMethod.setData(register);
		spyMethod.setSpyClass(spyClass);
		processMethods.put(spyMethod, spyMethod);

		if (spyClass != null) {
			JBPMethod methodOnClass =
				(JBPMethod) spyClass.getSpyMethods().get(spyMethod);
			if (methodOnClass == null) {
				spyClass.getSpyMethods().put(spyMethod, spyMethod);
			} else {
				methodOnClass.setData(register);
			}
		}

		/** Efetua resolucao dos metodos que deveram ser detalhados em output*/
		if (this.getDetailMethods() != null) {
			String methodSignature =
				spyClass.getName()
					+ "."
					+ spyMethod.getName()
					+ "("
					+ spyMethod.getSignature()
					+ ")";
			if (this.getDetailMethods().get(methodSignature) != null) {
				this.getDetailMethods().remove(methodSignature);
				if (this.getDetailMethods().isEmpty()) {
					this.detailMethods = null;
				}
				this.getResolvedDetailMethods().put(spyMethod, spyMethod);
			}
		}
	}

	protected void process(SpyRegisterBeginThread register)
		throws java.io.IOException {
		HashMap processThreads = process.getSpyThreads();
		JBPThread spyThread = null;

		keyThread.setThreadId(register.getThreadId());
		spyThread = (JBPThread) processThreads.get(keyThread);
		if (spyThread == null) {
			spyThread = new JBPThread();
		}

		spyThread.setData(register);

		processThreads.put(spyThread, spyThread);

		if (isShowRecords(spyThread) && spyThread != null) {
			doCommonOutput(spyThread, register);
			if (spyThread.getDetailedOutput() != null)
				spyThread.getDetailedOutput().println(
					"Thread \"" + spyThread.getName() + "\" Started");
		}
	}

	protected void doCommonOutput(JBPThread spyThread, SpyRegister register)
		throws java.io.IOException {
		verifyOutput(spyThread);
		PrintStream out = spyThread.getDetailedOutput();
		out.print(
			"<br>" + register.getClock() + " at " + register.getTimestamp());
		int nCall = spyThread.getSpyCalls().size();
		for (int i = 0; i < nCall; i++) {
			out.print("---");
		}

	}

	protected void verifyOutput(JBPThread spyThread)
		throws java.io.IOException {
		if (isShowRecords() && spyThread != null) {
			if (spyThread.getDetailedOutput() == null) {
				File threadOutput = null;
				if (useIteractionsOnOutput) {
					threadOutput =
						new File(
							outputDirectory,
							"ThreadOutput"
								+ spyThread.getThreadId()
								+ "_iteraction_"
								+ (iteractionDetail++)
								+ ".html.gz");
				} else {
					threadOutput =
						new File(
							outputDirectory,
							"ThreadOutput"
								+ spyThread.getThreadId()
								+ ".html.gz");
				}
				PrintStream printStream =
					new PrintStream(
						new GZIPOutputStream(
							new FileOutputStream(threadOutput)));
				spyThread.setDetailedOutputFile(threadOutput);
				spyThread.setDetailedOutput(printStream);
				printStream.println(
					"<html><title>Thread id = "
						+ spyThread.getThreadId()
						+ "</title><body>");
			}
		}
	}

	protected void process(SpyRegisterEndThread register) throws IOException {
		HashMap processThreads = process.getSpyThreads();
		JBPThread spyThread =
			lookupThreadByThreadId(
				register.getThreadId(),
				process.getSpyThreads(),
				register);
		if (spyThread == null) {
			fireProcessWarning(
				"WARNING: Cannot found Thread threadId = "
					+ register.getThreadId());
			return;
		}

		spyThread.setData(register);

		threadcloseThread(
			spyThread,
			register.getTime_t(),
			register.getClock(),
			register.getElapsedClock());

		if (isShowRecords(spyThread) && spyThread != null) {
			doCommonOutput(spyThread, register);
			spyThread.getDetailedOutput().println(
				"Thread \"" + spyThread.getName() + "\" End");
		}

	}

	public JBPProcess getProcess() {
		return process;
	}

	public void removeRunnerEventListener(RunnerEventListener l) {
		synchronized (runnerEventListeners) {
			if (runnerEventListeners.contains(l)) {
				runnerEventListeners.removeElement(l);
			}
		}
	}

	public void addRunnerEventListener(RunnerEventListener l) {
		synchronized (runnerEventListeners) {
			if (!runnerEventListeners.contains(l)) {
				runnerEventListeners.addElement(l);
			}
		}
	}

	protected void fireProcessTick(RunnerEvent e) {
		Vector listeners = (Vector)runnerEventListeners.clone();
		int count = listeners.size();
		for (int i = 0; i < count; i++) {
			((RunnerEventListener) listeners.elementAt(i)).processTick(e);
		}
	}

	protected void fireProcessWarning(String e) {
		Vector listeners = (Vector)runnerEventListeners.clone();
		int count = listeners.size();
		for (int i = 0; i < count; i++) {
			((RunnerEventListener) listeners.elementAt(i)).processWarning(
			counterLine,
			e);
		}
	}

	public void threadcloseThread(
		JBPThread thread,
		long time_t,
		long clock,
		long elapsedClock) {

		java.util.Stack spyCalls = thread.getSpyCalls();
		while (!spyCalls.empty()) {
			JBPCall currentCall = (JBPCall) spyCalls.peek();
			threadcloseCall(
				thread,
				currentCall.getSpyMethod(),
				null,
				time_t,
				clock,
				elapsedClock);
		}
		thread.setClosed(true);
	}

	/** This method does exactly what threadCloseThread does, but it keep all methods at spyCalls.
	 * That's to keep thread's view working and showing stack Tracke properly */
	public void threadFakeCloseThread(
		JBPThread thread) {

		java.util.Stack spyCalls = thread.getSpyCalls();
		java.util.Stack backupStack = new java.util.Stack();
		backupStack.addAll(spyCalls);

		while (!spyCalls.empty()) {
			JBPCall currentCall = (JBPCall) spyCalls.peek();
			threadcloseCall(
				thread,
				currentCall.getSpyMethod(),
				null,
				thread.getLastTime_T(),
				thread.getFinishClock(),
			    thread.getElapsedClockFinish());
		}
		//thread.setClosed(true);
		thread.getSpyCalls().empty();
		thread.getSpyCalls().addAll(backupStack);
	}



	/**
	 * Adiciona uma chamada na pilha de chamadas, para contagem posterior.
	 * @param spyMethodOnThread O metodo consumindo recursos no Thread
	 * @param spyMethodOnProcess O metodo consumindo recursos no processo (sumarizacao de todas as threads)
	 */
	protected void threadopenCall(
		JBPThread thread,
		JBPMethod spyMethodOnThread,
		JBPMethod spyMethodOnProcess,
		long time_t,
		long clock,
		long elapsedClock) {

		/*if (!thread.getSpyCalls().isEmpty()) {
		  doCrossMethods(thread.getSpyCalls(), spyMethodOnThread, spyMethodOnProcess, (JBPCall)thread.getSpyCalls().peek(),false);
		} */

		// para evitar contagem de metodos recursivos
		spyMethodOnThread.addCallingOnThread();

		JBPCall spyCall = new JBPCall();
		spyCall.setSpyMethod(spyMethodOnThread);
		spyCall.setMethodId(spyMethodOnThread.getMethodID());
		spyCall.setTimeTStart(time_t);
		spyCall.setClockStart(clock);
		spyCall.setElapsedClockStart(elapsedClock);
		thread.getSpyCalls().push(spyCall);

		spyMethodOnProcess.setSumDeep(
			spyMethodOnProcess.getSumDeep() + thread.getSpyCalls().size());
		if (thread.getSpyCalls().size() > spyMethodOnProcess.getMaxDeep()) {
			spyMethodOnProcess.setMaxDeep(thread.getSpyCalls().size());
		}

		// stack trace das chamadas
		//if (processStackOnMethods) {
		//	java.util.Stack stackCalls = thread.getSpyCalls();
		//}
	}

	/** Remove chamadas na pilha, calculando o tempo de execucao do metodo.
	 */
	public void threadcloseCall(
		JBPThread thread,
		JBPMethod spyMethodOnThread,
		JBPMethod spyMethodOnProcess,
		long time_t,
		long clock,
		long elapsedClock) {
		if (spyMethodOnProcess == null) {
			spyMethodOnProcess =
				(JBPMethod) process.getSpyMethods().get(spyMethodOnThread);
			if (spyMethodOnProcess == null) {
				System.out.println(
					"spyMethodOnProcess cannot be null at threadcloseCall. There is something wrong on the process");
				throw new RuntimeException("spyMethodOnProcess cannot be null at threadcloseCall. There is something wrong on the process");
			}
		}

		java.util.Stack spyCalls = thread.getSpyCalls();
		if (spyCalls.empty()) {
			return;
		}
		JBPCall spyCall = (JBPCall) spyCalls.peek();
		if (spyCall.getMethodId() != spyMethodOnThread.getMethodID()) {
			System.err.println(
				"WARNING on JBPCall: Cannot synchronize correctly methods calls");
			System.out.println(
				"WARNING on JBPCall: Cannot synchronize correctly methods calls");
			//spyCalls.pop();
			return;
		}

		try {
			spyCall.setClockFinish(clock);
			spyCall.setTimeTFinish(time_t);
			spyCall.setElapsedClockEnd(elapsedClock);

			JBPCall lastCall = null;
			if (spyCalls.size() > 1) {
				lastCall = (JBPCall) spyCalls.get(spyCalls.size() - 2);
			}

			if (spyCall.getClockStart() > spyCall.getClockFinish()) {
				System.out.println("WARNING: Invalid Start Clock");
			}

			if (spyMethodOnThread.getCurrentCallings() <= 1) {
				spyMethodOnThread.addTotalClock(
					spyCall.getClockFinish() - spyCall.getClockStart());
				spyMethodOnThread.addNumberOfCalls(1);
				spyMethodOnThread.addTotalElapsedClock(
					spyCall.getElapsedClockEnd()
						- spyCall.getElapsedClockStart());
			}

			//doCrossMethods(spyCalls, spyMethodOnThread, spyMethodOnProcess, lastCall,spyMethodOnThread.getCurrentCallings()<=1);
			if (lastCall != null) {
				stabilishReference(
					thread,
					(JBPMethod) process.getSpyMethods().get(
						lastCall.getSpyMethod()),
					spyCall,
					spyMethodOnProcess,
					true);
			}
            /* else {
                   // tratamento de metodos raiz
                   if (processStackOnMethods) {
                       JBPMethodCount spyRootCount =
                           (JBPMethodCount) thread.getSpyMethodCountRoots().get(
                               spyMethodOnThread);
                   }
               } */

			spyMethodOnThread.releaseCallingOnThread();
		} finally {
			JBPCall spyCallOnPop = (JBPCall) spyCalls.pop();
			if (spyCalls.isEmpty()) {
				if (processStackOnMethods) {
					JBPMethodCount spyRootCount =
						(JBPMethodCount) thread.getSpyMethodCountRoots().get(
							spyMethodOnThread);
					if (spyRootCount != null) {
						spyRootCount.addTime(
							spyCallOnPop.getClockFinish()
								- spyCallOnPop.getClockStart(),
							spyCallOnPop.getElapsedClockEnd()
								- spyCallOnPop.getElapsedClockStart());
					}
				}
			}
		}
	}

	/**
	 * Estabelece cruzamento entre os metodos baseando-se no stack-trace
	 * @param methodOnThread metodo executado, com referencia no Thread
	 * @param methodOnProcess metodo executado, com referencia no Processo (global)
	 * @param currentCalling Contador referente a methodOnProcess e methodOnThread
	 * @param caller Contador referente ao metodo que acionou methodOnProcess
	 * @param doCount Deve-se contar a chamada ou apenas marcar o relacionamento
	 */
	private void doCrossMethods(
		JBPMethod methodOnThread,
		JBPMethod methodOnProcess,
		JBPCall currentCalling,
		JBPCall caller,
		boolean doCount) {
		if (isProcessStackOnMethods()) {
			JBPMethod methodCallerOnProcess =
				(JBPMethod) process.getSpyMethods().get(caller.getSpyMethod());
			if (methodCallerOnProcess != null) {
				// Tratamento para
				JBPGenericCount methodCount =
					(JBPGenericCount) methodCallerOnProcess
						.getSpyMethodsCalling()
						.get(
						methodOnProcess);
				if (methodCount == null) {
					methodCount = new JBPGenericCount(methodOnProcess);
					methodCallerOnProcess.getSpyMethodsCalling().put(
						methodOnProcess,
						methodCount);
				}
				if (doCount) {
					methodCount.add();
					methodCount.setTotalClocks(
						methodCount.getTotalClocks()
							+ (caller.getClockFinish() - caller.getClockStart()));
				}

				JBPGenericCount methodCountCallerOnProcess =
					(JBPGenericCount) methodOnProcess.getSpyMethodsCallers().get(
						methodCallerOnProcess);
				if (methodCountCallerOnProcess == null) {
					methodCountCallerOnProcess =
						new JBPGenericCount(methodCallerOnProcess);
					methodOnProcess.getSpyMethodsCallers().put(
						methodCallerOnProcess,
						methodCountCallerOnProcess);
				}

				if (doCount) {
					methodCountCallerOnProcess.add();
					methodCountCallerOnProcess.setTotalClocks(
						methodCountCallerOnProcess.getTotalClocks()
							+ (caller.getClockFinish() - caller.getClockStart()));
				}
			}
		}
	}

	/**
	 * @param methodCaller Metodo que recebera as contagens (pode ser no
	 * Thread ou no processo, dependendo da situacao)
	 * @param stackCaller O stack contendo a chamada que esta sendo
	 * executada, referente a methodCalled. (Deve possuir os tempos
	 * preenchidos para poder efetuar doCount)
	 * @param methodCalled Metodo que recebera as contagens como chamado
	 * (como em methodCaller, pode ser em Thread ou Processo, dependendo da
	 * situacao)
	 * @param doCount as contagens devem ser efetuadas ou apenas devemos
	 * marcar as referencias para visualizacao
	 */
	private void stabilishReference(
		JBPThread thread,
		JBPMethod methodCaller,
		JBPCall stackCalled,
		JBPMethod methodCalled,
		boolean doCountOnSimpleCall) {

		JBPGenericCount countCaller =
			(JBPGenericCount) methodCalled.getSpyMethodsCallers().get(methodCaller);
		if (countCaller == null) {
			countCaller = new JBPGenericCount(methodCaller);
			methodCalled.getSpyMethodsCallers().put(methodCaller, countCaller);
		}

		if (doCountOnSimpleCall) {
			countCaller.add();
		}

		JBPMethodCount methodCount = thread.findMethodPath();
		if (methodCount != null && doCountOnSimpleCall) {
			methodCount.addTime(stackCalled.getClockFinish() - stackCalled.getClockStart(),
				stackCalled.getElapsedClockEnd() - stackCalled.getElapsedClockStart());
        }

    }

	public void debugThreads() {
		Iterator iterThreads =
			this.getProcess().getSpyThreads().values().iterator();
		while (iterThreads.hasNext()) {
			JBPThread thread = (JBPThread) iterThreads.next();
			debugMethods(thread);
		}

	}

	private void debugMethods(JBPThread thread) {
		if (thread.getSpyMethodCountRoots().isEmpty())
			return;
		System.out.println("**** Method Stack Trace on " + thread.getName());
		Iterator iter = thread.getSpyMethodCountRoots().values().iterator();
		while (iter.hasNext()) {
			JBPMethodCount count = (JBPMethodCount) iter.next();
			count.printDebugCalleds(0);
		}
	}

	/** Directory used to produce output registers */
	public void setOutputDirectory(java.io.File outputDirectory) {
		this.outputDirectory = outputDirectory;
	}

	/** Directory used to produce output registers */
	public java.io.File getOutputDirectory() {
		return outputDirectory;
	}
	public void setShowRecords(boolean showRecords) {
		this.showRecords = showRecords;
	}
	public boolean isShowRecords() {
		//return showRecords && this.levelAuthShowRecords>=0;
		return showRecords;
	}
	public boolean isShowRecords(JBPThread spyThread) {
		if (this.resolvedDetailMethods == null
			|| this.detailMethods == null
			|| this.getResolvedDetailMethods().isEmpty()
			&& this.detailMethods.isEmpty()) {
			return isShowRecords();
		} else {
			return showRecords
				&& spyThread != null
				&& spyThread.getLevelAuthShowRecords() >= 0;
		}
	}
	public void setProcessStackOnObjects(boolean processStackOnObjects) {
		this.processStackOnObjects = processStackOnObjects;
	}
	public boolean isProcessStackOnObjects() {
		return processStackOnObjects;
	}
	public void setProcessReleaseOnObjects(boolean processReleaseOnObjects) {
		this.processReleaseOnObjects = processReleaseOnObjects;
	}
	public boolean isProcessReleaseOnObjects() {
		return processReleaseOnObjects;
	}
	public void setProcessStackOnMethods(boolean processStackOnMethods) {
		this.processStackOnMethods = processStackOnMethods;
	}
	public boolean isProcessStackOnMethods() {
		return processStackOnMethods;
	}

	/** Os metodos sao identificados na assinatura padrao (Class.assinaturaPadrao(V))
	 *  Os metodos aqui identificados serao detalhados no outputDetail. */
	public java.util.HashMap getDetailMethods() {
		return detailMethods;
	}

	public void addDetailMethod(String detailMethod) {
		if (detailMethods == null) {
			detailMethods = new java.util.HashMap();
		}

		detailMethods.put(detailMethod, detailMethod);
	}

	public java.util.HashMap getResolvedDetailMethods() {
		if (resolvedDetailMethods == null) {
			resolvedDetailMethods = new HashMap();
		}
		return resolvedDetailMethods;
	}
	public boolean isUseIteractionsOnOutput() {
		return useIteractionsOnOutput;
	}
	public void setUseIteractionsOnOutput(boolean useIteractionsOnOutput) {
		this.useIteractionsOnOutput = useIteractionsOnOutput;
	}

}
