/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.engine;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;

import org.jboss.profiler.fileProcessor.SpyReader;

/**
  Uses the LogFileEngine by using a simple main method. It's for test purposes.
  @author Clebert Suconic
*/
public class StandaloneProcess implements RunnerEventListener {
    LogFileEngine engine;
    public void doProcess(String directory,String pid, int tick) {
       try {
         File fileDirectory = new File(directory);
         if (!fileDirectory.isDirectory()) {
            System.err.println(directory + " isn't a directory");
            return;
         }

         File[] mainFile = fileDirectory.listFiles(new SPYFilterMain(pid));
         if (mainFile.length==0) {
            System.err.println("PID Main File not found");
            return;
         }
         if (mainFile.length>1) {
            System.err.println("Cannot process more than one file");
            return;
         }

         File[] files = fileDirectory.listFiles(new SPYFilterThread(pid));

         InputStream mainStream = openStream(mainFile[0]);
         InputStream[] paramStreams = new InputStream[files.length];

         for (int i=0;i<files.length;i++) {
            System.out.println("Opening "+ files[i].getAbsolutePath());
            paramStreams[i] = openStream(files[i]);
         }

         SpyReader reader = new SpyReader(mainStream,paramStreams);
         engine = new LogFileEngine(reader);
         engine.addRunnerEventListener(this);
         engine.setProcessStackOnObjects(true);
         engine.setProcessStackOnMethods(true);

         engine.doProcess(false,pid,tick);

       }
       catch (Exception e){
        e.printStackTrace();
       }
    }

    protected static InputStream openStream (File file) throws IOException {
        InputStream input = new FileInputStream(file);
        if (file.getName().endsWith(".gz")) {
            input = new java.util.zip.GZIPInputStream(input);
        }
        return input;
    }

    public static void main(String[] arg) {
        if (!(arg.length==3 || arg.length==2)) {
            System.err.println("Usage: StandaloneProcess LogDir PID [Tick]");
            return;
        }
        int tick=10000;
        if (arg.length==3) tick = Integer.parseInt(arg[2]);
        StandaloneProcess standalone = new StandaloneProcess();
        standalone.doProcess(arg[0],arg[1],tick);
    }

    static class SPYFilterThread implements FilenameFilter {
        String pid;
        public SPYFilterThread(String pid) {
            this.pid=pid;
        }

        public boolean accept(File file, String name){
            return name.startsWith("serverspy_"+pid+"_thread");
        }
    }

    static class SPYFilterMain implements FilenameFilter {
        String pid;
        public SPYFilterMain(String pid) {
            this.pid=pid;
        }

        public boolean accept(File file, String name){
            return name.startsWith("serverspy_"+pid+"_main");
        }
    }

  public void processTick(RunnerEvent e) {
    System.err.println("tick " + e.getCurrentRegister());
  }

  public void processWarning(int line, String message) {
     System.err.println("Warning line " +line+ " -> " + message);
     System.out.println("Warning line " +line+ " -> " + message);
  }

    /**
     * @return Returns the engine.
     */
    public LogFileEngine getEngine() {
        return engine;
    }
}
