/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.profiler.engine;

import org.jboss.profiler.model.*;
import org.jboss.profiler.tracerModel.*;
import java.util.*;

/**
 * @author Yoko Seki (y-seki@sdl.hitachi.co.jp)
 */
public class TransactionSearchEngine
{
   private boolean transactionStart;
   private boolean transactionFinish;
   private boolean methodInclude;
   private boolean methodExclude;
   private boolean transactionElapse;
   private boolean methodElapse;
   private String includeAndOr;
   private String excludeAndOr;

   public HashMap doSearch (HashMap transactions, JBPProcess jbpProcess, JBTSearchCondition condition)
   {
      HashMap searchedTransactions = new HashMap();
      Iterator iter = transactions.values().iterator();
      while (iter.hasNext())
      {
         JBTTransaction transaction  = (JBTTransaction) iter.next();
         if (   ( !isTransactionStart()  || (isTransactionStart()&&transactionStartSatisfied(transaction,condition)) )
             && ( !isTransactionFinish() || (isTransactionFinish()&&transactionFinishSatisfied(transaction,condition)) )
             && ( !isMethodInclude()     || (isMethodInclude()&&methodIncludeSatisfied(transaction,condition,jbpProcess)) )
             && ( !isMethodExclude()     || (isMethodExclude()&&methodExcludeSatisfied(transaction,condition,jbpProcess)) )
             && ( !isTransactionElapse() || (isTransactionElapse()&&transactionElapseSatisfied(transaction,condition)) )
             && ( !isMethodElapse()      || (isMethodElapse()&&methodElapseSatisfied(transaction,condition)) )
            )
         { searchedTransactions.put(transaction,transaction); }
      }
      transactions.clear();
      return searchedTransactions;
   }

   protected boolean transactionStartSatisfied(JBTTransaction transaction, JBTSearchCondition condition)
   {
      long transactionStartTime = ((JBTMethod)transaction.getMethods().get(0)).getElapsedClockStart();

      Calendar cal1 = Calendar.getInstance();
      cal1.set(Calendar.YEAR,        condition.getTranStartY1());
      cal1.set(Calendar.MONTH,       condition.getTranStartM1()-1);
      cal1.set(Calendar.DATE,        condition.getTranStartD1());
      cal1.set(Calendar.HOUR_OF_DAY, condition.getTranStartHH1());
      cal1.set(Calendar.MINUTE,      condition.getTranStartMM1());
      cal1.set(Calendar.SECOND,      condition.getTranStartSS1());
      cal1.set(Calendar.MILLISECOND, condition.getTranStartMS1());

      Calendar cal2 = Calendar.getInstance();
      cal2.set(Calendar.YEAR,        condition.getTranStartY2());
      cal2.set(Calendar.MONTH,       condition.getTranStartM2()-1);
      cal2.set(Calendar.DATE,        condition.getTranStartD2());
      cal2.set(Calendar.HOUR_OF_DAY, condition.getTranStartHH2());
      cal2.set(Calendar.MINUTE,      condition.getTranStartMM2());
      cal2.set(Calendar.SECOND,      condition.getTranStartSS2());
      cal2.set(Calendar.MILLISECOND, condition.getTranStartMS2());

      if ((cal1.getTimeInMillis()<=transactionStartTime)&&(transactionStartTime<=cal2.getTimeInMillis()))
         return true;
      else return false;
   }

   protected boolean transactionFinishSatisfied(JBTTransaction transaction, JBTSearchCondition condition)
   {
      long transactionFinishTime = ((JBTMethod)transaction.getMethods().get(0)).getElapsedClockEnd();

      Calendar cal1 = Calendar.getInstance();
      cal1.set(Calendar.YEAR,        condition.getTranFinishY1());
      cal1.set(Calendar.MONTH,       condition.getTranFinishM1()-1);
      cal1.set(Calendar.DATE,        condition.getTranFinishD1());
      cal1.set(Calendar.HOUR_OF_DAY, condition.getTranFinishHH1());
      cal1.set(Calendar.MINUTE,      condition.getTranFinishMM1());
      cal1.set(Calendar.SECOND,      condition.getTranFinishSS1());
      cal1.set(Calendar.MILLISECOND, condition.getTranFinishMS1());

      Calendar cal2 = Calendar.getInstance();
      cal2.set(Calendar.YEAR,        condition.getTranFinishY2());
      cal2.set(Calendar.MONTH,       condition.getTranFinishM2()-1);
      cal2.set(Calendar.DATE,        condition.getTranFinishD2());
      cal2.set(Calendar.HOUR_OF_DAY, condition.getTranFinishHH2());
      cal2.set(Calendar.MINUTE,      condition.getTranFinishMM2());
      cal2.set(Calendar.SECOND,      condition.getTranFinishSS2());
      cal2.set(Calendar.MILLISECOND, condition.getTranFinishMS2());

      if ((cal1.getTimeInMillis()<=transactionFinishTime)&&(transactionFinishTime<=cal2.getTimeInMillis()))
         return true;
      else return false;
   }

   protected boolean methodIncludeSatisfied(JBTTransaction transaction,
                                            JBTSearchCondition condition,
                                            JBPProcess jbpProcess)
   {
      ArrayList searchMethods = new ArrayList();
      StringTokenizer tokenMethods = new StringTokenizer(condition.getIncludeMethodName(), " ");
      while (tokenMethods.hasMoreTokens())
      {
         searchMethods.add(tokenMethods.nextToken());
      }
      ArrayList transactionMethods = transaction.getMethods();

      if ((includeAndOr).equals("or"))   //in case of OR
      {
         for (int i=0; i<transactionMethods.size(); i++)
         {
            JBTMethod transactionMethod = (JBTMethod) transactionMethods.get(i);
            String methodName = getMethodNameFromLoadData(transactionMethod, jbpProcess);

            for (int j=0; j<searchMethods.size(); j++)
            {
               if (methodName.indexOf((String)searchMethods.get(j))>=0)
                  return true;
            }
         }
         return false;
      }

      else   //in case of AND
      {
         int[] flag = new int[searchMethods.size()];
         for (int i=0; i<searchMethods.size(); i++)
         {
            flag[i] = 0;
         }
         for (int j=0; j<transactionMethods.size(); j++)
         {
            JBTMethod transactionMethod = (JBTMethod) transactionMethods.get(j);
            String methodName = getMethodNameFromLoadData(transactionMethod, jbpProcess);

            for (int k=0; k<searchMethods.size(); k++)
            {
               if (methodName.indexOf((String)searchMethods.get(k))>=0)
                  flag[k] = 1;
            }
            int l = 0;
            while (l<searchMethods.size())
            {
               if (flag[l]==0) break;   //go to check next transactionMethod
               else   //in case of flag==1
               {
                  if (l==(searchMethods.size()-1)) return true;
                  else l++;
               }
            }
         }
         return false;
      }
   }

   protected boolean methodExcludeSatisfied(JBTTransaction transaction,
                                            JBTSearchCondition condition,
                                            JBPProcess jbpProcess)
   {
      ArrayList searchMethods = new ArrayList();
      StringTokenizer tokenMethods = new StringTokenizer(condition.getExcludeMethodName(), " ");
      while (tokenMethods.hasMoreTokens())
      {
         searchMethods.add(tokenMethods.nextToken());
      }

      ArrayList transactionMethods = transaction.getMethods();

      if ((excludeAndOr).equals("and")) //in case of AND
      {
         for (int i=0; i<transactionMethods.size(); i++)
         {
            JBTMethod transactionMethod = (JBTMethod) transactionMethods.get(i);
            String methodName = getMethodNameFromLoadData(transactionMethod, jbpProcess);

            for (int j=0; j<searchMethods.size(); j++)
            {
               if (methodName.indexOf((String)searchMethods.get(j))>=0)
                  return false;
            }
         }
         return true;
      }
      else //in case of OR
      {
         int[] flag = new int[searchMethods.size()];
         for (int i=0; i<searchMethods.size(); i++)
         {
            flag[i] = 0;
         }
         for (int j=0; j<transactionMethods.size(); j++)
         {
            JBTMethod transactionMethod = (JBTMethod) transactionMethods.get(j);
            String methodName = getMethodNameFromLoadData(transactionMethod, jbpProcess);

            for (int k=0; k<searchMethods.size(); k++)
            {
               if (methodName.indexOf((String)searchMethods.get(k))>=0)
                  flag[k] = 1;
            }
         }
         int l = 0;
         while (l<searchMethods.size())
         {
            if (flag[l]==0) return true;
            else l++;
         }
         return false;
      }
   }

   protected boolean transactionElapseSatisfied(JBTTransaction transaction, JBTSearchCondition condition)
   {
      long transactionStartTime  = ((JBTMethod)transaction.getMethods().get(0)).getElapsedClockStart();
      long transactionFinishTime = ((JBTMethod)transaction.getMethods().get(0)).getElapsedClockEnd();

      if (((long)condition.getTransactionTime()<(transactionFinishTime-transactionStartTime)) ||
          (transactionFinishTime==-1))
         return true;
      else return false;
   }

   protected boolean methodElapseSatisfied(JBTTransaction transaction, JBTSearchCondition condition)
   {
      ArrayList transactionMethods = transaction.getMethods();

      for (int i=0; i<transactionMethods.size(); i++)
      {
         JBTMethod transactionMethod = (JBTMethod)transactionMethods.get(i);
         long methodStartTime  = transactionMethod.getElapsedClockStart();
         long methodFinishTime = transactionMethod.getElapsedClockEnd();

         if (((long)condition.getMethodTime()<(methodFinishTime-methodStartTime)) ||
             (methodFinishTime==-1))
            return true;
      }
      return false;
   }


   String getMethodNameFromLoadData (JBTMethod transactionMethod, JBPProcess jbpProcess)
   {
      HashMap processMethods = jbpProcess.getSpyMethods();
      Iterator iter = processMethods.values().iterator();
      JBPMethod processMethod = new JBPMethod();
      while (iter.hasNext())
      {
         processMethod = (JBPMethod) iter.next();
         if (processMethod.getMethodID()==transactionMethod.getMethodID())
         {
            break;
         }
      }
      return processMethod.getSimplifiedName();
   }

   public boolean isTransactionStart() { return transactionStart; }
   public void setTransactionStart(boolean transactionStart)
   { this.transactionStart = transactionStart; }

   public boolean isTransactionFinish() { return transactionFinish; }
   public void setTransactionFinish(boolean transactionFinish)
   { this.transactionFinish = transactionFinish; }

   public boolean isMethodInclude() { return methodInclude; }
   public void setMethodInclude(boolean methodInclude)
   { this.methodInclude = methodInclude; }

   public boolean isMethodExclude() { return methodExclude; }
   public void setMethodExclude(boolean methodExclude)
   { this.methodExclude = methodExclude; }

   public boolean isTransactionElapse() { return transactionElapse; }
   public void setTransactionElapse(boolean transactionElapse)
   { this.transactionElapse = transactionElapse; }

   public boolean isMethodElapse() { return methodElapse; }
   public void setMethodElapse(boolean methodElapse)
   { this.methodElapse = methodElapse; }

   public String getIncludeAndOr() { return includeAndOr; }
   public void setIncludeAndOr(String includeAndOr)
   { this.includeAndOr = includeAndOr; }

   public String getExcludeAndOr() { return excludeAndOr; }
   public void setExcludeAndOr(String excludeAndOr)
   { this.excludeAndOr = excludeAndOr; }
}
