/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.fileProcessor;

import java.util.Date;
import java.io.DataInput;
import java.io.IOException;

/**
 * @author Clebert Suconic
 */
public class SpyRegister {
    public static final short CONST_LOADCLASS = 1;

    public static final short CONST_LOADMETHOD = 2;

    public static final short CONST_LOADOBJECT = 3;

    public static final short CONST_UNLOADOBJECT = 4;

    public static final short CONST_GCSTART = 5;

    public static final short CONST_GCFINISH = 6;

    public static final short CONST_THREADSTART = 7;

    public static final short CONST_THREADEND = 8;

    public static final short CONST_ENTERMETHOD = 9;

    public static final short CONST_EXITMETHOD = 10;

    public static final short CONST_MONITOR_IN = 11;

    public static final short CONST_MONITOR_OUT = 12;

    public static final short CONST_CONTENDED_ENTER = 13;

    public static final short CONST_CONTENDED_ENTERED = 14;

    public static final short CONST_CONTENDED_EXIT = 15;

    public static final short CONST_RAW_CONTENDED_ENTER = 16;

    public static final short CONST_RAW_CONTENDED_ENTERED = 17;

    public static final short CONST_RAW_CONTENDED_EXIT = 18;

    /** A que horas o processo foi inicializado (para o cauldo de elapsedClock) */
    public static long startOfProcess = 0;

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public Date getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(Date timestamp) {
        this.timestamp = timestamp;
    }

    public long getClock() {
        return clock;
    }

    public void setClock(long clock) {
        this.clock = clock;
    }

    /**
     * @return false if eof
     */
    public boolean read(java.io.DataInput dataInput) throws java.io.IOException {
        id = ((java.io.InputStream) dataInput).read();

        if (id == -1)
            return false; // eof

        time_t = dataInput.readLong();
        if (timestamp == null) {
            timestamp = new Date(time_t);
        } else {
            timestamp.setTime(time_t);
        }
        clock = dataInput.readLong();

        elapsedClock = (time_t - startOfProcess);

        // -- versao antiga
        //clock = dataInput.readInt();
        //elapsedClock = dataInput.readInt();

        return true;
    }

    public void setValues(SpyRegister spyToCopy) {
        this.setTimestamp(spyToCopy.getTimestamp());
        this.setTime_t(spyToCopy.getTime_t());
        this.setClock(spyToCopy.getClock());
        this.setElapsedClock(spyToCopy.getElapsedClock());
        this.setId(spyToCopy.getId());
    }

    /**
     * Read a ThreadStart register. The ID, clock and time_t is present on
     * identRegister, and isn`t necessary to read these attributes.
     */
    public void read(DataInput dataInput, SpyRegister identRegister)
            throws IOException {
        this.setValues(identRegister);
    }

    private int id;

    private Date timestamp;

    private long clock;

    private long time_t;

    private long elapsedClock;

    public String toString() {
        return "id = " + id + " timestamp = " + timestamp + " time_t = "
                + time_t + " clock = " + clock;
    }

    public long getTime_t() {
        return time_t;
    }

    public void setTime_t(long time_t) {
        this.time_t = time_t;
    }

    public void setElapsedClock(long elapsedClock) {
        this.elapsedClock = elapsedClock;
    }

    public long getElapsedClock() {
        return elapsedClock;
    }
}