/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.fileProcessor;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

/**
 * Read data using the same alignment as used on Windows.
@author Clebert Suconic
*/
public class WindowsDataInputStream extends InputStream implements DataInput {
    private void raiseNotImplemented() throws IOException {
        throw new IOException("Not implemented");
    }

    public WindowsDataInputStream(InputStream srcInputStream) {
        if (srcInputStream instanceof DataInputStream) {
            this.srcInputStream = (DataInputStream)srcInputStream;
        } else {
            this.srcInputStream = new DataInputStream(srcInputStream);
        }
    }

    public void readFully(byte[] par1) throws java.io.IOException {
        super.read(par1);
    }

    public void readFully(byte[] par1, int par2, int par3) throws java.io.IOException {
        super.read(par1, par2, par3);
    }

    public int skipBytes(int par1) throws java.io.IOException {
        return (int)super.skip(par1);
    }

    public boolean readBoolean() throws java.io.IOException {
        return this.read() != 0; // Windows uses this convention
    }

    public byte readByte() throws java.io.IOException {
        return (byte)this.read();
    }

    public int readUnsignedByte() throws IOException {
        return this.read();
    }

    public short readShort() throws IOException {
        int ch1 = this.read();
        int ch2 = this.read();
        if ((ch1 | ch2) < 0)
            throw new EOFException();
        return (short)((ch1 << 0) + (ch2 << 8));
    }

    public int readUnsignedShort() throws IOException {
        raiseNotImplemented();
        return 0;
    }

    public char readChar() throws IOException {
        int ch1 = this.read();
        int ch2 = this.read();
        if ((ch1 | ch2) < 0)
            throw new EOFException();
        return (char)((ch2 << 8) + (ch1 << 0));
    }

    /** Read a integer acording to Windows Alignment */
    public int readInt() throws IOException {
        /*int ch1 = this.read()<<0;
        int ch2 = this.read()<<8;
        int ch3 = this.read()<<16;
        int ch4 = this.read()<<24; */
        //return ((ch1 << 0) + (ch2 << 8) + (ch3 << 16) + (ch4 << 24));
        //return ((ch1 << 0) + (ch2 << 8) + (ch3 << 16) + (ch4 << 24));
        return ((this.read()<<0) | (this.read()<<8) | (this.read()<<16) | (this.read()<<24));
    }

    public long readLong() throws IOException {
//       return getAssociatedStream().readLong();
       /*long x1 = this.read();
       long x2 = this.read();
       long x3 = this.read();
       long x4 = this.read();
       long x5 = this.read();
       long x6 = this.read();
       long x7 = this.read();
       long x8 = this.read();
       return ((x1<<0) | (x2<<8) | (x3<<16) | (x4<<24) |
               (x5<<32)| (x6<<40)| (x7<<48) | (x8<<56)); */
       return (((long)this.read()<<0) | ((long)this.read()<<8) | ((long)this.read()<<16) | ((long)this.read()<<24) |
               ((long)this.read()<<32)| ((long)this.read()<<40)| ((long)this.read()<<48) | ((long)this.read()<<56));
    }

    public float readFloat() throws IOException {
        return getAssociatedStream().readFloat();
    }

    public double readDouble() throws IOException {
        return getAssociatedStream().readDouble();
    }

    /** @deprecated */
    public String readLine() throws IOException  {
        return getAssociatedStream().readLine();
    }

    public String readUTF() throws IOException {
        return srcInputStream.readUTF();
    }

    public int read() throws java.io.IOException {
        position++;
        return srcInputStream.read();
    }

    protected DataInputStream getAssociatedStream() {
        return srcInputStream;
    }

    public int getPosition() {
        return position;
    }

    public void forwardTo(int parmPosition) throws IOException {
        for ( ; this.getPosition() < parmPosition; ) {
            this.readByte();
        }
    }

    public synchronized void mark(int readlimit) {
        srcInputStream.mark(readlimit);
    }

    public synchronized void reset() throws IOException {
        srcInputStream.reset();
    }

    public boolean markSupported() {
        return srcInputStream.markSupported();
    }

    /** @link aggregationByValue */
    private DataInputStream srcInputStream;
    private int position = 0;
}
