/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.fileProcessor;

import java.io.DataOutputStream;
import java.io.DataOutput;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Write data using the same alignment as used on Windows.
 * @author Clebert Suconic
 */
public class WindowsDataOutputStream extends OutputStream implements DataOutput {
    public WindowsDataOutputStream(OutputStream srcOutputStream) {
        if (srcOutputStream instanceof DataOutputStream) {
            this.srcOutputStream = (DataOutputStream)srcOutputStream;
        } else {
            this.srcOutputStream = new DataOutputStream(srcOutputStream);
        }
    }

    /** Implementacao de OutputStream */
    public void write(int par1) throws java.io.IOException {
        position++;
        srcOutputStream.write(par1);
    }

    public void writeBoolean(boolean par1) throws IOException {
        write(par1 ? 1 : 0);
    }

    public void writeByte(int par1) throws IOException {
        write(par1);
    }

    public void writeShort(int v) throws IOException {
        srcOutputStream.write((v >>> 8) & 0xFF);
        srcOutputStream.write((v >>> 0) & 0xFF);
    }

    public void writeChar(int par1) throws IOException {
        write(par1);
    }

    public void writeInt(int v) throws IOException {
        this.write((v >>> 0) & 0xFF);
        this.write((v >>> 8) & 0xFF);
        this.write((v >>> 16) & 0xFF);
        this.write((v >>> 24) & 0xFF);
    }

    public void writeLong(long par1) throws IOException {
        writeInt((int)par1);
    }

    public void writeFloat(float par1) throws IOException {
        srcOutputStream.writeFloat(par1);
    }

    public void writeDouble(double par1) throws IOException {
        srcOutputStream.writeDouble(par1);
    }

    public void writeBytes(String par1) throws IOException {
        srcOutputStream.writeBytes(par1);
    }

    public void writeChars(String par1) throws IOException {
        srcOutputStream.writeChars(par1);
    }

    public void writeUTF(String par1) throws IOException {
        srcOutputStream.writeUTF(par1);
    }

    public int getPosition() {
        return position;
    }

    /* public short readShort() throws IOException {
        int ch1 = this.read();
        int ch2 = this.read();
        if ((ch1 | ch2) < 0)
            throw new EOFException();
        return (short)((ch1 << 0) + (ch2 << 8));
    } */

    /** @link aggregationByValue */
    private DataOutputStream srcOutputStream;

    /** @link dependency */

    /*#OleDataConverterUtility lnkOleDataConverterUtility;*/

    private int position = 0;
}
