/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/


package org.jboss.profiler.filecollection;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;

public class BulkAdd
{
    CountOutputStream countRegisters;
    DataOutputStream outRegisters;

    DataOutputStream outIndex;

    Index ind = new Index();

    FileCollection collection;

    public BulkAdd(RandomAccessFile registersChannel,RandomAccessFile indexesChannel, FileCollection collection) throws Exception
    {
        this.collection=collection;
        FileChannel fileChannel = registersChannel.getChannel();
        fileChannel.position(fileChannel.size());
        OutputStream output = Channels.newOutputStream(fileChannel);
        countRegisters = new CountOutputStream(new BufferedOutputStream(output),fileChannel.size());
        outRegisters = new DataOutputStream(countRegisters);

        if (indexesChannel!=null)
        {
            fileChannel = indexesChannel.getChannel();
            fileChannel.position(fileChannel.size());
            output = Channels.newOutputStream(fileChannel);
            outIndex = new DataOutputStream(new BufferedOutputStream(output));
        }
    }

    public void close() throws Exception
    {
        outRegisters.flush();
        if (outIndex!=null) {
            outIndex.flush();
        }
        collection.calculateSize();
    }

    public void add(FileCollectionInterface obj) throws Exception
    {
        long oldIndex = countRegisters.getCurrentCount();
        obj.writeObject(outRegisters);
        fixSize(oldIndex);

        addIndex(oldIndex);
        //collection.addIndex(oldIndex,countRegisters.getCurrentCount());

    }

    private void addIndex(long oldIndex) throws IOException
    {
        if (collection.fixedSize==0)
        {
            ind.posStart = oldIndex;
            ind.posEnd = countRegisters.getCurrentCount();
            ind.writeObject(outIndex);
        }
    }

    private void fixSize(long oldIndex) throws IOException
    {
        if (collection.fixedSize>0)
        {
            long currentSize = countRegisters.getCurrentCount() - oldIndex;
            if (currentSize>collection.fixedSize)
            {
                throw new RuntimeException("Invalid register size => currentSize=" + currentSize + " and fixedSize = " + collection.fixedSize);
            }

            // Fixing the size of the register
            for (;currentSize<collection.fixedSize;currentSize++)
            {
                outRegisters.write(0);
            }
        }
    }

}

