/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.profiler.filecollection;

import java.io.IOException;
import java.io.OutputStream;

public class CountOutputStream extends OutputStream
{
    long currentCount;
    OutputStream output;

    public CountOutputStream(OutputStream output, long currentCount)
    {
        this.output=output;
        this.currentCount=currentCount;
    }

    public void write(int b) throws IOException
    {
        this.currentCount++;
        this.output.write(b);
    }

    public void flush() throws IOException
    {
        output.flush();
    }



    /**
     * @return Returns the currentCount.
     */
    public long getCurrentCount()
    {
        return currentCount;
    }
    /**
     * @param currentCount The currentCount to set.
     */
    public void setCurrentCount(long currentCount)
    {
        this.currentCount = currentCount;
    }

    public void close() throws IOException
    {
        output.close();
    }
}

