/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.graphmodel;

import org.jboss.profiler.model.JBPMethod;

import java.io.Serializable;

/**
 * Method to be included on the Graph
@author Clebert Suconic
*/
public class GraphMethod implements Serializable {

  private static final long serialVersionUID = 100L;

  private String name;
  private String signature;
  private int level;
  private int producedBytes;
  private long methodId;
  private String className;
  private String key;
  /** Total of responsability on the graph on methods callers */
  private double weight;
  private String url;

  public GraphMethod() {
  }

  public GraphMethod(JBPMethod method, int level,String url) {
    this.className=method.getSpyClass().getName();
    this.name = method.getName();
    this.signature = method.getSignature();
    this.setMethodId(method.getMethodID());
    this.url=url;
    this.level=level;
  }

  public String getName() {
    return name;
  }
  public void setName(String name) {
    this.name = name;
  }
  public void setSignature(String signature) {
    this.signature = signature;
  }
  public String getSignature() {
    return signature;
  }
  public void setLevel(int level) {
    this.level = level;
  }
  public int getLevel() {
    return level;
  }
  public void setProducedBytes(int producedBytes) {
    this.producedBytes = producedBytes;
  }
  public int getProducedBytes() {
    return producedBytes;
  }
  public void setMethodId(long methodId) {
    this.methodId = methodId;
    this.key = Long.toString(methodId);
  }
  public long getMethodId() {
    return methodId;
  }

  public boolean equals(Object obj) {
     return (methodId==((GraphMethod)obj).getMethodId()) && level==((GraphMethod)obj).level;
  }

  public int hashCode() {
     return key.hashCode();
  }
  public void setClassName(String className) {
    this.className = className;
  }
  public String getClassName() {
    return className;
  }

  public String toString() {
     return name;
  }
  public void setWeight(double weight) {
    this.weight = weight;
  }
  public double getWeight() {
    return weight;
  }
  public void setUrl(String url) {
    this.url = url;
  }
  public String getUrl() {
    return url;
  }

}
