/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.graphmodel;

/**
 * The reference between two methods
@author Clebert Suconic
*/
public final class GraphReference implements java.io.Serializable {

  private static final long serialVersionUID = 100L;

  private String key;

  private int levelCallee;
  private int levelCalled;
  private int instancesCreated;
  private long methodCallee;
  private long methodCalled;
  private double percentageTime;
  private double percentageCPU;

  public GraphReference() {
  }

 public GraphReference(int levelCallee, long methodCallee, int levelCalled, long methodCalled, double percentageTime, double percentageCPU) {
   this.methodCallee = methodCallee;
   this.methodCalled = methodCalled;
   this.percentageCPU = percentageCPU;
   this.percentageTime=percentageTime;
   this.levelCalled=levelCalled;
   this.levelCallee=levelCallee;
   this.generateKey();
 }

 public GraphReference(int levelCallee, long methodCallee, int levelCalled, long methodCalled, int instancesCreated) {
   this.methodCallee = methodCallee;
   this.methodCalled = methodCalled;
   this.instancesCreated = instancesCreated;
   this.levelCalled=levelCalled;
   this.levelCallee=levelCallee;
   this.generateKey();
 }

  /**
   * @return
   */
  public int getLevelCalled() {
    return levelCalled;
  }

  /**
   * @param levelCalled
   */
  public void setLevelCalled(int levelCalled) {
    this.levelCalled = levelCalled;
  }

  /**
   * @return
   */
  public int getLevelCallee() {
    return levelCallee;
  }

  /**
   * @param levelCallee
   */
  public void setLevelCallee(int levelCallee) {
    this.levelCallee = levelCallee;
  }

  private void generateKey() {
    key = methodCallee + "" + methodCalled;
  }

  public long getMethodCallee() {
    return methodCallee;
  }

  public long getMethodCalled() {
    return methodCalled;
  }

  public void setPercentageTime(double percentageTime) {
    this.percentageTime = percentageTime;
  }

  public double getPercentageTime() {
    return percentageTime;
  }

  public void setPercentageCPU(double percentageCPU) {
    this.percentageCPU = percentageCPU;
  }

  public double getPercentageCPU() {
    return percentageCPU;
  }

  public boolean equals(Object obj) {
     return methodCallee == ((GraphReference)obj).methodCallee &&
            methodCalled == ((GraphReference)obj).methodCalled &&
			levelCalled == ((GraphReference)obj).levelCalled &&
			levelCallee == ((GraphReference)obj).levelCallee;
  }

  public int hashCode() {
     return key.hashCode();
  }

  public void setMethodCalled(long methodCalled) {
    this.methodCalled = methodCalled;
    this.generateKey();
  }

  public void setMethodCallee(long methodCallee) {
    this.methodCallee = methodCallee;
    this.generateKey();
  }


  /**
   * @return
   */
  public int getInstancesCreated() {
    return instancesCreated;
  }

  /**
   * @param instancesCreated
   */
  public void setInstancesCreated(int instancesCreated) {
    this.instancesCreated = instancesCreated;
  }

}
