/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.memoryprofiler.model;


/** Loaded class during snapshot
 * @author Clebert Suconic
 *  */
public class MemoryClass extends MemoryObject {

    long id;

    String signature;

    int numberOfInstances;

    long loadedSize;

    int classType;

    long classLoaderId;

    public static final int CLASS_TYPE_REGULAR = 0;

    public static final int CLASS_TYPE_LOADER = 1;

    public static final int CLASS_TYPE_THREAD = 1;

    public MemoryClass() {
    }

    public MemoryClass(long id, String signature, long classLoaderId) {
        this.id = id;
        this.signature = signature;
        this.classLoaderId = classLoaderId;
    }

    /**
     * @return Returns the classType.
     */
    public int getClassType() {
        return classType;
    }

    /**
     * @param classType
     *            The classType to set.
     */
    public void setClassType(int classType) {
        this.classType = classType;
    }

    /**
     * @return Returns the loadedSize.
     */
    public long getLoadedSize() {
        return loadedSize;
    }

    /**
     * @param loadedSize
     *            The loadedSize to set.
     */
    public void setLoadedSize(long loadedSize) {
        this.loadedSize = loadedSize;
    }

    /**
     * @return Returns the numberOfInstances.
     */
    public int getNumberOfInstances() {
        return numberOfInstances;
    }

    /**
     * @param numberOfInstances
     *            The numberOfInstances to set.
     */
    public void setNumberOfInstances(int numberOfInstances) {
        this.numberOfInstances = numberOfInstances;
    }

    /**
     * @return Returns the signature.
     */
    public String getSignature() {
        return signature;
    }

    /**
     * @param signature
     *            The signature to set.
     */
    public void setSignature(String signature) {
        this.signature = signature;
    }

    public boolean equals(Object obj) {
        return id == ((MemoryClass) obj).id;
    }

    public int hashCode() {
        return (int) id;
    }

    /**
     * @return Returns the id.
     */
    public long getId() {
        return id;
    }

    /**
     * @param id
     *            The id to set.
     */
    public void setId(long id) {
        this.id = id;
    }

    public void addSize(long size) {
        loadedSize += size;
        numberOfInstances++;
    }

    public String toString() {
        return id + " " + signature + " numberOfInstances=" + numberOfInstances + " loadedSize=" + loadedSize + " classtype = " + classType + " classLoaderId=" + classLoaderId;
    }

    /**
     * @return Returns the classLoaderId.
     */
    public long getClassLoaderId() {
        return classLoaderId;
    }

    /**
     * @param classLoaderId
     *            The classLoaderId to set.
     */
    public void setClassLoaderId(long classLoaderId) {
        this.classLoaderId = classLoaderId;
    }


}