/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.memoryprofiler.model;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Comparator;

import org.jboss.profiler.filecollection.FileCollectionInterface;

/**
 * @author Clebert Suconic
 */
public class MemoryObject implements FileCollectionInterface{

    public static class MemoryObjectComparator implements Comparator
    {

        /* (non-Javadoc)
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare(Object arg0, Object arg1)
        {

            long result = ((MemoryObject)arg1).id - ((MemoryObject)arg0).id;
            if (result>0) return 1;
            else if (result<0) return -1;
            else return 0;
        }
    }

    long id;

    long size;

    long classId;

    MemoryClass memoryClass = null;

    public MemoryObject() {
    }

    public MemoryObject(long id, long size, MemoryClass memoryClass) {
        this.id = id;
        this.size = size;
        this.classId = memoryClass.getClassId();
        this.memoryClass = memoryClass;
    }

    public MemoryObject(long id, long size, long classId) {
        this.id = id;
        this.size = size;
        this.classId = classId;
    }

    public boolean equals(Object obj) {
        return id == ((MemoryObject) obj).id;
    }

    public int hashCode() {
        // code grabbed from java.lang.Long#hashCode()
        return (int) (id ^ (id >>> 32));
    }

    /**
     * @return Returns the id.
     */
    public long getId() {
        return id;
    }

    /**
     * @param id
     *            The id to set.
     */
    public void setId(long id) {
        this.id = id;
    }

    /**
     * @return Returns the memoryClass.
     */
    public MemoryClass getMemoryClass() {
        return memoryClass;
    }

    /**
     * @param memoryClass
     *            The memoryClass to set.
     */
    public void setMemoryClass(MemoryClass memoryClass) {
        this.memoryClass = memoryClass;
    }

    /**
     * @return Returns the size.
     */
    public long getSize() {
        return size;
    }

    /**
     * @param size
     *            The size to set.
     */
    public void setSize(long size) {
        this.size = size;
    }


    public String toString()
    {
        return id + "-" + memoryClass;
    }


    /**
     * @return Returns the classId.
     */
    public long getClassId()
    {
        return classId;
    }
    /**
     * @param classId The classId to set.
     */
    public void setClassId(long classId)
    {
        this.classId = classId;
    }

    /* (non-Javadoc)
     * @see org.jboss.profiler.filecollection.FileCollectionInterface#readObject(java.io.DataInput)
     */
    public void readObject(DataInput ch) throws IOException
    {
        id = ch.readLong();
        size = ch.readLong();
        classId = ch.readLong();
    }

    /* (non-Javadoc)
     * @see org.jboss.profiler.filecollection.FileCollectionInterface#writeObject(java.io.DataOutput)
     */
    public void writeObject(DataOutput ch) throws IOException
    {
        ch.writeLong(id);
        ch.writeLong(size);
        ch.writeLong(classId);

    }

}