/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.memoryprofiler.model;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Comparator;

import org.jboss.profiler.filecollection.FileCollectionInterface;

/**
 * @author Clebert Suconic
 */
public class MemoryReference implements FileCollectionInterface
{

    long referee;

    long referred;

    boolean isThreadReference = false;

    transient MemoryObject objectReferred;

    public MemoryReference(long referee, long referred, boolean isThread)
    {
        this.referee = referee;
        this.referred = referred;
        this.isThreadReference = isThread;
    }

    public MemoryReference()
    {
    }

    /**
     * @param object
     */
    public MemoryReference(MemoryObject object)
    {
        this.objectReferred=object;
    }

    /**
     * @return Returns the isThreadReference.
     */
    public boolean isThreadReference()
    {
        return isThreadReference;
    }

    /**
     * @param isThreadReference
     *            The isThreadReference to set.
     */
    public void setThreadReference(boolean isThreadReference)
    {
        this.isThreadReference = isThreadReference;
    }

    /**
     * @return Returns the referee.
     */
    public long getReferee()
    {
        return referee;
    }

    /**
     * @param referee
     *            The referee to set.
     */
    public void setReferee(long referee)
    {
        this.referee = referee;
    }

    /**
     * @return Returns the referred.
     */
    public long getReferred()
    {
        return referred;
    }

    /**
     * @param referred
     *            The referred to set.
     */
    public void setReferred(long referred)
    {
        this.referred = referred;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.jboss.profiler.util.FileCollectionInterface#readObject(java.io.DataInput)
     */
    public void readObject(final DataInput ch) throws IOException
    {

        referee = ch.readLong();
        referred = ch.readLong();
        isThreadReference = ch.readBoolean();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.jboss.profiler.util.FileCollectionInterface#writeObject(java.io.DataOutput)
     */
    public void writeObject(final DataOutput ch) throws IOException
    {
        ch.writeLong(referee);
        ch.writeLong(referred);
        ch.writeBoolean(isThreadReference);
    }


    /**
     * @return Returns the objectReferred.
     */
    public MemoryObject getObjectReferred()
    {
        return objectReferred;
    }

    /**
     * @param objectReferred
     *            The objectReferred to set.
     */
    public void setObjectReferred(MemoryObject objectReferred)
    {
        this.objectReferred = objectReferred;
    }

    public static class MemoryReferenceComparator implements Comparator
    {
        boolean invertedCompare;

        public MemoryReferenceComparator(boolean invertedCompare)
        {
            this.invertedCompare=invertedCompare;
        }


        public int compare(Object arg0, Object arg1)
        {
            MemoryReference a = (MemoryReference)arg0;
            MemoryReference b = (MemoryReference)arg1;

            if (invertedCompare)
            {
                if (a.referred==b.referred)
                {
                    if (a.referee==-1 || b.referee==-1)
                    {
                        return 0; // We will be doing partial comparisson in this case
                    }
                    else
                    if (a.referee>b.referee)
                    {
                        return 1;
                    } else if (a.referee<b.referee)
                    {
                        return -1;
                    } else
                    return 0;
                } else if (a.referred>b.referred)
                {
                    return 1;
                } else
                {
                    return -1;
                }
            } else
            {
                if (a.referee==b.referee)
                {
                    if (a.referred==-1 || b.referred==-1)
                    {
                        return 0; // We will be doing partial comparisson in this case
                    }
                    else
                    if (a.referred>b.referred)
                    {
                        return 1;
                    } else if (a.referred<b.referred)
                    {
                        return -1;
                    } else
                    return 0;
                } else if (a.referee>b.referee)
                {
                    return 1;
                } else
                {
                    return -1;
                }
            }
        }

    }


    public String toString()
    {
        return "referee = " + referee +
               " referred = " + referred +
               " isThreadReference = " + isThreadReference;
    }
}