/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.io.Serializable;


/**
 * The event enterMethod (described by JVMPI/JVMTI) is identified in the graph,
 * as instances of JBPCall. This is more a class used in calculations than a
 * class used on visualizations. The only visual purpose of this class is to
 * see the currentStack of opened threads (as SPYCalls will still alive after
 * the process finishs).  Behavior during the process: In another words, when
 * the EnterMethod is read into Logfiles, this is created, when ExitMethod is
 * read, the equivalent instance of this class will be deleted.
 *
 * @author Clebert Suconic
 */
public class JBPCall implements Serializable {

    private static final long serialVersionUID = 100L;

    private JBPMethod spyMethod;
    private int methodId;
    private long clockFinish;
    private long clockStart;
    private long elapsedClockStart;
    private long elapsedClockEnd;
    private JBPMethodCount spyMethodCount;
    private long timeTFinish;
    private long timeTStart;


    public JBPCall() {
    }

    public long getTimeTStart() {
        return timeTStart;
    }

    public void setTimeTStart(long timeTStart) {
        this.timeTStart = timeTStart;
    }

    public long getTimeTFinish() {
        return timeTFinish;
    }

    public void setTimeTFinish(long timeTFinish) {
        this.timeTFinish = timeTFinish;
    }

    public long getClockStart() {
        return clockStart;
    }

    public void setClockStart(long clockStart) {
        this.clockStart = clockStart;
    }

    public long getClockFinish() {
        return clockFinish;
    }

    public void setClockFinish(long clockFinish) {
        this.clockFinish = clockFinish;
    }

    public JBPMethod getSpyMethod() {
        return spyMethod;
    }

    public void setSpyMethod(JBPMethod spyMethod) {
        this.spyMethod = spyMethod;
    }

    public int hashCode() {
        return methodId;
    }

    public boolean equals(Object obj) {
        return equals((JBPCall) obj);
    }

    public boolean equals(JBPCall obj) {
        return methodId == obj.methodId;
    }

    public int getMethodId() {
        return methodId;
    }

    public void setMethodId(int methodId) {
        this.methodId = methodId;
    }

    public void setElapsedClockStart(long elapsedClockStart) {
        this.elapsedClockStart = elapsedClockStart;
    }

    public long getElapsedClockStart() {
        return elapsedClockStart;
    }

    public void setElapsedClockEnd(long elapsedClockEnd) {
        this.elapsedClockEnd = elapsedClockEnd;
    }

    public long getElapsedClockEnd() {
        return elapsedClockEnd;
    }

    public JBPMethodCount getSpyMethodCount() {
        return spyMethodCount;
    }

    public void setSpyMethodCount(JBPMethodCount spyMethodCount) {
        this.spyMethodCount = spyMethodCount;
    }

}
