/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.util.HashMap;
import org.jboss.profiler.fileProcessor.*;
import java.util.Date;
import java.io.Serializable;

/**
 * This represents a loaded class, captured by the JVMPI/JVMTI agent.  Extra
 * information are placed here as data is being captured from log-files using
 * {@link org.jboss.profiler.engine.LogFileEngine} This class is related to
 * {@link JBPMethod}
 *
 * @author Clebert Suconic
 */
public class JBPClass implements Serializable {

  private static final long serialVersionUID = 100L;

    private String name;
    /** PK */
    private int classID;

    /**
     *@link aggregation
     *      @associates <{JBPMethod}>
     */
    private HashMap spyMethods = new HashMap();
    private int numberOfCreatedInstances;
    private int numberOfReleasedInstances;
    private Date timestampLoad;

    public String getName(){ return name; }

    public void setName(String name){ this.name = name; }

    public int getClassID(){ return classID; }

    public void setClassID(int classID){ this.classID = classID; }

    public int getNumberOfCreatedInstances(){
            return numberOfCreatedInstances;
        }

    public void setNumberOfCreatedInstances(int numberOfCreatedInstances){
            this.numberOfCreatedInstances = numberOfCreatedInstances;
        }

    public int getNumberOfReleasedInstances(){
            return numberOfReleasedInstances;
        }

    public void setNumberOfReleasedInstances(int numberOfReleasedInstances){
            this.numberOfReleasedInstances = numberOfReleasedInstances;
        }

    /** Method that helps the reading from the LogFiles.*/
    public void setData(SpyRegisterLoadClass spyClass) {
        this.setClassID(spyClass.getClassId());
        this.setName(spyClass.getClassName());
        this.setTimestampLoad(spyClass.getTimestamp());
    }

    public Date getTimestampLoad(){ return timestampLoad; }

    public void setTimestampLoad(Date timestampLoad){ this.timestampLoad = timestampLoad; }

    public String toString() {
        return name;
    }

    public HashMap getSpyMethods(){
            return spyMethods;
        }

    public boolean equals(Object obj) {
        return classID==((JBPClass)obj).classID;
    }

    public int hashCode() {
         return classID;
    }

}
