/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

/**
 * A generic counting class that is used to count Objects and methods callings.
 * A referenced object could be a Method or a Object itself.
 *
 * @author Clebert Suconic
 */
public class JBPGenericCount implements java.io.Serializable {

    private static final long serialVersionUID = 100L;

    /**
     * O Objeto referenciado, conforme documentacao da classe.
     * Pode ser um JBPClass, um JBPMethod ou um JBPThread
     */
    protected Object referencedObject;

    protected int count;

    private long totalClocks;

    private long timeWait;

    private int numberOfWaits;

    /**
     * Clocks desde o inicio do processo, independente do consumo de CPU
     */
    private long totalElapsedClock;

    public JBPGenericCount() {
    }

    public JBPGenericCount(Object referencedObject) {
        this.referencedObject = referencedObject;
    }

    public JBPGenericCount(Object referencedObject, int count) {
        this.referencedObject = referencedObject;
        this.count = count;
    }

    public Object getReferencedObject() {
        return referencedObject;
    }

    public void setReferencedObject(Object referencedObject) {
        this.referencedObject = referencedObject;
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    public final void addTime(long cpuClocks, long elapsedClocks) {
        count++;
        if (cpuClocks > 0) this.totalClocks += cpuClocks;
        if (elapsedClocks > 0) this.totalElapsedClock += elapsedClocks;
    }


    public final void addWaitTime(long timeWait,int numberOfWaits)
    {
        this.timeWait+=timeWait;
        this.numberOfWaits+=numberOfWaits;
    }

    public final void add() {
        count++;
    }

    public void setTotalClocks(long totalClocks) {
        this.totalClocks = totalClocks;
    }

    public long getTotalClocks() {
        return totalClocks;
    }


    /**
     * Usado para fazer consolidacoes
     */
    public void sum(JBPGenericCount count) {
        this.count += count.getCount();
        this.totalClocks += count.getTotalClocks();
        this.totalElapsedClock += count.getTotalElapsedClock();
        this.numberOfWaits += count.getNumberOfWaits();
        this.timeWait += count.getTimeWait();
    }

    public long getTotalElapsedClock() {
        return totalElapsedClock;
    }

    public void setTotalElapsedClock(long totalElapsedClock) {
        this.totalElapsedClock = totalElapsedClock;
    }

    public int getNumberOfWaits() {
        return numberOfWaits;
    }

    public void setNumberOfWaits(int numberOfWaits) {
        this.numberOfWaits = numberOfWaits;
    }

    public long getTimeWait() {
        return timeWait;
    }

    public void setTimeWait(long timeWait) {
        this.timeWait = timeWait;
    }

}
