/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.io.Serializable;

/**
*
@author Clebert Suconic
*/
public class JBPMemoryCount extends JBPGenericCount implements Serializable, Cloneable {

  private static final long serialVersionUID = 100L;

    protected int countRelease;

    /** size in bytes */
    protected int size;

    /** releasedSize in bytes */
    protected int releasedSize;

    public JBPMemoryCount(Object referencedObject) {
        super(referencedObject);
    }

    public JBPMemoryCount(Object referencedObject, int count, int size) {
        super(referencedObject,count);
        this.size=size;
    }

    private JBPMemoryCount() {
    }

    public void sum(JBPMemoryCount instToSum) {
      this.size+=instToSum.getSize();
      this.releasedSize+=instToSum.getReleasedSize();
      this.countRelease+=instToSum.getCountRelease();
      this.count+=instToSum.getCount();
    }

    public final void addSize(int psize) {
        count++;
        size+=psize;
    }

    public boolean equals(Object obj) {
        return referencedObject.equals(((JBPMemoryCount)obj).referencedObject);
    }

    public int hashCode() {
        return referencedObject.hashCode();
    }

    public Object clone() {
        return new JBPMemoryCount(referencedObject,count-countRelease,size-releasedSize);
    }

    public int getCountRelease(){
            return countRelease;
        }

    public void setCountRelease(int countRelease){
            this.countRelease = countRelease;
        }

    public final void addRelease(int bytesRelease) {
        this.countRelease++;
        this.releasedSize += bytesRelease;
    }

    /** size in bytes */
    public int getSize(){
            return size;
        }

    /** size in bytes */
    public void setSize(int size){
            this.size = size;
        }
  public void setReleasedSize(int releasedSize) {
    this.releasedSize = releasedSize;
  }
  public int getReleasedSize() {
    return releasedSize;
  }

}
