/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.profiler.model;


import java.util.*;

import org.jboss.profiler.fileProcessor.*;

import java.io.Serializable;


/**
 * The method loaded within the class and captured from agent through
 * <p/>
 * loadClasses events.
 *
 * @author Clebert Suconic
 */

public class JBPMethod implements Cloneable, Serializable {


    private static final long serialVersionUID = 100L;


    /**
     * PK
     */

    private int methodID;

    private String name;

    private String signature;


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    private long totalClock;

    private int numberOfCalls;

    private boolean opened;

    private Date timestampLoad;

    private int classId;

    private JBPClass spyClass;

    private int numberOfCompletedCalls;

    private long clockInWait;

    private int numberOfWaits;


    /**
     * Threads que acionaram este Thread. Contera uma classe de relacionamento chamada SPYMethodThread
     */

    private ArrayList threadList = new ArrayList();

    private long lastClockMonitoring;


    /**
     * @associates <{JBPGenericCount}>
     */

    private HashMap spyMethodsCalling = new HashMap();


    /**
     * @associates <{JBPGenericCount}>
     */

    private HashMap spyMethodsCallers = new HashMap();

    /*public ArrayList getMethodStackTraceArrayCountLevel() {

      return methodStackTraceArrayCountLevel;

    }*/

    /** ArrayList of HashMaps... by Level identifying a statistic of stack traces */

    //private ArrayList methodStackTraceArrayCountLevel = new ArrayList();


    /**
     * Entenda-se por deep a profundidade das chamadas encadeadas entre metodos
     */

    private int sumDeep;


    /**
     * Entenda-se por deep a profundidade das chamadas encadeadas entre metodos.
     * <p/>
     * Esta medida, o maximo deep atingido
     */

    private int maxDeep;

    private long totalElapsedClock;


    /**
     * Para o calculo das chamadas, chamadas recursivas no devem ser levadas em
     * <p/>
     * conta, para no somar resultados, dando um nmero muito maior que o
     * <p/>
     * esperado.  CurrentCallings eh utilizado efetuar esta comparacao
     */

    private transient int currentCallings;


    /**
     * Para uso de JBPMethod quando em utilizacao na Thread, para localizacoes
     * <p/>
     * dos locais aonde possa haver chamadas ao metodo
     *
     * @associates <{JBPMethodCount}>
     */

    private ArrayList spyMethodCount = new ArrayList();


    public int getMethodID() {
        return methodID;
    }


    public void setMethodID(int methodID) {
        this.methodID = methodID;
    }


    public String getName() {
        return name;
    }


    public void setName(String name) {
        this.name = name;
    }


    public String getFullName() {

        StringBuffer fullName = new StringBuffer();


        if (getSpyClass() != null) {

            fullName.append(getSpyClass().getName());

            fullName.append('.');

        } else {

            fullName.append("Unknow.");

        }

        //fullName+=getName() + getSignature();


        fullName.append(getName());


        org.jboss.profiler.util.SPYConvertUtil.convertParameterList(getSignature(), fullName);


        return fullName.toString();

    }


    /**
     * Only className.methodName (without signature)
     */

    public String getSimplifiedName() {

        String simpleName = "";


        if (getSpyClass() != null) {

            simpleName += getSpyClass().getName() + ".";

        } else {

            simpleName += "Unknow.";

        }

        simpleName += getName();


        return simpleName;

    }


    public String getSignature() {
        return signature;
    }


    public void setSignature(String signature) {
        this.signature = signature;
    }


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    public long getTotalClock() {
        return totalClock;
    }


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    public void setTotalClock(long totalClock) {
        this.totalClock = totalClock;
    }


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    public void addTotalClock(long totalClock) {

        if (totalClock < 0) {

            System.out.println("Can't accept negative values");

        } else {

            this.totalClock += totalClock;

        }

        if (this.totalClock < 0) {

            System.out.println("negative values at metho");

            throw new RuntimeException("Can't accept negative clock");

        }

    }


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    public void addTotalElapsedClock(long ptotalElapsedClock) {
        this.totalElapsedClock += ptotalElapsedClock;
    }


    /**
     * Numero de clocks necessarios para executar o metodo
     */

    public void addClockInWait(long clockInWait) {
        this.clockInWait += clockInWait;
        this.numberOfWaits++;
    }


    public int getNumberOfCalls() {
        return numberOfCalls;
    }


    public void setNumberOfCalls(int numberOfCalls) {
        this.numberOfCalls = numberOfCalls;
    }


    public boolean isOpened() {
        return opened;
    }


    public void setOpened(boolean opened) {
        this.opened = opened;
    }


    public Date getTimestampLoad() {
        return timestampLoad;
    }


    public void setTimestampLoad(Date timestampLoad) {
        this.timestampLoad = timestampLoad;
    }


    public void setData(SpyRegisterLoadMethod data) {

        this.setClassId(data.getClassId());

        this.setName(data.getMethodName());

        this.setMethodID(data.getMethodId());

        this.setSignature(data.getSignature());

        this.setTimestampLoad(data.getTimestamp());

    }


    public Object clone() {

        JBPMethod newmethod = new JBPMethod();

        newmethod.setName(this.getName());

        newmethod.setSignature(this.getSignature());

        newmethod.setClassId(this.getClassId());

        newmethod.setMethodID(this.getMethodID());

        newmethod.setSpyClass(this.getSpyClass());

        // nao clonaremos o array para economizar recursos (memoria)

        //newmethod.methodStackTraceArrayCountLevel=this.methodStackTraceArrayCountLevel;


        return newmethod; // implementar

    }


    public int getClassId() {

        return classId;

    }


    public void setClassId(int classId) {

        this.classId = classId;

    }


    public JBPClass getSpyClass() {
        return spyClass;
    }


    public void setSpyClass(JBPClass spyClass) {
        this.spyClass = spyClass;
    }


    public String toString() {

        return "methodId = " + methodID + " class = " + spyClass + " name = " + name + " (" + signature + ")" +

                " numberOfCalls = " + numberOfCalls + " totalClock = " + totalClock;

    }


    public int hashCode() {

        return methodID;

    }


    public boolean equals(Object obj) {

        return equals((JBPMethod) obj);

    }


    public boolean equals(JBPMethod obj) {

        return methodID == obj.methodID;

    }


    public void addNumberOfCalls(int numberOfCalls) {

        this.numberOfCalls += numberOfCalls;

    }


    public int getNumberOfCompletedCalls() {

        return numberOfCompletedCalls;

    }


    public void setNumberOfCompletedCalls(int numberOfCompletedCalls) {

        this.numberOfCompletedCalls = numberOfCompletedCalls;

    }


    public void addNumberOfCompletedCalls(int numberOfCompletedCalls) {

        this.numberOfCompletedCalls += numberOfCompletedCalls;

    }


    public void setClockInWait(long clockInWait) {

        this.clockInWait = clockInWait;

    }


    public long getClockInWait() {

        return clockInWait;

    }


    /**
     * Soma os valores numericos de um metodo ao atual
     */

    public void sum(JBPMethod methodToSum) {

        this.addNumberOfCalls(methodToSum.getNumberOfCalls());

        this.addNumberOfCompletedCalls(methodToSum.getNumberOfCompletedCalls());

        this.addTotalClock(methodToSum.getTotalClock());

        this.clockInWait = methodToSum.getClockInWait();
        this.numberOfWaits += methodToSum.getNumberOfWaits();

        this.addTotalElapsedClock(methodToSum.getTotalElapsedClock());

    }


    public void addThread(JBPThread thread, JBPMethod methodOnThread) {

        JBPMethodThread methodThread = new JBPMethodThread(thread, this, methodOnThread);

        threadList.add(methodThread);

    }


    public Collection getThreadList() {

        return threadList;

    }


    /**
     * Soma quantidade de clocks na lista de Threads em que o metodo foi executado
     */

    public long sumTotalElapsedTimeOnThreadList() {

        Iterator iter = getThreadList().iterator();

        long totalElapsedClocks = 0;

        while (iter.hasNext()) {

            JBPMethodThread thread = (JBPMethodThread) iter.next();

            totalElapsedClocks += (thread.getSpyThread().getElapsedClockFinish() - thread.getSpyThread().getElapsedClockStart());

        }

        return totalElapsedClocks;

    }


    /**
     * Last clock when Monitoring entered
     */

    public void setLastClockMonitoring(long lastClockMonitoring) {

        this.lastClockMonitoring = lastClockMonitoring;

    }

    /**
     * Last clock when Monitoring entered
     */

    public long getLastClockMonitoring() {

        return lastClockMonitoring;

    }


    public HashMap getSpyMethodsCalling() {

        return spyMethodsCalling;

    }


    public HashMap getSpyMethodsCallers() {

        return spyMethodsCallers;

    }


    public void setSumDeep(int sumDeep) {

        this.sumDeep = sumDeep;

    }


    public int getSumDeep() {

        return sumDeep;

    }


    public void setMaxDeep(int maxDeep) {

        this.maxDeep = maxDeep;

    }


    public int getMaxDeep() {

        return maxDeep;

    }


    public void setTotalElapsedClock(long totalElapsedClock) {

        this.totalElapsedClock = totalElapsedClock;

    }


    public long getTotalElapsedClock() {

        return totalElapsedClock;

    }


    public void setCurrentCallings(int currentCallings) {

        this.currentCallings = currentCallings;

    }


    public int getCurrentCallings() {

        return currentCallings;

    }


    /**
     * Veja documentacao no atributo currentCallings
     */

    public void addCallingOnThread() {

        currentCallings++;

    }


    /**
     * Veja documentacao no atributo currentCallings
     */

    public void releaseCallingOnThread() {

        if (--currentCallings < 0) currentCallings = 0;


    }

    public ArrayList getSpyMethodCount() {

        return spyMethodCount;

    }


    public void addSpyMethodCount(JBPMethodCount methodCount) {

        getSpyMethodCount().add(methodCount);

    }

    public int getNumberOfWaits() {
        return numberOfWaits;
    }

    public void setNumberOfWaits(int numberOfWaits) {
        this.numberOfWaits = numberOfWaits;
    }

}

