/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.Serializable;

import org.jboss.profiler.filecollection.FileCollectionInterface;

/**
 * An objected allocated by the applications.
 * This methods also marks the history where it was created using graphPathCreation.
 * As lots of objects are created we will place the Collection using TemporaryFiles. (FileCollection)
@author Clebert Suconic
*/
public class JBPObject implements Serializable, FileCollectionInterface {

	private static final long serialVersionUID = 100L;

	private int bytesAllocated;
	private int classId;
	private int graphPathCreation[];
	private int objectId;

	private int threadIdCreated;

	public JBPObject(int classId, int objectId, int bytesAllocated) {
		this.classId = classId;
		this.objectId = objectId;
		this.bytesAllocated = bytesAllocated;
	}

	public JBPObject() {
	}

	public boolean equals(Object obj) {
		return objectId == ((JBPObject) obj).objectId;
	}
	public int getBytesAllocated() {
		return bytesAllocated;
	}

	/**
	 * @return
	 */
	public int getClassId() {
		return classId;
	}

	/**
	 * The path of methodIds to the place where this objects was created
	 * @return
	 */
	public int[] getGraphPathCreation() {
		return graphPathCreation;
	}

	public int getObjectId() {
		return objectId;
	}

	/**
	 * @return
	 */
	public int getThreadIdCreated() {
		return threadIdCreated;
	}

	public int hashCode() {
		return objectId;
	}


	/**
	 * @param classId
	 */
	public void setClassId(int classId) {
		this.classId = classId;
	}

	/**
	 * The path of methodIds to the place where this objects was created
	 * @param graphPathCreation
	 */
	public void setGraphPathCreation(int[] graphPathCreation) {
		this.graphPathCreation = graphPathCreation;
	}

	public void setObjectId(int objectId) {
		this.objectId = objectId;
	}

	/**
	 * @param threadIdCreated
	 */
	public void setThreadIdCreated(int threadIdCreated) {
		this.threadIdCreated = threadIdCreated;
	}

	/*
	 * @see br.com.summatech.serverSpy.util.FileCollectionInterface#writeObject(java.io.DataOutput)
	 */
	public void writeObject(DataOutput ch) throws IOException {
		ch.writeInt(bytesAllocated);
		ch.writeInt(classId);
		ch.writeInt(objectId);
		ch.writeInt(threadIdCreated);

		ch.writeInt(graphPathCreation.length);

		for (int i=0;i<graphPathCreation.length;i++) {
		     ch.writeInt(graphPathCreation[i]);
		}
	}

	/*
	 * @see br.com.summatech.serverSpy.util.FileCollectionInterface#readObject(java.io.DataInput)
	 */
	public void readObject(DataInput ch) throws IOException {
		bytesAllocated = ch.readInt();
		classId = ch.readInt();
		objectId = ch.readInt();
		threadIdCreated = ch.readInt();


		graphPathCreation = new int [ch.readInt()];

		for (int i=0;i<graphPathCreation.length;i++) {
			graphPathCreation[i] = ch.readInt();
		}
	}

}
