/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.Serializable;

import org.jboss.profiler.filecollection.FileCollectionInterface;

/**
 * Every object allocated will generate a instance of JBPObjectInstance, and
 * this is used to find memory-leaks, and non released objects, as we can only
 * have information about those IDs.  As lots of objects is being created in a
 * Java application, we will use {@link FileCollection} class as the
 * Collection container.
@author Clebert Suconic
*/
public class JBPObjectInstance implements FileCollectionInterface, Comparable, Serializable {

	private static final long serialVersionUID = 100L;

	/** The ObjectId of the GC operation*/
	private int objectId;
	/** The id of the GC happened*/
	private int gcNumber;

	/**
	 * @param gcNumber
	 * @param objectId
	 */
	public JBPObjectInstance(int gcNumber, int objectId) {
		this.gcNumber = gcNumber;
		this.objectId = objectId;
	}

	public JBPObjectInstance() {
	}

	/**
	 * @return
	 */
	public int getGcNumber() {
		return gcNumber;
	}

	/**
	 * @param gcNumber
	 */
	public void setGcNumber(int gcNumber) {
		this.gcNumber = gcNumber;
	}

	/**
	 * @return
	 */
	public int getObjectId() {
		return objectId;
	}

	/**
	 * @param objectId
	 */
	public void setObjectId(int objectId) {
		this.objectId = objectId;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		return objectId - ((JBPObjectInstance)o).objectId;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o) {
		return objectId - ((JBPObjectInstance)o).objectId==0;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return objectId;
	}

	/* (non-Javadoc)
	 * @see br.com.summatech.serverSpy.util.FileCollectionInterface#readObject(java.io.DataInput)
	 */
	public void readObject(DataInput ch) throws IOException {
		objectId = ch.readInt();
		gcNumber = ch.readInt();

	}

	/* (non-Javadoc)
	 * @see br.com.summatech.serverSpy.util.FileCollectionInterface#writeObject(java.io.DataOutput)
	 */
	public void writeObject(DataOutput ch) throws IOException {
		ch.writeInt(objectId);
		ch.writeInt(gcNumber);
	}

}
