/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.model;

import java.util.*;
import java.io.PrintStream;
import java.io.Serializable;
import org.jboss.profiler.fileProcessor.SpyRegisterBeginThread;
import org.jboss.profiler.fileProcessor.SpyRegisterEndThread;
import org.jboss.profiler.util.SPYConsts;

/**
 * A thread captured by the agent. That contains also informations about objects and methods loaded into current thread.
@author Clebert Suconic
*/
public class JBPThread implements PrintDebug, Serializable, SPYConsts {

    private static final long serialVersionUID = 100L;

    private String name;

    /** @associates <{JBPClass}>*/
    private HashMap spyObjects;

    /**
     * Usa Integer com methodId para identificar a HashMap
     * @associates <{JBPMethod}>*/
    private HashMap spyMethods = new HashMap();
    private int numberOfOpenedMethods;
    private int numberOfClosedMethods;
    private int threadId;
    private int envId;
    private java.util.Date startTimestamp;
    private Date finishTimestamp;
    private boolean closed;

    /**
     *@link aggregation
     *      @associates <{JBPCall}>
     */
    private Stack spyCalls = new Stack();
    private transient java.io.PrintStream detailedOutput;
    private transient java.io.File detailedOutputFile;
    private long startClock=-1; // nao inicializado
    private long finishClock;

  /**
   * Se showRecords, indica com 0 quantos niveis autorizados existem na lista
   * de detalhe de output (usualmente 0 ou 1, mas podendo ser mais por
   * coincidencia de selecao de usuario (visualiza A que aciona B (que tb sera
   * visualizado))
   */
  private int levelAuthShowRecords=-1;

  /**
   * Embora tecnicamente so pode haver um root em uma Thread, mantemos em HashMap
   * ja que o processo pode comecar a capturar na metade do processo, podendo aparecer novos roots (em funcao do JVMPI ser inicializado antes)
   * @associates <{JBPMethodCount}>
   * @label roots
   */
  private HashMap spyMethodCountRoots = new HashMap();
  private long elapsedClockStart;
  private long elapsedClockFinish;

  /** Last time_t captured from thread. (for using on closeThreads)*/
  private long lastTime_T;

    public void setData(SpyRegisterBeginThread thread) {
        this.setName(thread.getThreadName());
        this.setThreadId(thread.getThreadId());
        this.setEnvId(thread.getEnvId());
        //this.setStartTimestamp(thread.getTimestamp());
        //if (this.getStartClock()<0) this.setStartClock(thread.getClock());
        //this.setElapsedClockStart(thread.getElapsedClock());
    }

    public void setData(SpyRegisterEndThread thread) {
        //this.setFinishTimestamp(thread.getTimestamp());
        //this.setFinishClock(thread.getClock());
        //this.setElapsedClockFinish(thread.getElapsedClock());
        this.setClosed(true);
    }

    public String getName(){ return name; }

    public void setName(String name){ this.name = name; }

    public int getNumberOfOpenedMethods(){
            return numberOfOpenedMethods;
        }

    public void setNumberOfOpenedMethods(int numberOfOpenedMethods){
            this.numberOfOpenedMethods = numberOfOpenedMethods;
        }

    public int getNumberOfClosedMethods(){
            return numberOfClosedMethods;
        }

    public void setNumberOfClosedMethods(int numberOfClosedMethods){
            this.numberOfClosedMethods = numberOfClosedMethods;
        }


    public int getThreadId(){
            return threadId;
        }

    public void setThreadId(int threadId){
            this.threadId = threadId;
        }

    public int getEnvId(){
            return envId;
        }

    public void setEnvId(int envId){
            this.envId = envId;
        }

    public java.util.Date getStartTimestamp(){ return startTimestamp; }

    public void setStartTimestamp(Date startTimestamp){ this.startTimestamp = startTimestamp; }

    public int hashCode() {
        return threadId;
    }

    public boolean equals(Object obj) {
        return equals((JBPThread)obj);
    }

    public boolean equals(JBPThread obj) {
        return threadId==obj.threadId;
    }

    public long getStartClock(){
            return startClock;
        }

    public void setStartClock(long startClock){
            this.startClock = startClock;
        }

    public Date getFinishTimestamp(){ return finishTimestamp; }

    public void setFinishTimestamp(Date finishTimestamp){ this.finishTimestamp = finishTimestamp; }

    public long getFinishClock(){
            return finishClock;
        }

    public void setFinishClock(long finishClock){
            this.finishClock = finishClock;
        }

  public void addLevelAuthShow() {
     levelAuthShowRecords++;
  }

  public void releaseLevelAuthShow() {
     levelAuthShowRecords--;
  }


    public String toString() {
        return "threadId = " +threadId + " envId = " + envId + " startTimestamp = " + startTimestamp + " finishTimestamp = " + finishTimestamp + " threadName = "+ name + "closed = " + (closed?"closed":"opened");
    }

    public boolean isClosed(){ return closed; }

    public void setClosed(boolean closed){ this.closed = closed; }

    public HashMap getSpyMethods(){
            return spyMethods;
        }

    public void setSpyMethods(HashMap spyMethods){
            this.spyMethods = spyMethods;
        }


    /** */
    public JBPMethodCount findMethodPath(int methodIds[],JBPProcess spyProcess, JBPMethod tmpKeyMethod) {

        if (methodIds.length==0) return null;

        JBPMethodCount currentCount = null;
        for (int i=0;i<methodIds.length;i++) {
			JBPMethod methodFound = spyProcess.findMethod(tmpKeyMethod,methodIds[i]);
        	if (methodFound==null) return null;
        	if (i==0) currentCount = (JBPMethodCount) this.getSpyMethodCountRoots().get(methodFound);
        	else currentCount = currentCount.getCalledCount(methodFound);

        	if (currentCount==null) break;
        }


    	return currentCount;

    }

    /** Using the current Stack of SpyCalss find the botton JBPMethodCount */
    public JBPMethodCount findMethodPath() {
        if (spyCalls.isEmpty())
        {
            return null;
        }

        // Ao invez de fazer a busca em toda a arvore, simplificamos a busca, procurando em spyCalls superiores, a ja existencia
        // de JBPMethodCount preenchido, economizando assim uma grande quantidade de CPU
        int pathPreenchido=0;
        for (pathPreenchido=spyCalls.size()-1;pathPreenchido>=0;pathPreenchido--) {
            JBPCall spyCall = (JBPCall)spyCalls.get(pathPreenchido);
            if (spyCall.getSpyMethodCount()!=null) {
                break;
            }
        }

        JBPMethodCount count = null;
        JBPMethodCount count2 = null;
        JBPCall tmpCall = null;
        JBPCall tmpCall2 = null;
        if (pathPreenchido<0) {
            tmpCall = (JBPCall) spyCalls.get(0);
            count = (JBPMethodCount) this.getSpyMethodCountRoots().get(tmpCall.getSpyMethod());
            if (count==null) {
              count = new JBPMethodCount(tmpCall.getSpyMethod());
              this.getSpyMethodCountRoots().put(tmpCall.getSpyMethod(),count);
              tmpCall.getSpyMethod().addSpyMethodCount(count);
            }
            pathPreenchido=0;
        } else {
            tmpCall = (JBPCall) spyCalls.get(pathPreenchido);
            count = tmpCall.getSpyMethodCount();
            if (count==null) {
              throw new RuntimeException("Error on algorithm... variable count canoot bet null at this point");
            }
        }

        final int lastIndex = spyCalls.size()-2;

        for (;pathPreenchido<=lastIndex;pathPreenchido++){
           tmpCall=(JBPCall)spyCalls.get(pathPreenchido);
           tmpCall2=(JBPCall)spyCalls.get(pathPreenchido+1);
           if (inDebug) {
              System.out.println("call_1 = " + tmpCall.getSpyMethod().getFullName() + " calling call_2 = "  + tmpCall2.getSpyMethod().getFullName());
           }
           count2 = count.getCalledCount(tmpCall2.getSpyMethod());
           count2.getCallerCount(tmpCall.getSpyMethod());

           if (inDebug) {
              System.out.println("Count   = " + count.getSpyMethod().getFullName() + " -> " );
              System.out.println("**Count2= " + count2.getSpyMethod().getFullName());
           }
           tmpCall.setSpyMethodCount(count);
           count=count2;
        }

        return count;
    }

    public void printDebug(PrintStream out){
        out.println("Methods called on Thread " + threadId);
        java.util.Iterator iteratorInstance = spyMethods.values().iterator();
        while (iteratorInstance.hasNext()) {
            System.out.println(iteratorInstance.next());
        }
    }

    public Stack getSpyCalls(){
            return spyCalls;
    }

  /** Consolida a arvore de chamadas para o Thread corrente */
  public JBPMethodCount consolidaCallings(JBPMethod method) {
     JBPMethodCount countConsolidado = new JBPMethodCount (method);

      JBPMethod methodOnThread = (JBPMethod)this.getSpyMethods().get(method);

      Iterator counts = methodOnThread.getSpyMethodCount().iterator();
      while (counts.hasNext()) {
         JBPMethodCount spyMethodCountTmp = (JBPMethodCount)counts.next();
         if (inDebug) spyMethodCountTmp.printDebugCalleds(0);
         copyCount(countConsolidado,spyMethodCountTmp);
      }

     return countConsolidado;
  }

  /** Efetua a copia da arvore */
  private void copyCount(JBPMethodCount countConsolidado,JBPMethodCount spyMethodCountToCopy) {
     countConsolidado.sum(spyMethodCountToCopy);

      // copiando informacoes sobre objetos criados
      Iterator iterMemoryCount = spyMethodCountToCopy.getCountMemory().values().iterator();
      while (iterMemoryCount.hasNext()) {
        JBPMemoryCount memCount = (JBPMemoryCount)iterMemoryCount.next();
        countConsolidado.getCountForClass((JBPClass)memCount.getReferencedObject()).sum(memCount);
      }


     // Tratamento calleds
     Iterator iterCalled = spyMethodCountToCopy.getCountCalleds().values().iterator();
     while (iterCalled.hasNext()) {
        JBPMethodCount methodCountChildToCopy = (JBPMethodCount)iterCalled.next();
        JBPMethodCount methodCountChildConsolidado= (JBPMethodCount)countConsolidado.getCountCalleds().get(methodCountChildToCopy.getSpyMethod());

        if (methodCountChildConsolidado==null) {
           JBPMethod methodChildOnProcess = (JBPMethod)this.getSpyMethods().get(methodCountChildToCopy.getSpyMethod());
           methodCountChildConsolidado = new JBPMethodCount(methodChildOnProcess);
           countConsolidado.getCountCalleds().put(methodChildOnProcess,methodCountChildConsolidado);
        }


        copyCount(methodCountChildConsolidado,methodCountChildToCopy);

     }

     /*Iterator iterCaller = spyMethodCountToCopy.getCountCallers().values().iterator();
     while (iterCaller.hasNext()) {
        JBPMethodCount methodCountChildToCopy = (JBPMethodCount)iterCalled.next();
        JBPMethodCount methodCountChildConsolidado= (JBPMethodCount)countConsolidado.getCountCalleds().get(methodCountChildToCopy.getSpyMethod());

        if (methodCountChildConsolidado==null) {
           JBPMethod methodChildOnProcess = this.getSpyMethods().get(methodCountChildConsolidado.getSpyMethod());
           methodCountChildConsolidado = new JBPMethodCount(methodChildOnProcess);
           countConsolidado.getCountCalleds().put(methodChildOnProcess,methodCountChildConsolidado);
        }

        copyCount(methodCountChildConsolidado,methodCountChildToCopy);

     }*/

  }

  public void setDetailedOutput(java.io.PrintStream detailedOutput) {
    this.detailedOutput = detailedOutput;
  }
  public java.io.PrintStream getDetailedOutput() {
    return detailedOutput;
  }
  public void setDetailedOutputFile(java.io.File detailedOutputFile) {
    this.detailedOutputFile = detailedOutputFile;
  }
  public java.io.File getDetailedOutputFile() {
    return detailedOutputFile;
  }

  public JBPMethod getCurrentMethod() {
    if (spyCalls.size()==0) return null;
    JBPCall spyCall = (JBPCall)this.spyCalls.peek();
    return spyCall.getSpyMethod();
  }
  public int getLevelAuthShowRecords() {
    return levelAuthShowRecords;
  }
  public void setLevelAuthShowRecords(int levelAuthShowRecords) {
    this.levelAuthShowRecords = levelAuthShowRecords;
  }
  public void setElapsedClockStart(long elapsedClockStart) {
    this.elapsedClockStart = elapsedClockStart;
  }
  public long getElapsedClockStart() {
    return elapsedClockStart;
  }
  public void setElapsedClockFinish(long elapsedClockFinish) {
    this.elapsedClockFinish = elapsedClockFinish;
  }
  public long getElapsedClockFinish() {
    return elapsedClockFinish;
  }
  public HashMap getSpyMethodCountRoots() {
    return spyMethodCountRoots;
  }


  /**
   * @return
   */
  public long getLastTime_T() {
	  return lastTime_T;
  }

  /**
   * @param l
   */
  public void setLastTime_T(long l) {
	  lastTime_T = l;
  }

}
