/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.profiler.tracerModel;

import java.util.*;
import org.jboss.profiler.fileProcessor.*;
import java.io.Serializable;

/**
 * @author Yoko Seki (y-seki@hitachi.co.jp)
 * @author Clebert Suconic (clebert.suconic@jboss.com) - Added locking measurement
 */
public class JBTMethod implements Serializable
{
   private int methodID;
   private int depth;
   private long elapsedClockStart;
   private long elapsedClockEnd;
   private long clockStart;
   private long clockEnd;
   private long lockEnter=Long.MAX_VALUE;
   private long lockEntered=Long.MAX_VALUE;
   private long lockTime=0;
   private int numberOfLocks=0;

   public int getMethodID () { return methodID;}
   public void setMethodID ( int methodID ) { this.methodID = methodID; }

   public int getDepth () { return depth; }
   public void setDepth ( int depth ) { this.depth = depth; };

   public long getElapsedClockStart () {
      return elapsedClockStart;
   }
   public void setElapsedClockStart (long elapsedClockStart) {
      this.elapsedClockStart = elapsedClockStart;
   }

   public long getElapsedClockEnd () {
      return elapsedClockEnd;
   }
   public void setElapsedClockEnd (long elapsedClockEnd) {
      this.elapsedClockEnd = elapsedClockEnd;
   }

   public long getClockStart () {
      return clockStart;
   }
   public void setClockStart (long clockStart) {
      this.clockStart = clockStart;
   }

   public long getClockEnd () {
      return clockEnd;
   }
   public void setClockEnd (long clockEnd) {
      this.clockEnd = clockEnd;
   }

    public long getLockEnter()
    {
        return lockEnter;
    }

    public void setLockEnter(long lockEnter)
    {
        this.lockEnter = lockEnter;
    }

    public long getLockEntered()
    {
        return lockEntered;
    }

    public void setLockEntered(long lockEntered)
    {
        this.lockEntered = lockEntered;
    }

    public long getLockTime()
    {
        return lockTime;
    }

    public void setLockTime(long lockTime)
    {
        this.lockTime = lockTime;
    }

    public int getNumberOfLocks()
    {
        return numberOfLocks;
    }

    public void setNumberOfLocks(int numberOfLocks)
    {
        this.numberOfLocks = numberOfLocks;
    }

    public void setDataStart (SpyRegisterEnterMethod data)
   {
      this.setMethodID(data.getMethodId());
      this.setElapsedClockStart(data.getElapsedClock()+SpyRegister.startOfProcess);
      this.setElapsedClockEnd(-1);  //temporary value
      this.setClockStart(data.getClock());
      this.setClockEnd(-1);  //temporary value
   }

   public void setDataEnd (SpyRegisterExitMethod data)
   {
      this.setElapsedClockEnd(data.getElapsedClock()+SpyRegister.startOfProcess);
      this.setClockEnd(data.getClock());
   }

}

