/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.util;

import java.util.Comparator;
import org.jboss.profiler.model.*;

/** Compara instancias de JBPMethod, utilizando um parametro para identificar o parametro de comparacao
@author Clebert Suconic
*/
public class SPYMethodComparator implements Comparator {
  String order;
  public SPYMethodComparator(String order) {
     this.order=order;
  }

  public int compare(Object o1, Object o2) {
                     JBPMethod s1 = (JBPMethod)o1;
                     JBPMethod s2 = (JBPMethod)o2;
                     int ret = 0;

                     if (order.equals("cpu")) {
                       long tmpRet = (s2.getTotalClock() -
                                 s1.getTotalClock());

                       if (tmpRet>0) ret=1;
                       else if (tmpRet<0) ret=-1;
                     } else if (order.equals("time")){
                       ret = (int)(s2.getTotalElapsedClock() -
                                 s1.getTotalElapsedClock());
                     } else if (order.equals("maxDeep")){
                       ret = (int)(s2.getMaxDeep() -
                                 s1.getMaxDeep());
                     } else if (order.equals("avgDeep") && s1.getNumberOfCalls()>0 && s2.getNumberOfCalls()>0){
                       ret = (int)(s2.getSumDeep()/s2.getNumberOfCalls() -
                                 s1.getSumDeep()/s1.getNumberOfCalls());
                     } else if (order.equals("calls") && s1.getNumberOfCalls()>0 && s2.getNumberOfCalls()>0){
                       ret = (int)(s2.getNumberOfCalls() - s1.getNumberOfCalls());
                     } else if (order.equals("lock"))
                     {
                       if (s2.getClockInWait()>s1.getClockInWait())
                       {
                           ret=1;
                       } else
                       if (s2.getClockInWait()<s1.getClockInWait())
                       {
                           ret=-1;
                       } else
                       {
                           ret=0;
                       }
                     } else if (order.equals("nrLocks"))
                     {
                         ret = s2.getNumberOfWaits() - s1.getNumberOfWaits();
                     }


                     if (ret!=0) {
                        if (ret>0) return 1;
                        else if (ret<0) return -1;
                        else return 0;
                     }
                     if ( s1.getName()!=null &&
                         s2.getName()!=null && s1.getSpyClass()!=null && s2.getSpyClass()!=null) {
                         return (s1.getSpyClass().getName() + s1.getName() + s1.getSignature()).compareTo(
                                  s2.getSpyClass().getName() + s2.getName() + s2.getSignature()
                                 );
                     } else {
                         return s2.getMethodID() - s1.getMethodID();
                     }
  }
}