/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.util;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import org.jboss.profiler.model.JBPClass;
import org.jboss.profiler.model.JBPGenericCount;
import org.jboss.profiler.model.JBPMemoryCount;
import org.jboss.profiler.model.JBPMethod;
import org.jboss.profiler.model.JBPMethodCount;
import org.jboss.profiler.model.JBPProcess;

/**
 * @author Clebert Suconic
 */
public class XmlGenerationUtil {
    private static final String IDENT_ON_LEVEL = "     ";

    public XmlGenerationUtil() {
        super();
    }

    /** Construct a list of existent roots */
    private static TreeSet filterAndConsolidateRoots(JBPProcess spyProcess) {
        TreeSet setRoots = new TreeSet(new SPYMethodCountComparator());
        HashMap roots = spyProcess.consolidateRoots();
        Iterator iterRoots = roots.values().iterator();

        while (iterRoots.hasNext()) {
            JBPGenericCount rootInstance = (JBPGenericCount) iterRoots.next();
            JBPMethod methodOnProcess = (JBPMethod) spyProcess.getSpyMethods()
                    .get(rootInstance.getReferencedObject());
            JBPGenericCount rootConsolidado = spyProcess
                    .consolidaCallings(methodOnProcess);
            setRoots.add(rootConsolidado);
        }

        return setRoots;
    }


    public static void generateXML(String className, PrintStream out, JBPProcess process,
            double minPercent, int maxLevel, boolean printMemory,
            boolean memoryLeaks, java.text.NumberFormat numberFormat) {

        JBPClass jbpClass = process.findClassByName(className);

        if (jbpClass!=null) {

            ArrayList listCounts = aggregateMethods(process, jbpClass);

            out.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
            out.print("<class name=\"" + className + "\"");
            out.println(">");
            internalGenerateXML(out,process,minPercent,maxLevel,printMemory,memoryLeaks,numberFormat,listCounts.iterator());
            out.println("</class>");
            return;
        }

    }

    /**
     * @param out
     * @param process
     * @param minPercent
     * @param maxLevel
     * @param printMemory
     * @param memoryLeaks
     * @param numberFormat
     */
    public static void generateXML(PrintStream out, JBPProcess process,
            double minPercent, int maxLevel, boolean printMemory,
            boolean memoryLeaks, java.text.NumberFormat numberFormat) {
        TreeSet setRoots = filterAndConsolidateRoots(process);
        Iterator methodsIterator = setRoots.iterator();

        out.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
        out.print("<PID pid=\"" + process.getPid() + "\"");
        out.print(" cpuClock=\"" + process.getTotalClocks() + "\"");
        out.print(" elapsedTime=\"" + process.getTotalElapsedClocks() + "\"");
        out.println(">");
        internalGenerateXML(out, process, minPercent, maxLevel, printMemory, memoryLeaks, numberFormat, methodsIterator);
        out.println("</PID>");
    }

    /**
     * @param process
     * @param jbpClass
     * @return
     */
    private static ArrayList aggregateMethods(JBPProcess process, JBPClass jbpClass) {
        ArrayList listCounts = new ArrayList();

        Iterator iterMethods = jbpClass.getSpyMethods().values().iterator();
        while (iterMethods.hasNext()) {
            JBPMethod method = (JBPMethod)iterMethods.next();
            JBPGenericCount consolidation = process.consolidaCallings(method);
            listCounts.add(consolidation);
        }
        return listCounts;
    }


    /**
     * @param out
     * @param process
     * @param minPercent
     * @param maxLevel
     * @param printMemory
     * @param memoryLeaks
     * @param numberFormat
     * @param methodsIterator
     */
    private static void internalGenerateXML(PrintStream out, JBPProcess process, double minPercent, int maxLevel, boolean printMemory, boolean memoryLeaks, java.text.NumberFormat numberFormat, Iterator methodsIterator) {


        while (methodsIterator.hasNext()) {
            JBPMethodCount count = (JBPMethodCount) methodsIterator.next();

            openTree(maxLevel, minPercent, 0, out, count, count, process,
                    numberFormat, printMemory);

        }

        if (memoryLeaks)
            printMemoryLeaks(out, process);

    }

    /** Encode que remove <>$ */
    private static String myEncode1(String stringToEncode) {
        char[] chars = stringToEncode.toCharArray();

        StringBuffer stringReturn = new StringBuffer(chars.length);
        for (int i = 0; i < chars.length; i++) {
            if ("<>$".indexOf(chars[i]) == -1)
                stringReturn.append(chars[i]);
        }

        return stringReturn.toString();
    }

    /** Encode que troca <>por &lt; e &gt; */
    private static String myEncode2(String stringToEncode) {
        char[] chars = stringToEncode.toCharArray();

        StringBuffer stringReturn = new StringBuffer(chars.length);
        for (int i = 0; i < chars.length; i++) {
            if (chars[i] == '<')
                stringReturn.append("&lt;");
            else if (chars[i] == '>')
                stringReturn.append("&gt;");
            else
                stringReturn.append(chars[i]);
        }

        return stringReturn.toString();
    }

    /** @return True if the method have the percentage ok */
    private static boolean openMethod(double minPercentage, int level,
            PrintStream out, JBPMethodCount count, JBPMethodCount rootMethod,
            JBPProcess process, boolean isRoot,
            java.text.NumberFormat numberFormat) {
        JBPMethod methodOnProcess = (JBPMethod) process.getSpyMethods().get(
                count.getSpyMethod());
        double totalClocks = 0, totalTime = 0;
        if (isRoot) {
            totalClocks = process.getTotalClocks();
            totalTime = process.getTotalElapsedClocks();
        } else {
            totalClocks = rootMethod.getTotalClocks();
            totalTime = rootMethod.getTotalElapsedClock();
        }

        double cpuPercentValue = 0.0;

        double timePercentValue = 0.0;

        /*
         * if (isRoot) { cpuPercentValue =
         * (((double)methodOnProcess.getTotalClock()) / totalClocks) *
         * (double)100; timePercentValue =
         * (((double)methodOnProcess.getTotalElapsedClock()) / totalTime) *
         * (double)100; } else { cpuPercentValue =
         * (((double)count.getTotalClocks()) / totalClocks) * (double)100;
         * timePercentValue = (((double)count.getTotalElapsedClock()) /
         * totalTime) * (double)100; } -- trecho desabilitado, depois que
         * mudamos a rotina para enviar apenas consolidacoes
         */

        cpuPercentValue = (((double) count.getTotalClocks()) / totalClocks)
                * (double) 100;
        timePercentValue = (((double) count.getTotalElapsedClock()) / totalTime)
                * (double) 100;

        String cpuPercentual = numberFormat.format(cpuPercentValue);
        String timePercentual = numberFormat.format(timePercentValue);

        if (minPercentage <= 0 || cpuPercentValue > minPercentage
                || timePercentValue > minPercentage) {
            for (int i = 0; i < level; i++) {
                out.print(IDENT_ON_LEVEL);
            }

            out.print("<method name=\"" + myEncode1(methodOnProcess.getName())
                    + "\"");
            out.print(" cpuPercenptual=\"" + cpuPercentual + "\"");
            out.print(" timePercenptual=\"" + timePercentual + "\"");
            out.print(" fullName=\"" + myEncode2(methodOnProcess.getFullName())
                    + "\"");
            out.println(">");

            return true;
        } else {
            return false;
        }
    }

    private static void openTree(int maxLevel, double minPercentage, int level,
            PrintStream out, JBPMethodCount count, JBPMethodCount root,
            JBPProcess process, java.text.NumberFormat numberFormat,
            boolean printMemory) {
        if (!openMethod(minPercentage, level, out, count, root, process,
                root == count, numberFormat))
            return;

        TreeSet setMethods = new TreeSet(new SPYMethodCountComparator());
        setMethods.addAll(count.getCountCalleds().values());

        Iterator subMethods = setMethods.iterator();

        while (subMethods.hasNext()) {
            JBPMethodCount subCount = (JBPMethodCount) subMethods.next();
            if (maxLevel == 0 || level < maxLevel)
                openTree(maxLevel, minPercentage, level + 1, out, subCount,
                        root, process, numberFormat, printMemory);
        }

        if (printMemory)
            printMemory(level, out, count);

        closeMethod(level, out, count, process);
    }

    private static void printLevel(PrintStream out, int level) {
        for (int i = 0; i < level; i++) {
            out.print(IDENT_ON_LEVEL);
        }
    }

    private static void printMemory(int level, PrintStream out, JBPMethodCount count) {
        TreeSet setMemory = null;

        // para ordenar pelo maior tempo
        setMemory = new TreeSet(new Comparator() {
            public int compare(Object o1, Object o2) {
                JBPMemoryCount s1 = (JBPMemoryCount) o1;
                JBPGenericCount s2 = (JBPGenericCount) o2;
                int x = s2.getCount() - s1.getCount();
                if (x != 0) {
                    return x;
                } else if (((JBPClass) s1.getReferencedObject()).getName() != null
                        && ((JBPClass) s2.getReferencedObject()).getName() != null) {
                    return ((JBPClass) s1.getReferencedObject()).getName()
                            .compareTo(
                                    ((JBPClass) s2.getReferencedObject())
                                            .getName());
                } else {
                    return ((JBPClass) s1.getReferencedObject()).getClassID()
                            - ((JBPClass) s2.getReferencedObject())
                                    .getClassID();
                }
            }
        });

        HashMap hashMemory = count.createConsolidatedObjects();
        setMemory.addAll(hashMemory.values());

        printLevel(out, level);
        out.println("<memory>");

        Iterator iter = setMemory.iterator();
        while (iter.hasNext()) {
            JBPMemoryCount iterCount = (JBPMemoryCount) iter.next();
            JBPClass spyClass = (JBPClass) iterCount.getReferencedObject();
            for (int i = 0; i <= level; i++) {
                out.print(IDENT_ON_LEVEL);
            }
            out.println("<memoryItem instanceOf=\"" + spyClass.getName()
                    + "\" instances=\"" + iterCount.getCount() + "\" bytes=\""
                    + iterCount.getSize() + "\"" + " releasedInstances=\""
                    + iterCount.getCountRelease() + "\" releasedBytes=\""
                    + iterCount.getReleasedSize() + "\"/>");

        }

        for (int i = 0; i < level; i++) {
            out.print(IDENT_ON_LEVEL);
        }
        out.println("</memory>");

    }

    private static void printMemoryLeakMethod(int level, PrintStream out,
            JBPMethodCount count) {
        printLevel(out, level);
        out.println("<methodOnMemory methodName=\""
                + myEncode2(count.getSpyMethod().getName()) + "\""
                + " instances=\"" + count.getCount() + "\"" + " fullName=\""
                + myEncode2(count.getSpyMethod().getFullName()) + "\"" + ">");

        Iterator iterValues = count.getCountCalleds().values().iterator();
        while (iterValues.hasNext()) {
            JBPMethodCount methodCount = (JBPMethodCount) iterValues.next();
            printMemoryLeakMethod(level + 1, out, methodCount);
        }

        printLevel(out, level);
        out.println("</methodOnMemory>");
    }

    private static void printMemoryLeaks(PrintStream out, JBPProcess spyProcess) {
        printLevel(out, 1);
        out.println("<memoryOperations>");
        // para ordenar pelo maior tempo

        Iterator iter = spyProcess.getSpyObjectTreeByClass().keySet()
                .iterator();

        while (iter.hasNext()) {
            JBPClass spyClass = (JBPClass) iter.next();
            printLevel(out, 2);
            out.println("<memoryInstance className=\"" + spyClass.getName()
                    + "\">");
            Collection listMethods = (Collection) spyProcess
                    .findMemoryLeaks(spyClass);

            Iterator iterMethods = listMethods.iterator();

            while (iterMethods.hasNext()) {
                JBPMethodCount methodCount = (JBPMethodCount) iterMethods
                        .next();
                printMemoryLeakMethod(3, out, methodCount);
            }

            printLevel(out, 2);
            out.println("</memoryInstance>");

        }

        printLevel(out, 1);
        out.println("</memoryOperations>");

    }

    private static void closeMethod(int level, PrintStream out, JBPMethodCount count,
            JBPProcess process) {
        //JBPMethod methodOnProcess =
        // (JBPMethod)process.getSpyMethods().get(count.getSpyMethod());
        for (int i = 0; i < level; i++) {
            out.print(IDENT_ON_LEVEL);
        }
        out.println("</method>");
    }

}