/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler;

import java.io.IOException;

import org.jboss.aop.AspectManager;
import org.jboss.aop.advice.AdviceBinding;
import org.jboss.aop.pointcut.ast.ParseException;
import org.jboss.profiler.aop.profiler.ProfilerAgentInterceptor;
import org.jboss.profiler.aop.profiler.control.ProfilerController;

/**
 * @author Clebert Suconic
 */
public class AOPProfilerService implements AOPProfilerServiceMBean {

    /* (non-Javadoc)
     * @see org.jboss.profiler.AOPProfilerServiceMBean#active(java.lang.String)
     */
    public void activate(String directory) throws IOException {
        ProfilerController.getInstance().setOutputLocal(directory);
        ProfilerController.getInstance().setStarted(true);
    }

    /* (non-Javadoc)
     * @see org.jboss.profiler.AOPProfilerServiceMBean#addPackage(java.lang.String)
     */
    public void addPointcut(String pointcutExpression) throws ParseException {
        AdviceBinding binding = new AdviceBinding("all("+pointcutExpression+")", null);
        binding.addInterceptor(ProfilerAgentInterceptor.class);
        AspectManager.instance().addBinding(binding);
    }

    /* (non-Javadoc)
     * @see org.jboss.profiler.AOPProfilerServiceMBean#stop()
     */
    public void stop() throws IOException {
        ProfilerController.getInstance().setStarted(false);
    }

}
