/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jbossprofiler.aop.monitoring;

import java.util.ArrayList;
import java.util.Iterator;

import org.jboss.profiler.aop.monitoring.control.MonitoringController;
import org.jboss.profiler.aop.monitoring.model.MeasureAggregation;
import org.jboss.profiler.aop.monitoring.model.MonitoringMeasure;
import org.jboss.profiler.aop.monitoring.model.TimeValue;

import junit.framework.TestCase;

/**
 * @author Clebert Suconic
 */
public class MonitoringControlTester extends TestCase {
    public MonitoringControlTester(String name) {
        super(name);
    }

    public MonitoringControlTester() {
        super();
    }

    public void testSingleThreadSingleMeasureSingleTime() {
        MonitoringController.clear();
        long time = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            MonitoringController.openStack("Test","Test", time);

            MonitoringController.closeStack("Test","Test", time, 0, 0);
        }

        for (int i = 0; i < 1000; i++) {
            MonitoringController.openStack("Test2","Test2", time);

            MonitoringController.closeStack("Test2","Test2", time, 0, 0);
        }

        MonitoringMeasure measure = MonitoringController.findMeasureByName("Test");
        assertEquals(1, measure.getValues().size());

        TimeValue value = (TimeValue) measure.getValues().values().toArray()[0];
        assertEquals(1000, value.getCallings());
        assertEquals (2,MonitoringController.getAggregationsSet().size());

        MeasureAggregation aggregation = MonitoringController.findAggregation("Test");
        if (aggregation!=null) {
     	   Iterator measures = aggregation.getMeasureSet().iterator();

     	   while (measures.hasNext()) {
     	       measure = (MonitoringMeasure)measures.next();
     	       TimeValue rootValue = measure.getRootValue();
     	   }
        }

    }

    class Tester extends Thread {
        int iterations = 0;

        long time;

        public Tester(String name, int iterations, long time) {
            super(name);
            this.iterations = iterations;
            this.time = time;
        }

        public void run() {
            for (int i = 0; i < iterations; i++) {
                MonitoringController.openStack("Test Aggregation","Test", time);
                Thread.yield();
                MonitoringController.closeStack("Test Aggregation","Test", time, 0, 0);
            }

        }
    }

    public void testMultiThreadSingleMeasureSingleTime() {

        MonitoringController.clear();
        long time = System.currentTimeMillis();

        ArrayList list = new ArrayList();

        for (int i=0;i<15;i++) {
            Tester tst = new Tester("ThreadTester" + i, 1000,time);
            list.add(tst);
            tst.start();
        }

        Iterator iter = list.iterator();
        while (iter.hasNext()) {
            Tester tst = (Tester) iter.next();
            try {
                tst.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }


        MonitoringMeasure measure = MonitoringController.findMeasureByName("Test");
        assertEquals(1, measure.getValues().size());

        TimeValue value = (TimeValue) measure.getValues().values().toArray()[0];
        assertEquals(15000, value.getCallings());
        assertEquals (1,MonitoringController.getAggregationsSet().size());


    }

}