/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import org.jboss.profiler.model.*;
import org.jfree.data.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import java.awt.image.BufferedImage;


/**
 * Gera grafico de pizza em funcao da arvore de chamadas no thread/processo
 @author Clebert Suconic
 */
public class ChartView extends HttpServlet {
  private static final String CONTENT_TYPE = "image/jpeg";
  //Initialize global variables
  public void init() throws ServletException {
  }
  //Process the HTTP Get request
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    response.setContentType(CONTENT_TYPE);
    response.setHeader("pragma", "no-cache");

    Integer methodId = new Integer(request.getParameter("methodId"));
    JBPMethod keyMethod = new JBPMethod();
    keyMethod.setMethodID(methodId.intValue());

    javax.servlet.http.HttpSession session = request.getSession();

    org.jboss.profiler.web.form.FilterForm  beanFilter = (org.jboss.profiler.web.form.FilterForm)request.getSession().getAttribute("filterForm");

    org.jboss.profiler.model.JBPProcess spyProcess = beanFilter.getJbpProcess();

    Integer threadId = null;
    JBPThread thread = null;

    if (request.getParameter("threadId")!=null) {
      JBPThread threadKey = new JBPThread();
      threadId = new Integer(request.getParameter("threadId"));
      threadKey.setThreadId(threadId.intValue());
      thread=(JBPThread)spyProcess.getSpyThreads().get(threadKey);
    }

    HashMap spyMethods = spyProcess.getSpyMethods();
    JBPMethod spyMethodProcess = (JBPMethod)spyMethods.get(keyMethod);

    JBPMethodCount consolidacao = (JBPMethodCount)request.getSession().getAttribute("consolidacaoMetodo");
    if (thread==null && (consolidacao==null || consolidacao.getSpyMethod().equals(spyMethodProcess) || session.getAttribute("consolidacaoThread")!=null)) {
      consolidacao = spyProcess.consolidaCallings(spyMethodProcess);
      session.setAttribute("consolidacaoMetodo",consolidacao);
      session.removeAttribute("consolidacaoThread");
    } else {
      JBPThread consolidacaoThread = (JBPThread)session.getAttribute("consolidacaoThread");
      if (thread!=null && (consolidacao==null || consolidacaoThread==null || consolidacaoThread.getThreadId()!=threadId.intValue() || !consolidacao.getSpyMethod().equals(spyMethodProcess))) {
        consolidacao=thread.consolidaCallings(spyMethodProcess);
        session.setAttribute("consolidacaoMetodo",consolidacao);
        session.setAttribute("consolidacaoThread",thread);
      }
    }

    // qual instancia sera utilizada no calculo
    JBPMethod spyMethodToCalc = spyMethodProcess;
    if (thread!=null) {
      spyMethodToCalc=(JBPMethod)thread.getSpyMethods().get(spyMethodToCalc);
    }


    int numberOfPaths=0;
    if (request.getParameter("npath")!=null) {
      numberOfPaths=Integer.parseInt(request.getParameter("npath"));
    }

    int methodsId[] = new int[numberOfPaths];
    JBPMethod metodosPesquisados[] = new JBPMethod[numberOfPaths];
    JBPMethodCount[] consolidacoesPesquisadas = new JBPMethodCount[numberOfPaths];

    String urlBasesrc = "methodId="+methodId ; // url para seguir o caminho
    if (thread!=null) {
      urlBasesrc+="&threadId=" + threadId;
    }

    for (int i=0;i<numberOfPaths;i++) {
      methodsId[i] = Integer.parseInt(request.getParameter("m" + i));
      urlBasesrc+="&m"+i+"="+methodsId[i];
      keyMethod.setMethodID(methodsId[i]);
      metodosPesquisados[i]=(JBPMethod)spyProcess.getSpyMethods().get(keyMethod);
    }

    //String urlBase = "./methodStack.jsp?" + urlBasesrc +"&npath="+(numberOfPaths+1)+"&m"+numberOfPaths+"=";
    //String urlBaseObjects = "./methodStackObjects.jsp?" + urlBasesrc + "&npath="+(numberOfPaths);

    for (int i=0;i<numberOfPaths;i++) {
      consolidacoesPesquisadas[i]=consolidacao.getCalledCount(metodosPesquisados[i]);
      consolidacao = consolidacoesPesquisadas[i];
    }

    JBPMethod methodConsolidado = consolidacao.getSpyMethod();
    methodConsolidado = (JBPMethod)spyMethods.get(methodConsolidado);

    generateChart(response.getOutputStream(),spyProcess,spyMethodToCalc,consolidacao);
  }

  private void generateChart(OutputStream out, JBPProcess spyProcess, JBPMethod spyMethodToCalc,JBPMethodCount consolidacao)  throws IOException  {
      DefaultCategoryDataset datasetBar = new DefaultCategoryDataset();

      TreeSet setMethods = null;

      // para ordenar pelo maior tempo
      setMethods = new TreeSet(new Comparator() {
                 public int compare (Object o1, Object o2) {
                     JBPGenericCount s1 = (JBPGenericCount)o1;
                     JBPGenericCount s2 = (JBPGenericCount)o2;
                     long x = s2.getTotalClocks() - s1.getTotalClocks();
                     if (x!=0) {
                        if (x>0) return 1;
                        else return -1;
                     } else
                     if ( ((JBPMethod)s1.getReferencedObject()).getName()!=null &&
                          ((JBPMethod)s2.getReferencedObject()).getName()!=null) {
                         return ((JBPMethod)s1.getReferencedObject()).getName().compareTo(((JBPMethod)s2.getReferencedObject()).getName());
                     } else {
                         return ((JBPMethod)s1.getReferencedObject()).getMethodID() - ((JBPMethod)s2.getReferencedObject()).getMethodID();
                     }
                 }});

      setMethods.addAll(consolidacao.getCountCalleds().values());

      Iterator iterCount = setMethods.iterator();


      double totalCPU=0;
      double totalTime=0;
      double totalCPUOthers=0; // a partir de 90%, criaremos uma coluna com outros
      double totalTimeOthers=0; // a partir de 90%, criaremos uma coluna com outros

      double totalPercentCPU=0;
      double totalPercentTime=0;

      while (iterCount.hasNext()) {
          JBPGenericCount spyCount = (JBPGenericCount)iterCount.next();
          JBPMethod method = (JBPMethod)spyCount.getReferencedObject();

          JBPMethod methodProcess = (JBPMethod)spyProcess.getSpyMethods().get(method);

          double percentCPU = (((double)spyCount.getTotalClocks()/(double)spyMethodToCalc.getTotalClock())*(double)100);
          double percentTime = (((double)spyCount.getTotalElapsedClock()/(double)spyMethodToCalc.getTotalElapsedClock())*(double)100);


          if (totalCPU<90) {
 	      datasetBar.addValue(spyCount.getTotalClocks(), "CPU", methodProcess.getName());
          } else {
              totalCPUOthers+=spyCount.getTotalClocks();
          }

          if (totalTime<90) {
 	      datasetBar.addValue(spyCount.getTotalElapsedClock(), "Time", methodProcess.getName());
          } else {
              totalTimeOthers+=spyCount.getTotalElapsedClock();
          }

          totalCPU+=percentCPU;
          totalTime+=percentTime;

      }

      if (totalCPUOthers>0) {
 	      datasetBar.addValue(totalCPUOthers, "CPU", "Others");
      }

      if (totalTimeOthers>0) {
 	      datasetBar.addValue(totalTimeOthers, "Time", "Others");
      }

      JFreeChart chart = ChartFactory.createPieChart("",datasetBar, org.jfree.chart.plot.PiePlot.PER_ROW ,true, false, false);
      PiePlot piePlot = (PiePlot)chart.getPlot();
      piePlot.setSectionLabelType(PiePlot.PERCENT_LABELS);
      saveChart(out, chart);

  }

  private void saveChart(OutputStream out, JFreeChart chart) throws IOException {
          com.sun.image.codec.jpeg.JPEGImageEncoder encoder = com.sun.image.codec.jpeg.JPEGCodec.createJPEGEncoder(out);
          BufferedImage image = chart.createBufferedImage(800, 600);
          encoder.encode(image);
          out.flush();
  }


  //Clean up resources
  public void destroy() {
  }
}