/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.servlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.profiler.graphmodel.*;
import org.jboss.profiler.model.JBPClass;
import org.jboss.profiler.model.JBPMethod;
import org.jboss.profiler.model.JBPMethodCount;

/**
@author Clebert Suconic
*/
public class GraphMemoryServer extends HttpServlet {
	private static final String CONTENT_TYPE = "application/javaObject";
	//Initialize global variables
	public void init() throws ServletException {
	}
	//Process the HTTP Get request
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		try {
			response.setContentType(CONTENT_TYPE);
			response.setHeader("pragma", "no-cache");

			JBPMethod keyMethod = new JBPMethod();
			JBPClass keyClass = new JBPClass();

			javax.servlet.http.HttpSession session = request.getSession();

			org.jboss.profiler.web.form.FilterForm beanFilter =
				(org.jboss.profiler.web.form.FilterForm) request
					.getSession()
					.getAttribute("filterForm");

			org.jboss.profiler.model.JBPProcess spyProcess =
				beanFilter.getJbpProcess();

			int classId = Integer.parseInt(request.getParameter("classId"));
			keyClass.setClassID(classId);
			JBPClass spyClass =
				(JBPClass) spyProcess.getSpyClasses().get(keyClass);

			HashMap spyMethods = spyProcess.getSpyMethods();
			JBPMethod spyMethodProcess = (JBPMethod) spyMethods.get(keyMethod);

			Collection listMethods = null;
			if (request.getSession().getAttribute("instancesStack.class")
				== null
				|| !(
					(JBPClass) session.getAttribute(
						"instancesStack.class")).equals(
					spyClass)) {
				session.setAttribute("instancesStack.class", spyClass);
				listMethods = spyProcess.findMemoryLeaks(spyClass);
				session.setAttribute("listMethodsLeaks", listMethods);
			} else {
				listMethods =
					(Collection) session.getAttribute("listMethodsLeaks");
			}

			boolean viewCreation = false;
			if (request.getParameter("viewCreation") != null)
				viewCreation = true;

			int pathSize = 0;
			if (request.getParameter("pathSize") != null)
				pathSize = Integer.parseInt(request.getParameter("pathSize"));

			int methodsId[] = new int[pathSize];
			JBPMethod metodosPesquisados[] = new JBPMethod[pathSize];

			JBPMethodCount rootCount = null;
			JBPMethodCount consolidacao = null;

			String urlBasesrc = "classId=" + classId;
			for (int i = 0; i < pathSize; i++) {
				methodsId[i] = Integer.parseInt(request.getParameter("m" + i));
				metodosPesquisados[i] =
					(JBPMethod) spyProcess.findMethod(keyMethod, methodsId[i]);
				urlBasesrc += "&m" + i + "=" + methodsId[i];

				if (i == 0) {
					rootCount =
						findMethodOnConsolidations(listMethods, methodsId[i]);
					consolidacao = rootCount;
				} else {
					consolidacao =
						consolidacao.getCalledCount(metodosPesquisados[i]);
				}
			}

			GraphCallings graph =
				generateCallings(
					urlBasesrc,
					pathSize,
					classId,
					spyMethods,
					consolidacao,
					viewCreation);
			java.io.ObjectOutputStream outObj =
				new java.io.ObjectOutputStream(response.getOutputStream());
			outObj.writeObject(graph);
			outObj.flush();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private JBPMethodCount findMethodOnConsolidations(
		Collection listMethods,
		int methodId) {
		Iterator iter = listMethods.iterator();
		while (iter.hasNext()) {
			JBPMethodCount tmpCount = (JBPMethodCount) iter.next();
			if (tmpCount.getSpyMethod().getMethodID() == methodId) {
				return tmpCount;
			}
		}

		return null;
	}

	private GraphCallings generateCallings(
		String baseUrl,
		int pathSize,
		int classId,
		java.util.HashMap spyMethods,
		JBPMethodCount consolidacao,
		boolean viewCreation) {

		java.util.Stack stack = new java.util.Stack();

		HashMap hashMethods = new HashMap();
		HashMap hashCallings = new HashMap();
		GraphCallings graph = new GraphCallings();

		// para ajudar na referencia entre chamadas
		GraphReference referenceKey = new GraphReference();

		long oldMethod = 0;

		int level = 0;
		// root

		JBPMethod methodConsolidado = consolidacao.getSpyMethod();

		generateCallingsRecursive(
			baseUrl,
			pathSize,
			level,
			oldMethod,
			classId,
			consolidacao,
			hashMethods,
			hashCallings,
			referenceKey,
			stack,
			viewCreation);

		TreeSet treeSet = new TreeSet(new CompareMethods());
		treeSet.addAll(hashMethods.values());

		ArrayList listMethods = new ArrayList();
		listMethods.addAll(treeSet);
		ArrayList listRefs = new ArrayList();
		listRefs.addAll(hashCallings.values());
		graph.setMethods(listMethods);
		graph.setReferences(listRefs);

		graph.printDebug();

		return graph;
	}

	private void generateCallingsRecursive(
		String baseUrl,
		int pathSize,
		int level,
		long methodIdCallee,
		int classId,
		JBPMethodCount consolidacao,
		HashMap hashMethods,
		HashMap hashCallings,
		GraphReference referenceKey,
		java.util.Stack path,
		boolean viewCreation) {

		GraphMethod key = new GraphMethod();
		Iterator iterValues =
			consolidacao.getCountCalleds().values().iterator();
		while (iterValues.hasNext()) {
			JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();

			key.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
			key.setLevel(level + 1);
			if (hashMethods.get(key) == null) {
				GraphMethod newMethod =
					new GraphMethod(
						subConsolidacao.getSpyMethod(),
						level+1,
						createURL(
							pathSize,
							baseUrl,
							classId,
							path,
							subConsolidacao.getSpyMethod(),
							viewCreation));
				hashMethods.put(newMethod, newMethod);
			}
		}

		iterValues = consolidacao.getCountCalleds().values().iterator();
		while (iterValues.hasNext()) {
			JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();
			int countInUse =
				viewCreation
					? subConsolidacao.getCount()
					: subConsolidacao.getActiveCount();

			if (viewCreation && countInUse == 0)
				continue;

			key.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
			key.setLevel(level + 1);
			GraphMethod graphMethod = (GraphMethod) hashMethods.get(key);

			if (graphMethod == null) {
				System.out.println("Sorry but I didn't find MethodId=" + key.getMethodId() + " level=" + key.getLevel());
				throw new RuntimeException("Unexpected null method");
			}

			graphMethod.setWeight(graphMethod.getWeight() + countInUse);

			if (methodIdCallee != 0) {
				referenceKey.setMethodCallee(methodIdCallee);
				referenceKey.setLevelCallee(level - 1);
				referenceKey.setMethodCalled(
					subConsolidacao.getSpyMethod().getMethodID());
				referenceKey.setLevelCalled(level);
				if (hashCallings.get(referenceKey) == null) {
					GraphReference ref =
						new GraphReference(
							level,
							methodIdCallee,
							level + 1,
							subConsolidacao.getSpyMethod().getMethodID(),
							countInUse);
					hashCallings.put(ref, ref);
				}
			}

			path.push(subConsolidacao.getSpyMethod());

			generateCallingsRecursive(
				baseUrl,
				pathSize,
				level + 1,
				subConsolidacao.getSpyMethod().getMethodID(),
				classId,
				subConsolidacao,
				hashMethods,
				hashCallings,
				referenceKey,
				path,
				viewCreation);
			path.pop();

		}

	}

	private String createURL(
		int basicPathLength,
		String basicURL,
		int classId,
		java.util.Stack path,
		JBPMethod currentMethod,
		boolean viewCreation) {

		StringBuffer url = new StringBuffer();
		url.append("?" + basicURL);
		//url.append("&classId=" + classId);
		int pathLength = basicPathLength;
		if (path != null)
			pathLength = path.size();

		for (int i = 0; i < pathLength; i++) {
			JBPMethod spyMethod = (JBPMethod) path.get(i);
			url.append(
				"&m" + (i + basicPathLength) + "=" + spyMethod.getMethodID());
		}

		if (currentMethod != null) {
			pathLength += 1;
			url.append(
				"&m"
					+ (pathLength - 1 + basicPathLength)
					+ "="
					+ currentMethod.getMethodID());
		}

		url.append("&pathSize=" + (pathLength + basicPathLength));
		if (viewCreation)
			url.append("&viewCreation=true");
		return url.toString();
	}

	//Clean up resources
	public void destroy() {
	}

	/** Comparator class that will sort GraphMethod TreeSet */
	static class CompareMethods implements java.util.Comparator {
		public int compare(Object left, Object right) {
			int iret =
				((GraphMethod) left).getLevel()
					- ((GraphMethod) right).getLevel();
			if (iret != 0)
				return iret;
			double fret =
				((GraphMethod) left).getWeight()
					- ((GraphMethod) right).getWeight();

			if (fret > 0)
				return -1;
			else if (fret < 0)
				return 1;
			else
				return ((GraphMethod) left).toString().compareTo(
					((GraphMethod) right).toString());
		}
	}

}
