/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.servlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeSet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.profiler.graphmodel.GraphCallings;
import org.jboss.profiler.graphmodel.GraphMethod;
import org.jboss.profiler.graphmodel.GraphReference;
import org.jboss.profiler.model.JBPMethod;
import org.jboss.profiler.model.JBPMethodCount;
import org.jboss.profiler.model.JBPThread;

/**
 * Gera grafico de pizza em funcao da arvore de chamadas no thread/processo
 *
 * @author Clebert Suconic
 */
public class GraphServer extends HttpServlet {
    private static final String CONTENT_TYPE = "application/javaObject";

    private static final boolean printOut = true;

    //Initialize global variables
    public void init() throws ServletException {
    }

    //Process the HTTP Get request
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            response.setContentType(CONTENT_TYPE);
            response.setHeader("pragma", "no-cache");

            Integer methodId = new Integer(request.getParameter("methodId"));
            JBPMethod keyMethod = new JBPMethod();
            keyMethod.setMethodID(methodId.intValue());

            javax.servlet.http.HttpSession session = request.getSession();

            org.jboss.profiler.web.form.FilterForm beanFilter = (org.jboss.profiler.web.form.FilterForm) request
                    .getSession().getAttribute("filterForm");

            org.jboss.profiler.model.JBPProcess spyProcess = beanFilter
                    .getJbpProcess();

            Integer threadId = null;
            JBPThread thread = null;

            if (request.getParameter("threadId") != null) {
                JBPThread threadKey = new JBPThread();
                threadId = new Integer(request.getParameter("threadId"));
                threadKey.setThreadId(threadId.intValue());
                thread = (JBPThread) spyProcess.getSpyThreads().get(threadKey);
            }

            HashMap spyMethods = spyProcess.getSpyMethods();
            JBPMethod spyMethodProcess = (JBPMethod) spyMethods.get(keyMethod);

            JBPMethodCount consolidacao = (JBPMethodCount) request.getSession()
                    .getAttribute("consolidacaoMetodo");
            if (thread == null
                    && (consolidacao == null
                            || consolidacao.getSpyMethod().equals(
                                    spyMethodProcess) || session
                            .getAttribute("consolidacaoThread") != null)) {
                consolidacao = spyProcess.consolidaCallings(spyMethodProcess);
                session.setAttribute("consolidacaoMetodo", consolidacao);
                session.removeAttribute("consolidacaoThread");
            } else {
                JBPThread consolidacaoThread = (JBPThread) session
                        .getAttribute("consolidacaoThread");
                if (thread != null
                        && (consolidacao == null
                                || consolidacaoThread == null
                                || consolidacaoThread.getThreadId() != threadId
                                        .intValue() || !consolidacao
                                .getSpyMethod().equals(spyMethodProcess))) {
                    consolidacao = thread.consolidaCallings(spyMethodProcess);
                    session.setAttribute("consolidacaoMetodo", consolidacao);
                    session.setAttribute("consolidacaoThread", thread);
                }
            }

            // qual instancia sera utilizada no calculo
            JBPMethod spyMethodToCalc = spyMethodProcess;
            if (thread != null) {
                spyMethodToCalc = (JBPMethod) thread.getSpyMethods().get(
                        spyMethodToCalc);
            }

            int numberOfPaths = 0;
            if (request.getParameter("npath") != null) {
                numberOfPaths = Integer.parseInt(request.getParameter("npath"));
            }

            int methodsId[] = new int[numberOfPaths];
            JBPMethod metodosPesquisados[] = new JBPMethod[numberOfPaths];

            String urlBasesrc = "methodId=" + methodId; // url para seguir o
                                                        // caminho
            if (thread != null) {
                urlBasesrc += "&threadId=" + threadId;
            }

            for (int i = 0; i < numberOfPaths; i++) {
                methodsId[i] = Integer.parseInt(request.getParameter("m" + i));
                // retirar
                urlBasesrc += "&m" + i + "=" + methodsId[i];
                keyMethod.setMethodID(methodsId[i]);
                metodosPesquisados[i] = (JBPMethod) spyProcess.getSpyMethods()
                        .get(keyMethod);
            }

            GraphCallings graph = generateCallings(spyMethods, consolidacao,
                    spyMethodToCalc, metodosPesquisados);

            java.io.ObjectOutputStream outObj = new java.io.ObjectOutputStream(
                    response.getOutputStream());
            outObj.writeObject(graph);
            outObj.flush();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String createURL(JBPMethod root, java.util.Stack path,
            JBPMethod currentMethod) {
        StringBuffer url = new StringBuffer();
        url.append("?methodId=" + root.getMethodID());
        int pathLength = 0;
        if (path != null)
            pathLength = path.size();
        for (int i = 0; i < pathLength; i++) {
            JBPMethod spyMethod = (JBPMethod) path.get(i);
            url.append("&m" + i + "=" + spyMethod.getMethodID());
        }
        if (currentMethod != null) {
            pathLength += 1;
            url.append("&m" + (pathLength - 1) + "="
                    + currentMethod.getMethodID());
        }
        url.append("&npath=" + pathLength);
        return url.toString();
    }

    private GraphCallings generateCallings(java.util.HashMap spyMethods,
            JBPMethodCount consolidacao, JBPMethod rootMethod, JBPMethod[] path) {

        java.util.Stack stack = new java.util.Stack();

        HashMap hashMethods = new HashMap();
        HashMap hashCallings = new HashMap();
        GraphCallings graph = new GraphCallings();

        // para ajudar na referencia entre chamadas
        GraphReference referenceKey = new GraphReference();

        int numberOfPaths = path.length;

        long oldMethod = 0;

        int level = 0;
        // root
        if (numberOfPaths == 0) {
            oldMethod = consolidacao.getSpyMethod().getMethodID();
            GraphMethod newMethod = new GraphMethod((JBPMethod) spyMethods
                    .get(consolidacao.getSpyMethod()), level++, createURL(
                    rootMethod, null, null));
            hashMethods.put(newMethod, newMethod);
        }

        GraphMethod methodKey = new GraphMethod();

        for (int i = 0; i < numberOfPaths; i++) {
            consolidacao = consolidacao.getCalledCount(path[i]);

            if (i >= (numberOfPaths - 1)) {
                methodKey.setMethodId(path[i].getMethodID());
                methodKey.setLevel(level++);

                //Long methodKey = new Long(path[i].getMethodID());
                GraphMethod calledGraphMethod = (GraphMethod) hashMethods
                        .get(methodKey);

                if (calledGraphMethod == null) {
                    calledGraphMethod = new GraphMethod((JBPMethod) spyMethods
                            .get(path[i]), level++, createURL(rootMethod,
                            stack, path[i]));
                    hashMethods.put(calledGraphMethod, calledGraphMethod);
                }

                if (oldMethod != 0) {
                    referenceKey.setMethodCallee(oldMethod);
                    referenceKey.setMethodCalled(path[i].getMethodID());
                    if (hashCallings.get(referenceKey) == null) {
                        double percenptual = (double) consolidacao
                                .getTotalElapsedClock()
                                / (double) rootMethod.getTotalElapsedClock()
                                * (double) 100;
                        GraphReference ref = new GraphReference(level - 1,
                                oldMethod, level, path[i].getMethodID(),
                                percenptual, (double) consolidacao
                                        .getTotalClocks()
                                        / (double) rootMethod.getTotalClock()
                                        * (double) 100);
                        hashCallings.put(ref, ref);
                    }
                }

                oldMethod = path[i].getMethodID();
            }

            stack.push(path[i]);
        }

        JBPMethod methodConsolidado = consolidacao.getSpyMethod();
        methodConsolidado = (JBPMethod) spyMethods.get(methodConsolidado);

        generateCallingsRecursive(level, oldMethod, rootMethod, consolidacao,
                hashMethods, hashCallings, referenceKey, stack);

        TreeSet treeSet = new TreeSet(new CompareMethods());
        treeSet.addAll(hashMethods.values());

        ArrayList listMethods = new ArrayList();
        listMethods.addAll(treeSet);
        ArrayList listRefs = new ArrayList();
        listRefs.addAll(hashCallings.values());
        graph.setMethods(listMethods);
        graph.setReferences(listRefs);

        graph.printDebug();

        return graph;
    }

    private void generateCallingsRecursive(int level, long methodIdCallee,
            JBPMethod rootMethod, JBPMethodCount consolidacao,
            HashMap hashMethods, HashMap hashCallings,
            GraphReference referenceKey, java.util.Stack path) {

        GraphMethod keyMethod = new GraphMethod();
        Iterator iterValues = consolidacao.getCountCalleds().values()
                .iterator();
        while (iterValues.hasNext()) {
            JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();

            double percenptual = ((double) subConsolidacao
                    .getTotalElapsedClock() / (double) rootMethod
                    .getTotalElapsedClock())
                    * (double) 100;
            if (percenptual < 1.00)
                continue;

            keyMethod.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
            keyMethod.setLevel(level);

            if (hashMethods.get(keyMethod) == null) {
                GraphMethod newMethod = new GraphMethod(subConsolidacao
                        .getSpyMethod(), level, createURL(rootMethod, path,
                        subConsolidacao.getSpyMethod()));
                hashMethods.put(newMethod, newMethod);
            }
        }

        iterValues = consolidacao.getCountCalleds().values().iterator();
        while (iterValues.hasNext()) {
            JBPMethodCount subConsolidacao = (JBPMethodCount) iterValues.next();

            keyMethod.setMethodId(subConsolidacao.getSpyMethod().getMethodID());
            keyMethod.setLevel(level);
            GraphMethod graphMethod = (GraphMethod) hashMethods.get(keyMethod);

            double percenptual = ((double) subConsolidacao
                    .getTotalElapsedClock() / (double) rootMethod
                    .getTotalElapsedClock())
                    * (double) 100;
            if (percenptual < 1.00)
                continue;

            if (graphMethod == null) {
                throw new RuntimeException("Unexpected null method");
            }

            referenceKey.setMethodCallee(methodIdCallee);
            referenceKey.setLevelCallee(level - 1);
            referenceKey.setLevelCalled(level);
            referenceKey.setMethodCalled(subConsolidacao.getSpyMethod()
                    .getMethodID());
            if (hashCallings.get(referenceKey) == null) {
                GraphReference ref = new GraphReference(level - 1,
                        methodIdCallee, level, subConsolidacao.getSpyMethod()
                                .getMethodID(), ((double) subConsolidacao
                                .getTotalElapsedClock() / (double) rootMethod
                                .getTotalElapsedClock())
                                * (double) 100, (double) subConsolidacao
                                .getTotalClocks()
                                / (double) rootMethod.getTotalClock()
                                * (double) 100);
                graphMethod.setWeight(graphMethod.getWeight() + percenptual);
                hashCallings.put(ref, ref);
            }

            path.push(subConsolidacao.getSpyMethod());
            generateCallingsRecursive(level + 1, subConsolidacao.getSpyMethod()
                    .getMethodID(), rootMethod, subConsolidacao, hashMethods,
                    hashCallings, referenceKey, path);
            path.pop();

        }

    }

    //Clean up resources
    public void destroy() {
    }

    /** Comparator class that will sort GraphMethod TreeSet */
    class CompareMethods implements java.util.Comparator {
        public int compare(Object left, Object right) {
            int iret = ((GraphMethod) left).getLevel()
                    - ((GraphMethod) right).getLevel();
            if (iret != 0)
                return iret;
            double fret = ((GraphMethod) left).getWeight()
                    - ((GraphMethod) right).getWeight();

            if (fret > 0)
                return -1;
            else if (fret < 0)
                return 1;
            else
                return ((GraphMethod) left).toString().compareTo(
                        ((GraphMethod) right).toString());
        }
    }

}