/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import org.jboss.profiler.util.SPYMethodCountComparator;
import org.jboss.profiler.web.form.*;
import org.jboss.profiler.model.*;
import org.apache.poi.hssf.usermodel.*;

/** Para poder enviar e receber contadores de metodos privados
@author Clebert Suconic
*/
class Contador {
   short row;
}

/**
@author Clebert Suconic
*/
public class ServletExcel extends HttpServlet {
  public static int requestCount=0;
  private static final String CONTENT_TYPE = "application/vnd.ms-excel";;
  //Initialize global variables
  public void init() throws ServletException {
  }
  //Process the HTTP Get request
  public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
     doGet(request,response);
  }

  //Process the HTTP Get request
  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

    int maxLevel = 0;

    if (request.getParameter("maxLevel")!=null) {
       maxLevel = Integer.parseInt(request.getParameter("maxLevel"));
    }

    double minPercent = 0;
    if (request.getParameter("minPercent")!=null) {
       minPercent = Double.parseDouble(request.getParameter("minPercent"));
    }


    FilterForm form = (FilterForm) request.getSession(false).getAttribute("filterForm");
    HashMap roots = form.getJbpProcess().consolidateRoots();

    response.setContentType(CONTENT_TYPE);
    HSSFWorkbook workbook = new HSSFWorkbook();
    HSSFSheet sheet = workbook.createSheet("Objects");

    HSSFRow rowTitle =  sheet.createRow((short)0);
    HSSFCell cell = rowTitle.createCell((short)0);
    cell.setCellValue("Process runned between " +form.getJbpProcess().getDate().toString() + " and " + form.getJbpProcess().getFinishDate().toString());

    HSSFCellStyle style = workbook.createCellStyle();
    style.setDataFormat((short)2);


    rowTitle = sheet.createRow((short)1);
    rowTitle.createCell((short)0).setCellValue("Total Clocks (nanos)");
    rowTitle.createCell((short)1).setCellValue(form.getJbpProcess().getTotalClocks());
    rowTitle.createCell((short)2).setCellValue("Total Time (ms)");
    rowTitle.createCell((short)3).setCellValue(form.getJbpProcess().getTotalElapsedClocks());
    rowTitle.getCell((short)1).setCellStyle(style);
    rowTitle.getCell((short)3).setCellStyle(style);

    rowTitle = sheet.createRow((short)3);
    rowTitle.createCell((short)0).setCellValue("Short Name");
    rowTitle.createCell((short)1).setCellValue("% CPU");
    rowTitle.createCell((short)2).setCellValue("% Time");
    rowTitle.createCell((short)3).setCellValue("Complete Name");

    Contador x = new Contador();
    x.row=4;

    TreeSet setRoots = new TreeSet(new SPYMethodCountComparator());
    setRoots.addAll(roots.values());
    Iterator rootsIterator = setRoots.iterator();
    while (rootsIterator.hasNext()) {
       JBPMethodCount count = (JBPMethodCount)rootsIterator.next();
       writeCount(workbook, style,minPercent, maxLevel,0,x,count,sheet,form.getJbpProcess(),null);
    }

    OutputStream out = new BufferedOutputStream(response.getOutputStream());
    workbook.write(out);
    out.close();
  }

  private void writeCount(HSSFWorkbook workbook, HSSFCellStyle style, double minPercent, int maxLevel,int level, Contador x, JBPMethodCount count, HSSFSheet sheet, JBPProcess process, JBPMethodCount rootMethod) {
     if (maxLevel>0 && level>=maxLevel) return;
     double percent = 0;
     if (rootMethod==null) percent = ((double)count.getTotalElapsedClock()/(double)process.getTotalElapsedClocks())*(double)100;
     else percent=((double)count.getTotalElapsedClock()/(double)rootMethod.getTotalElapsedClock())*(double)100;

     double percentClocks=0;
     if (rootMethod==null) percentClocks = ((double)count.getTotalClocks()/(double)process.getTotalClocks())*(double)100;
     else percentClocks=((double)count.getTotalClocks()/(double)rootMethod.getTotalClocks())*(double)100;

     if (percent<minPercent && percentClocks<minPercent) return;


     StringBuffer buffer = new StringBuffer();
     for (int i=0;i<level;i++) {
        buffer.append(' ');
     }

     JBPMethod methodOnProcess = (JBPMethod)process.getSpyMethods().get(count.getSpyMethod());
     HSSFRow row = sheet.createRow(x.row++);
     //row.createCell((short)0).setCellValue(levelBuffer.toString());

     row.createCell((short)0).setCellValue(buffer.toString() + methodOnProcess.getName());
     row.createCell((short)1).setCellValue(percentClocks);
     row.createCell((short)2).setCellValue(percent);
     row.createCell((short)3).setCellValue(methodOnProcess.getFullName());

     for (short i=0;i<4;i++) {
        row.getCell(i).setCellStyle(style);
     }


    TreeSet setMethods = new TreeSet(new SPYMethodCountComparator());
    setMethods.addAll(count.getCountCalleds().values());

     Iterator subMethods = setMethods.iterator();

     while (subMethods.hasNext()) {
        JBPMethodCount rootToBeSent = rootMethod;
        JBPMethodCount subCount = (JBPMethodCount)subMethods.next();
        if (rootToBeSent==null) rootToBeSent = count;
        writeCount(workbook, style, minPercent,maxLevel,level+1,x,subCount,sheet,process,rootToBeSent);
     }
  }

  //Clean up resources
  public void destroy() {
  }
}