/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.servlets;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.profiler.model.JBPProcess;
import org.jboss.profiler.util.XmlGenerationUtil;
import org.jboss.profiler.web.form.FilterForm;

/** Creates a consolidated view with all method Callings
@author Clebert Suconic
*/
public class ServletXML extends HttpServlet {

	/** Number of spaces on each level */

	public static int requestCount = 0;
	private static final String CONTENT_TYPE = "text/xml";
	private static final String ZIP_CONTENT_TYPE =
		"application/x-zip-compressed";
	//Initialize global variables
	public void init() throws ServletException {
	}
	//Process the HTTP Get request
	public void doPost(
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException, IOException {
		doGet(request, response);
	}

	//Process the HTTP Get request
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		try {
			java.text.NumberFormat numberFormat = new java.text.DecimalFormat();

			int maxLevel = 0;

			if (request.getParameter("maxLevel") != null) {
				maxLevel = Integer.parseInt(request.getParameter("maxLevel"));
			}

			double minPercent = 0;
			if (request.getParameter("minPercent") != null) {
				minPercent =
					Double.parseDouble(request.getParameter("minPercent"));
			}

			boolean isZip = false;
			if (request.getParameter("isZip") != null
				&& request.getParameter("isZip").equals("yes")) {
				isZip = true;
			}

			boolean printMemory = false;
			if (request.getParameter("printMemory") != null) {
				printMemory = true;
			}

			boolean memoryLeaks = false;
			if (request.getParameter("memoryLeaks") != null) {
				memoryLeaks = true;
			}

			ZipOutputStream zipOut = null;
			ZipEntry entry = null;
			PrintStream out = null;

			if (isZip) {
				zipOut = new ZipOutputStream(response.getOutputStream());
				entry = new ZipEntry("callings.xml");
				zipOut.putNextEntry(entry);
				out = new PrintStream(zipOut);
				response.setContentType(ZIP_CONTENT_TYPE);
			} else {
				out =
					new PrintStream(
						new BufferedOutputStream(response.getOutputStream()));
				response.setContentType(CONTENT_TYPE);
			}

			FilterForm form =
				(FilterForm) request.getSession(false).getAttribute(
					"filterForm");

			JBPProcess process = form.getJbpProcess();

			XmlGenerationUtil.generateXML(out, process, minPercent, maxLevel, printMemory, memoryLeaks, numberFormat);

			if (isZip) {
				out.flush();
				zipOut.closeEntry();
				zipOut.close();
			} else {
				out.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	//Clean up resources
	public void destroy() {
	}
}