/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.profiler.web.tabhelp;

import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

/**
 * Gera uma tabela, baseando-se em uma colecao, utilizando
@author Clebert Suconic
*/
public class SimpleTab{

  static class ColumnConfig {
     ColumnConfig(String columnName, String propertyName, boolean useCustomHtmlRenderer) {
        this.columnName=columnName;
        this.propertyName=propertyName;
        this.useCustomHtmlRenderer=useCustomHtmlRenderer;
     }
     String columnName;
     String propertyName;
     boolean useCustomHtmlRenderer;
     java.beans.PropertyDescriptor property = null;

  }

  ArrayList columnConfigs = new ArrayList();

  public SimpleTab() {
  }

  private transient CustomEditListener customEditListener;
  public void setCustomEditListener(CustomEditListener l) {
     this.customEditListener = l;
  }

  protected String fireRenderColumn(String propertyName, Object data) {
    if (customEditListener != null) {
       return customEditListener.renderColumn(propertyName,data);
    } else {
       return null;
    }
  }

  /** Configure how the data will be sent to the HTML output.
   *  @param columnName Name of the column
   *  @param propertyName Name of the property on the collection's attribute
   *  @param useCustomHtmlRenderer is it necessary to use the event system (renderColumn) prior to render the information on the tab*/
  public void addColumn(String columnName, String propertyName, boolean useCustomHtmlRenderer) {
     columnConfigs.add(new ColumnConfig (columnName, propertyName, useCustomHtmlRenderer));
  }

  public void doOutput(HttpServletRequest request, PrintStream out, Collection collection, int startRow, int endRow) throws IOException {
    try {
     ColumnConfig configs[] = new ColumnConfig[columnConfigs.size()];
     int i=0;
     Iterator iter=columnConfigs.iterator();
     while (iter.hasNext()) {
       configs[i++]=(ColumnConfig)iter.next();
     }

     out.println("<table>");

     out.println("<tr>");
     for (i=0;i<configs.length;i++) {
        out.print("<td>" + configs[i].propertyName + "</td>");
     }
     out.println("</tr>");

     iter = collection.iterator();

     Object[] emptyArgs = new Object[0];

     java.beans.BeanInfo info = null;
     while (iter.hasNext()) {
       Object value = iter.next();
       if (info==null) {
          info = java.beans.Introspector.getBeanInfo(value.getClass());
          java.beans.PropertyDescriptor descriptors[] = info.getPropertyDescriptors();
          for (i=0;i<configs.length;i++) {
               for (int j=0;j<descriptors.length;j++) {
                  if (configs[i].propertyName.equals(descriptors[j].getName())) {
                     configs[i].property = descriptors[j];
                  }
               }
          }
       }
       out.println("<tr>");
       for (i=0;i<configs.length;i++) {
          out.println("<td>");
          if (configs[i].property==null) {
             out.println("&nbsp;");
          } else {
             Object propValue = configs[i].property.getReadMethod().invoke(value,emptyArgs);
             String rendered = null;
             if (configs[i].useCustomHtmlRenderer) {
                rendered = fireRenderColumn(configs[i].propertyName,propValue);
             }
             if (rendered==null) {
                rendered=propValue.toString();
             }
             out.println(rendered);
          }
          out.println("</td>");
       }
       out.println("</tr>");
     }

     out.println("</table>");


    }
    catch (java.beans.IntrospectionException e) {
      e.printStackTrace();
      throw new IOException ("Configuration error on parameters of SimpleTab utility");
    }
    catch (java.lang.IllegalAccessException e) {
      e.printStackTrace();
      throw new IOException ("Configuration error on parameters of SimpleTab utility");
    }
    catch (java.lang.reflect.InvocationTargetException e) {
      e.printStackTrace();
      throw new IOException ("Configuration error on parameters of SimpleTab utility");
    }

  }

}
